#!/usr/bin/env python
# Copyright 2016 Cumulus Networks, Inc
#
"""
mlx - utility for accessing the Mellanox state

Written by: Ken Yin
"""

import os
import sys
import argparse
import getopt
import json
from pprint import pprint
from collections import OrderedDict

# Add the path of the SDK
sys.path.append("/usr/lib/python2.7/site-packages/python_sdk_api")
sys.path.append("/usr/lib/python2.7/dist-packages/python_sdk_api")
from sx_api import *
import sxd_api

router_action = { SX_ROUTER_ACTION_DROP   : 'DROP',
                  SX_ROUTER_ACTION_TRAP   : 'TRAP',
                  SX_ROUTER_ACTION_FORWARD: 'FORWARD',
                  SX_ROUTER_ACTION_MIRROR : 'MIRROR',
                  SX_ROUTER_ACTION_SPAN   : 'SPAN'}

fdb_action = { 0: 'FWD',
               1: 'TRAP',
               2: 'CMIRROR',
               3: 'FWD_TO_RTR',
               4: 'DISCARD',
               5: 'INVALID',
               6: 'MIN',
               7: 'MAX' }

router_type = { SX_UC_ROUTE_TYPE_NEXT_HOP: 'NEXT_HOP',
                SX_UC_ROUTE_TYPE_LOCAL   : 'LOCAL',
                SX_UC_ROUTE_TYPE_IP2ME   : 'IP2ME'}

ecmp_type = { SX_ECMP_TYPE_STATIC_E    : 'STATIC',
              SX_ECMP_TYPE_RESILIENT_E : 'RESILIENT',
              SX_ECMP_TYPE_CONSISTENT_E: 'CONSISTENT',
              SX_ECMP_TYPE_DYNAMIC_E   : 'DYNAMIC'}

container_type = { SX_ECMP_CONTAINER_TYPE_IP  : 'IP',
                   SX_ECMP_CONTAINER_TYPE_MPLS: 'MPLS'}

fdb_entry_type = { 0: 'UC_STATIC',
                   1: 'UC_REMOTE',
                   2: 'UC_NONAGEABLE',
                   3: 'UC_AGEABLE',
                   4: 'UC_ALL',
                   5: 'UC_AGED',
                   6: 'UC_NONAGED',
                   7: 'POLICY_INVALID',
                   8: 'MIN',
                   9: 'MAX' }

shared_pool_mode_dict = {0:"STATIC" ,
                         1:"DYNAMIC"}

shared_pool_direction_dict = {0 : "INGRESS" ,
                              1 : "EGRESS"}

port_buffer_type_dict = {0 : "INGRESS_PORT",
                         1 : "INGRESS_PORT_PRIORITY_GROUP",
                         2 : "EGRESS_PORT",
                         3 : "EGRESS_PORT_TRAFFIC_CLASS",
                         4 : "MULTICAST",
                         5 : "MULTICAST_PORT" }

pool_label = { 0 : 'Ingress',
               1 : 'Ingress',
               2 : 'Ingress',
               3 : 'Ingress',
               4 : 'Egress/MC',
               5 : 'Egress',
               6 : 'Egress',
               7 : 'Egress/CPU',
               8 : '8??',
               9 : '9??',
               10: '10??' }

pfc_priority_list = [SX_PORT_FLOW_CTRL_PRIO_0,
                     SX_PORT_FLOW_CTRL_PRIO_1,
                     SX_PORT_FLOW_CTRL_PRIO_2,
                     SX_PORT_FLOW_CTRL_PRIO_3,
                     SX_PORT_FLOW_CTRL_PRIO_4,
                     SX_PORT_FLOW_CTRL_PRIO_5,
                     SX_PORT_FLOW_CTRL_PRIO_6,
                     SX_PORT_FLOW_CTRL_PRIO_7]

switch_priority_count = 14

priority_value_list = [ SX_PORT_PRIO_ID_0,
                        SX_PORT_PRIO_ID_1,
                        SX_PORT_PRIO_ID_2,
                        SX_PORT_PRIO_ID_3,
                        SX_PORT_PRIO_ID_4,
                        SX_PORT_PRIO_ID_5,
                        SX_PORT_PRIO_ID_6,
                        SX_PORT_PRIO_ID_7 ]

# Counter Group Types
valid_group_types = [ 63 ]        # GRP_ALL
valid_group_types += range(0,7)   # GRP_IEEE_802_DOT_3 to GRP_DISCARD
valid_group_types += range(16,26) # GRP_PER_PRIO to GRP_TC_7
valid_group_types += range(33,41) # GRP_PER_PRIO_0 to GRP_PER_PRIO_7

def hex_2_ip(ip_addr, ip_prefix):

    if ip_prefix :
        version = ip_prefix.version
        if version == SX_IP_VERSION_IPV4 :
            addr = ip_prefix.prefix.ipv4.addr
        else :
            addr = ip_prefix.prefix.ipv6.addr
    else :
        version = ip_addr.version
        if version == SX_IP_VERSION_IPV4 :
            addr = ip_addr.addr.ipv4
        else :
            addr = ip_addr.addr.ipv6

    if version == SX_IP_VERSION_IPV4 :
        return hex_2_ipv4(addr)
    elif version == SX_IP_VERSION_IPV6 :
        return hex_2_ipv6(addr)

def hex_2_ipv4(ipv4_addr):
    hex_addr = ipv4_addr.s_addr
    if isinstance(hex_addr, str):
        if len(hex_addr) < 8:
            hex_addr = "%08x" % int(hex_addr, 16)
        elif len(hex_addr) > 8:
            assert False, "Invalid format for IPv4 address"
    elif isinstance(hex_addr, int):
        hex_addr = "%08x" % hex_addr

    addr = ""
    for i in range(4):
        element = hex_addr[(i*2): (2*(i+1))]
        addr += "%d." % int(element, 16)

    return addr[:-1]

def hex_2_ipv6(ipv6_addr):
    addr = ''
    byte_count = 0;
    for i in range(4):
        for j in reversed(range(4)) :
            if not byte_count & 1 :
                nibble = ''
            byte_index = ((i*4) + j)
            byte = sx_cos_ecn_t_arr_getitem(ipv6_addr.__in6_u.__u6_addr8, byte_index)
            nibble += '{:02x}'.format(byte)
            if byte_count & 1 :
                addr += '%s:' % nibble
            byte_count += 1
    return addr[:-1]

def count_mask_bits(shift_value,length) :
    count = 0
    for j in range(length) :
        check = shift_value & 1
        if check:
            count += 1
        shift_value = shift_value >> 1
    return count

def get_mask_length(ip_prefix):
    count = 0

    if ip_prefix.version == SX_IP_VERSION_IPV6:
        mask = ip_prefix.prefix.ipv6.mask
        for i in range(16):
            shift_value = sx_cos_ecn_t_arr_getitem(mask.__in6_u.__u6_addr8, i)
            count += count_mask_bits(shift_value, 8)
    else:
        mask = ip_prefix.prefix.ipv4.mask.s_addr
        shift_value = mask
        count += count_mask_bits(shift_value, 32)

    return count

def port_type_id_get(log_port):
    return (log_port & SX_PORT_TYPE_ID_MASK) >> SX_PORT_TYPE_ID_OFFS

def port_phy_id_get(log_port):
    return (log_port & SX_PORT_PHY_ID_MASK) >> SX_PORT_PHY_ID_OFFS

def port_is_vport(log_port):
    return (port_type_id_get(log_port) & SX_PORT_TYPE_VPORT) != 0

def port_is_nve(log_port):
    return (port_type_id_get(log_port) & SX_PORT_TYPE_NVE) != 0

def port_vlan_id_get(log_port):
    if port_is_vport(log_port):
        return (log_port & SX_PORT_VLAN_ID_MASK) >> SX_PORT_VLAN_ID_OFFS
    return 0

def router_action_is_forward(action):
    return action != router_action[SX_ROUTER_ACTION_DROP] and \
           action != router_action[SX_ROUTER_ACTION_TRAP]

class mlxSdk(object):
    def __init__(self, handle):
        self.handle = handle
        self.vrid_list = []

    def get_vrids_in_use(self, update = False):
        ifc_attr_p = new_sx_router_attributes_t_p()

        if not self.vrid_list or update:
            for i in range(254): # max id is 253
                rc = sx_api_router_get(self.handle, i, ifc_attr_p)
                if rc == SX_STATUS_SUCCESS:
                    self.vrid_list.append(i)

        delete_sx_router_attributes_t_p(ifc_attr_p)

        return self.vrid_list

    def get_mc_vrids_in_use(self, update = False):
        ifc_attr_p = new_sx_router_attributes_t_p()

        if not self.vrid_list or update:
            for i in range(254): # max id is 253
                rc = sx_api_router_get(self.handle, i, ifc_attr_p)
                if rc == SX_STATUS_SUCCESS and ifc_attr_p.ipv4_mc_enable == SX_ROUTER_ENABLE_STATE_ENABLE:
                    self.vrid_list.append(i)

        delete_sx_router_attributes_t_p(ifc_attr_p)

        return self.vrid_list

    def _reset_network_addr_p_(self, table, args):
        entry, vrid = table[-1]
        sx_ip_prefix_t_p_assign(self.addr_type, entry.network_addr)

        return self.addr_type, vrid

    def get_ip_uc_route_table(self, version, vrid):
        if not vrid:
            vrid_list = self.get_vrids_in_use()
        else:
            vrid_list = [ vrid ]

        self.func_type      = sx_api_router_uc_route_get
        self.arr_type       =  new_sx_uc_route_get_entry_t_arr
        self.del_arr_func   =  delete_sx_uc_route_get_entry_t_arr
        self.addr_type      =  new_sx_ip_prefix_t_p()
        self.entry_type     = sx_uc_route_get_entry_t_arr_getitem
        self.filter_type    = None
        self.get_first      = SX_ACCESS_CMD_GET_FIRST
        self.get_next       = SX_ACCESS_CMD_GETNEXT
        self.max_entry_size = 20
        self.args = ['handle', 'cmd', 'id', 'addr', 'filter', 'list', 'count']
        self.reset_args = {'addr': self._reset_network_addr_p_}

        ip_addr = sx_ip_prefix_t()
        ip_addr.version = version
        if version == SX_IP_VERSION_IPV4 :
            ip_addr.prefix.ipv4.addr.s_addr = 0
        else :
            # ip_addr.prefix.ipv6.addr = 0
            pass # no set method
        sx_ip_prefix_t_p_assign(self.addr_type, ip_addr)

        table = self._get_table_(id_list=vrid_list)
        rt = OrderedDict()

        nhs = {}
        next_hop_cache = {}
        for route_entry, v_rid in table:
            rt_entry = {}
            rt_entry['vrid']    = v_rid
            rt_entry['action']  = router_action[route_entry.route_data.action]
            rt_entry['type']    = router_type[route_entry.route_data.type]
            rt_entry['prio']    = route_entry.route_data.trap_attr.prio
            rt_entry['ecmp_id'] = route_entry.route_data.uc_route_param.ecmp_id
            rt_entry['erif']    = route_entry.route_data.uc_route_param.local_egress_rif
            rt_entry['next_hop_cnt'] = route_entry.route_data.next_hop_cnt
            rt_entry['next_hops'] = []
            for idx in range(rt_entry['next_hop_cnt']) :
                next_hop = sx_ip_addr_t_arr_getitem(route_entry.route_data.next_hop_list_p, idx)
                naddr = hex_2_ip(next_hop, None)
                rt_entry['next_hops'].append(naddr)

            addr = hex_2_ip(None, route_entry.network_addr)
            mask = get_mask_length(route_entry.network_addr)
            ecmp_id = rt_entry['ecmp_id']
            if (ecmp_id > 0)                                and \
               (rt_entry['next_hop_cnt'] == 0)              and \
               router_action_is_forward(rt_entry['action']) and \
               (rt_entry['type'] == router_type[SX_UC_ROUTE_TYPE_NEXT_HOP]):
                # Get the next hop information from the ECMP entries
                if ecmp_id not in nhs:
                    nhs = self.get_hw_ecmp_next_hop_entries(ecmp_id, nhs)
                    if ecmp_id in nhs:
                        next_hop_cache[ecmp_id] = []
                        for next_hop in nhs[ecmp_id]:
                            nh = next_hop.next_hop_key.next_hop_key_entry.ip_next_hop.address
                            naddr = hex_2_ip(nh, None)

                            # filter for duplicates
                            if naddr in next_hop_cache[ecmp_id]:
                                continue
                            next_hop_cache[ecmp_id].append(naddr)

                # next_hop_cnt and ecmp_id can't be set at the same time in the HW
                # so, manually set the next_hop_cnt entry
                if ecmp_id in next_hop_cache:
                    for naddr in next_hop_cache[ecmp_id]:
                        rt_entry['next_hops'].append(naddr)
            rt_entry['next_hop_cnt'] = len(rt_entry['next_hops'])
            if len(rt_entry['next_hops']) == 0 :
                rt_entry['next_hops'].append('0')

            rt["%d %s/%d" % (v_rid, addr, mask)] = rt_entry
        return rt

    def get_ip_uc_route_count(self, version, vrid):
        if not vrid:
            vrid_list = self.get_vrids_in_use()
        else:
            vrid_list = [ vrid ]

        self.func_type      = sx_api_router_uc_route_get
        self.arr_type       =  new_sx_uc_route_get_entry_t_arr
        self.del_arr_func   =  delete_sx_uc_route_get_entry_t_arr
        self.addr_type      =  new_sx_ip_prefix_t_p()
        self.entry_type     = sx_uc_route_get_entry_t_arr_getitem
        self.filter_type    = None
        self.get_first      = SX_ACCESS_CMD_GET
        self.get_next       = SX_ACCESS_CMD_GET
        self.max_entry_size = 0
        self.args = ['handle', 'cmd', 'id', 'addr', 'filter', 'list', 'count']
        self.reset_args = {'addr': self._reset_network_addr_p_}

        ip_addr = sx_ip_prefix_t()
        ip_addr.version = version
        if version == SX_IP_VERSION_IPV4 :
            ip_addr.prefix.ipv4.addr.s_addr = 0
        else :
            # ip_addr.prefix.ipv6.addr = 0
            pass # no set method
        sx_ip_prefix_t_p_assign(self.addr_type, ip_addr)

        table_count = self._get_table_count_(id_list=vrid_list)

        return {'route_count': table_count}

    def _reset_mc_key_p_(self, table, args):
        entry, vrid = table[-1]
        sx_mc_route_key_t_p_assign(self.mc_key_type, entry.mc_route_key)

        return self.mc_key_type, vrid

    def get_ip_mc_route_table(self, vrid=None, version=4):
        if not vrid:
            vrid_list = self.get_mc_vrids_in_use()
        else:
            vrid_list = [ vrid ]

        self.func_type          = sx_api_router_mc_route_get
        self.arr_type           = new_sx_mc_route_get_entry_t_arr
        self.del_arr_func       = delete_sx_mc_route_get_entry_t_arr
        self.addr_type          = None
        self.mc_key_type        = new_sx_mc_route_key_t_p()
        self.entry_type         = sx_mc_route_get_entry_t_arr_getitem
        self.filter_type        = None
        self.get_first          = SX_ACCESS_CMD_GET_FIRST
        self.get_next           = SX_ACCESS_CMD_GETNEXT
        self.max_entry_size     = 20
        self.args   = ['handle', 'cmd', 'id', 'mc_key', 'filter', 'list', 'count']
        self.reset_args = {'mc_key': self._reset_mc_key_p_}

        mc_key = sx_mc_route_key_t()
        mc_key.source_addr.version = version
        mc_key.mc_group_addr.version = version
        if version == SX_IP_VERSION_IPV4 :
            mc_key.source_addr.prefix.ipv4.addr.s_addr = 0
            mc_key.mc_group_addr.prefix.ipv4.addr.s_addr = 0
        else :
            pass
        mc_key.ingress_rif = 0
        sx_mc_route_key_t_p_assign(self.mc_key_type, mc_key)

        table = self._get_table_(id_list=vrid_list)
        mrt = OrderedDict()

        for mce, v_rid in table:
            mc_entry = {}
            mc_entry['vrid']    = v_rid
            mc_entry['iif'] = mce.mc_route_key.ingress_rif
            mc_entry['egress_rif_cnt'] = mce.mc_route_data.egress_rif_cnt
            mc_entry['egress_rifs'] = []
            for idx in range(mc_entry['egress_rif_cnt']) :
                erif = sx_router_interface_t_arr_getitem(mce.mc_route_data.egress_rif_list_p, idx)
                mc_entry['egress_rifs'].append(erif)

            src_addr = hex_2_ip(None, mce.mc_route_key.source_addr)
            grp_addr = hex_2_ip(None, mce.mc_route_key.mc_group_addr)

            mrt["%s/%s" % (src_addr, grp_addr)] = mc_entry
        return mrt

    def invalid_port_id(self):
        return SX_INVALID_PORT

    def _reset_ip_addr_p_(self, table, args):
        entry, vrid = table[-1]
        sx_ip_addr_t_p_assign(self.addr_type, entry.ip_addr)

        # Set the ID value to the last RIF value
        rif = entry.neigh_data.rif

        return self.addr_type, rif

    def get_ecmp_table(self, eid=None) :
        if not eid:
            eid_list = []
        else:
            eid_list = [ eid ]

        self.func_type   = sx_api_router_ecmp_get
        self.arr_type    = new_sx_next_hop_t_arr
        self.del_arr_func = delete_sx_next_hop_t_arr
        self.entry_type  = sx_next_hop_t_arr_getitem
        self.args        = ['handle', 'id', 'list', 'count']
        self.addr_type   = None
        self.filter_type = None
        self.get_first   = None
        self.get_next    = None
        self.max_entry_size = 4096
        self.reset_args = {}
        table = self._get_table_(id_list=eid_list)
        et = {}

        for next_hop, ecmp_id in table:
            rif = next_hop.next_hop_key.next_hop_key_entry.ip_next_hop.rif;
            if ecmp_id in et :
                et[ecmp_id].append(rif)
            else :
                et[ecmp_id] = [rif]
        return et

    def get_operational_ecmp_table(self, eid=None) :
        if not eid:
            eid_list = []
        else:
            eid_list = [ eid ]

        self.func_type   = sx_api_router_operational_ecmp_get
        self.arr_type    = new_sx_next_hop_t_arr
        self.del_arr_func = delete_sx_next_hop_t_arr
        self.entry_type  = sx_next_hop_t_arr_getitem
        self.args        = ['handle', 'id', 'list', 'count']
        self.addr_type   = None
        self.filter_type = None
        self.get_first   = None
        self.get_next    = None
        self.max_entry_size = 4096
        self.reset_args = {}
        table = self._get_table_(id_list=eid_list)
        et = {}

        for next_hop, ecmp_id in table:
            nh_entry = {}
            if next_hop.next_hop_key.type == SX_NEXT_HOP_TYPE_IP:
                nh_entry['type'] = 'IP'
                naddr = hex_2_ip(next_hop.next_hop_key.next_hop_key_entry.ip_next_hop.address, None)
                nh_entry['address'] = naddr
                nh_entry['rif']     = next_hop.next_hop_key.next_hop_key_entry.ip_next_hop.rif
            elif next_hop.next_hop_key.type == SX_NEXT_HOP_TYPE_TUNNEL_ENCAP:
                nh_entry['type'] = 'TUNNEL_ENCAP'
                naddr = hex_2_ip(next_hop.next_hop_key.next_hop_key_entry.ip_tunnel.underlay_dip, None)
                nh_entry['underlay_dip'] = naddr
                nh_entry['tunnel_id']    = next_hop.next_hop_key.next_hop_key_entry.ip_tunnel.tunnel_id
            else:
                continue

            if ecmp_id in et :
                et[ecmp_id].append(nh_entry)
            else :
                et[ecmp_id] = [nh_entry]
        return et

    def get_hw_ecmp_next_hop_entries(self, ecmp_id, ecmp_next_hop_dict):
        if (not ecmp_id) or (ecmp_id in ecmp_next_hop_dict):
            # invalid parameter or we've already cached the next hop state
            return ecmp_next_hop_dict

        # get the number of next hop entries in hardware for this ECMP id
        ecmp_next_hop_dict[ecmp_id] = []
        next_hop_count_p = new_uint32_t_p()
        uint32_t_p_assign(next_hop_count_p, 0)
        rc = sx_api_router_operational_ecmp_get(self.handle, ecmp_id, None, next_hop_count_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_router_operational_ecmp_get failed: %d' % rc
            delete_uint32_t_p(next_hop_count_p)
            sys.stdout.close()
            exit(-1)
        next_hop_count  = uint32_t_p_value(next_hop_count_p)
        next_hop_list_p = new_sx_next_hop_t_arr(next_hop_count)
        rc = sx_api_router_operational_ecmp_get(self.handle, ecmp_id, next_hop_list_p, next_hop_count_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_router_operational_ecmp_get %d next hops failed: %d' % (next_hop_count, rc)
            delete_uint32_t_p(next_hop_count_p)
            delete_sx_next_hop_t_arr(next_hop_list_p)
            sys.stdout.close()
            exit(-1)
        next_hop_count  = uint32_t_p_value(next_hop_count_p)
        for nh_idx in range(next_hop_count):
            next_hop = sx_next_hop_t_arr_getitem(next_hop_list_p, nh_idx)
            ecmp_next_hop_dict[ecmp_id].append(next_hop)

        delete_uint32_t_p(next_hop_count_p)
        delete_sx_next_hop_t_arr(next_hop_list_p)
        return ecmp_next_hop_dict

    def get_hw_ecmp_attributes(self, ecmp_id):
        attrs = {}
        if (not ecmp_id):
            return attrs

        # get the number of next hop entries in hardware for this ECMP id
        attributes_p = new_sx_ecmp_attributes_t_p()
        uint32_t_p_assign(nh_count_p, 0)
        rc = sx_api_router_ecmp_attributes_get(self.handle, ecmp_id, attributes_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_router_operational_ecmp_get failed: %d' % rc
            delete_sx_ecmp_attribute_t_p(attributes_p)
            sys.stdout.close()
            exit(-1)

        data = sx_ecmp_attributes_t_p_value(attributes_p)
        attrs = {'ecmp_type'           : ecmp_type.get(data.ecmp_type, 'UNKNOWN'),
                 'container_type'      : container_type.get(data.container_type, 'UNKNOWN'),
                 'active_flow_timer'   : data.active_flow_timer,
                 'group_size'          : data.group_size,
                 'max_unbalanced_time' : data.max_unbalanced_time}

        delete_sx_ecmp_attribute_t_p(attributes_p)
        return attrs

    def get_router_interface(self) :
        vrid_p = new_sx_router_id_t_p()
        ifc_p  = new_sx_router_interface_param_t_p()
        ifc_attr_p = new_sx_interface_attributes_t_p()
        interface_info = {}
        interface_info['labels'] = ['type', 'vrid', 'mtu', 'swid', 'port', 'vlan', 'vport', 'bridge id', 'egress mode', 'mac_addr']
        interface_info['entries'] = {}
        for rif in range(401) :
            rc = sx_api_router_interface_get(self.handle,
                                             rif,
                                             vrid_p,
                                             ifc_p,
                                             ifc_attr_p)
            if rc != SX_STATUS_SUCCESS:
                if rc != SX_STATUS_ENTRY_NOT_FOUND :
                    print 'sx_api_router_interface_attributes failed for rif %d: %d' % (rif, rc)
                    break
                else :
                    continue
            ifc = sx_router_interface_param_t_p_value(ifc_p)
            data = sx_interface_attributes_t_p_value(ifc_attr_p)
            interface_info['entries'][rif] = {}
            rif_info = {}
            for label in interface_info['labels'] :
                rif_info[label] = None
            rif_info['vrid'] = sx_router_id_t_p_value(vrid_p)
            rif_info['mtu'] = data.mtu
            rif_info['mac'] = data.mac_addr.to_str()
            if ifc.type == SX_L2_INTERFACE_TYPE_VLAN:
                rif_info['type'] = 'vlan'
                rif_info['swid'] = ifc.ifc.vlan.swid
                rif_info['vlan'] = ifc.ifc.vlan.vlan
            elif ifc.type == SX_L2_INTERFACE_TYPE_PORT_VLAN :
                rif_info['type'] = 'port vlan'
                rif_info['port'] = ifc.ifc.port_vlan.port
                rif_info['vlan'] = ifc.ifc.port_vlan.vlan
            elif ifc.type == SX_L2_INTERFACE_TYPE_VPORT :
                rif_info['type'] = 'vport'
                rif_info['vport'] = ifc.ifc.vport.vport
            elif ifc.type == SX_L2_INTERFACE_TYPE_PKEY :
                rif_info['type'] = 'pkey'
            elif ifc.type == SX_L2_INTERFACE_TYPE_BRIDGE_PORT :
                rif_info['type'] = 'bridge port'
                rif_info['port'] = ifc.ifc.bridge_port.port
                rif_info['vlan'] = ifc.ifc.bridge_port.vlan
                rif_info['egress mode'] = ifc.ifc.bridge_port.egress_mode
            elif ifc.type == SX_L2_INTERFACE_TYPE_BRIDGE :
                rif_info['type'] = 'bridge'
                rif_info['swid'] = ifc.ifc.bridge.swid
                rif_info['bridge id'] = ifc.ifc.bridge.bridge
            interface_info['entries'][rif] = rif_info

        delete_sx_router_id_t_p(vrid_p)
        delete_sx_router_interface_param_t_p(ifc_p)
        delete_sx_interface_attributes_t_p(ifc_attr_p)
        return interface_info

    def set_router_interface(self, vrid, rif) :
        rif_p      = new_sx_router_interface_t_p()
        sx_router_interface_t_p_assign(rif_p, rif)
        vrid_p = new_sx_router_id_t_p()
        sx_router_id_t_p_assign(vrid_p, 0)
        ifc_p = new_sx_router_interface_param_t_p()
        ifc = sx_router_interface_param_t_p_value(ifc_p)
        ifc.type = SX_L2_INTERFACE_TYPE_BRIDGE
        sx_router_interface_param_t_p_assign(ifc_p, ifc)
        ifc_attr_p = new_sx_interface_attributes_t_p()
        ifc_attr = sx_interface_attributes_t_p_value(ifc_attr_p)
        ifc_attr.mtu = 9000
        ifc_attr.multicast_ttl_threshold = 64
        ifc_attr.loopback_enable = 0
        sx_interface_attributes_t_p_assign(ifc_attr_p, ifc_attr)
        rc = sx_api_router_interface_set(self.handle,
                                         SX_ACCESS_CMD_EDIT,
                                         sx_router_id_t_p_value(vrid_p),
                                         ifc_p,
                                         ifc_attr_p,
                                         rif_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_router_interface_attributes failed: %d' % rc
            delete_sx_router_interface_t_p(rif_p)
            delete_sx_router_id_t_p(vrid_p)
            delete_sx_router_interface_param_t_p(ifc_p)
            delete_sx_interface_attributes_t_p(ifc_attr_p)
            sys.stdout.close()
            exit(-1)

        delete_sx_router_interface_t_p(rif_p)
        delete_sx_router_id_t_p(vrid_p)
        delete_sx_router_interface_param_t_p(ifc_p)
        delete_sx_interface_attributes_t_p(ifc_attr_p)

    def get_intf_table(self, rif) :
        num_entries_p = new_uint32_t_p()
        uint32_t_p_assign(num_entries_p, 20)
        mac_list      = new_sx_mac_addr_t_arr(64)
        table    = []
        rc = sx_api_router_interface_mac_get(self.handle, rif, mac_list, num_entries_p)
        if rc == SX_STATUS_SUCCESS:
            num_entries = uint32_t_p_value(num_entries_p)
            for i in range(num_entries):
                entry = sx_mac_addr_t_arr_getitem(mac_list, i)
                table.append(entry)
        else:
            delete_uint32_t_p(num_entries_p)
            delete_sx_mac_addr_t_arr(mac_list)
            sys.stdout.close()
            exit(-1)
        it = {}
        for mac_addr in table:
            mac  = mac_addr.to_str()
            it[rif] = mac

        delete_uint32_t_p(num_entries_p)
        delete_sx_mac_addr_t_arr(mac_list)
        return it

    def get_ip_neighbor_table(self, version, rif):
        # rif value is ignored if the command is not SX_ACCESS_CMD_GET
        rid_list = [ rif ]
        self.func_type = sx_api_router_neigh_get
        self.arr_type = new_sx_neigh_get_entry_t_arr
        self.del_arr_func = delete_sx_neigh_get_entry_t_arr
        self.addr_type =  new_sx_ip_addr_t_p()
        self.entry_type = sx_neigh_get_entry_t_arr_getitem
        self.filter_type = None
        self.get_first = SX_ACCESS_CMD_GET_FIRST
        self.get_next = SX_ACCESS_CMD_GETNEXT
        self.max_entry_size = 20
        self.args = ['handle', 'cmd', 'id', 'addr', 'filter', 'list', 'count']
        self.reset_args = {'addr': self._reset_ip_addr_p_}

        ip_addr = sx_ip_addr_t()
        ip_addr.version = version
        if version == SX_IP_VERSION_IPV4 :
            ip_addr.addr.ipv4.addr = 0
        else :
            ip_addr.addr.ipv6.addr = 0
        sx_ip_addr_t_p_assign(self.addr_type, ip_addr)

        table = self._get_table_(id_list=rid_list)

        nt = {}
        addr_idx = 1
        for neighbor, vrid in table:
            addr = hex_2_ip(neighbor.ip_addr, None)
            mac  = neighbor.neigh_data.mac_addr.to_str()
            rif  = neighbor.neigh_data.rif
            key = '{0} {1}'.format(addr, rif)
            nt[key] = {}
            nt[key]['mac']    = mac
            nt[key]['vrid']   = vrid
            nt[key]['action'] = router_action[neighbor.neigh_data.action]
            nt[key]['prio']   = neighbor.neigh_data.trap_attr.prio
            nt[key]['rif']    = rif

        delete_sx_ip_addr_t_p(self.addr_type)
        return nt

    def get_neighbor_table(self, version, rif):
        if version == 4 :
            return self.get_ip_neighbor_table(version=SX_IP_VERSION_IPV4, rif=rif)
        else :
            return self.get_ip_neighbor_table(version=SX_IP_VERSION_IPV6, rif=rif)

    def get_uc_route_table(self, version, vrid):
        if version == 4:
            return self.get_ip_uc_route_table(version=SX_IP_VERSION_IPV4, vrid=vrid)
        else:
            return self.get_ip_uc_route_table(version=SX_IP_VERSION_IPV6, vrid=vrid)

    def get_uc_route_count(self, version, vrid):
        if version == 4:
            return self.get_ip_uc_route_count(version=SX_IP_VERSION_IPV4, vrid=vrid)
        else:
            return self.get_ip_uc_route_count(version=SX_IP_VERSION_IPV6, vrid=vrid)

    def get_mc_route_table(self, version, vrid):
        if version == 4 :
            return self.get_ip_mc_route_table(version=SX_IP_VERSION_IPV4, vrid=vrid)
        else :
            return self.get_ip_mc_route_table(version=SX_IP_VERSION_IPV6, vrid=vrid)

    def _pack_args_(self, vrid, num_entries, args_map):
        args = []
        cmd_indx = 0
        for i, arg in enumerate(self.args):
            if arg in args_map:
                args.append(args_map[arg])
            else:
                if arg == 'id':
                    args.append(vrid)
                elif arg == 'cmd':
                    args.append(self.get_first)
                    cmd_indx = i
                elif arg =='count':
                    args.append(num_entries)
                elif arg == 'elements':
                    args.append(self.element_type)
                elif arg == 'mc_key':
                    args.append(self.mc_key_type)

        return cmd_indx, args

    def _get_table_(self, id_list=[0], version=4):
        if (not self.get_first) and (not self.get_next):
            entry_list = self.arr_type(self.max_entry_size)
        else:
            entry_list = self.arr_type(64)
        num_entries = new_uint32_t_p()

        args_map = { 'handle': self.handle,
                     'addr': self.addr_type,
                     'filter': self.filter_type,
                     'list': entry_list }
        table = []
        for vrid in id_list:
            num = 0

            # The Maximum number of entries to get by the MLX SDK for
            # the different types of tables
            uint32_t_p_assign(num_entries, self.max_entry_size)

            while True:
                cmd_indx, args = self._pack_args_(vrid, num_entries, args_map)

                rc = self.func_type(*args)

                if rc == SX_STATUS_SUCCESS:
                    num = uint32_t_p_value(args[-1])
                    if (num <= self.max_entry_size):
                        for i in range(num):
                            entry = self.entry_type(entry_list, i)
                            table.append((entry, vrid))

                        break
                    elif (not self.get_first) and (not self.get_next) and \
                         (num > self.max_entry_size):
                        # The get first and get next operations aren't supported
                        # and we need to resize the array to be the correct size to
                        # get the value
                        entry_list = self.arr_type(num)
                        uint32_t_p_assign(num_entries, num)
                        self.max_entry_size = num
                else:
                    break

            # Get next set of entries if the numbered return is the maximum
            # number of entries (20 for the router structures, 64 for the fdb)
            while (rc == SX_STATUS_SUCCESS) and \
                  (num == self.max_entry_size):
                # If the get_next command isn't supported, skip this
                if (not self.get_next) and (not self.get_first):
                    break

                uint32_t_p_assign(num_entries, self.max_entry_size)

                # Reset other variables
                rid = -1
                for varname in self.reset_args:
                    args_map[varname], rid = self.reset_args[varname](table, args)

                # re-pack args
                cmd_indx, args = self._pack_args_(vrid, num_entries, args_map)
                args[cmd_indx] = self.get_next

                if 'id' in self.args and rid != -1:
                    iidx = self.args.index('id')
                    args[iidx] = rid

                rc = self.func_type(*args)

                if rc == SX_STATUS_SUCCESS:
                    num = uint32_t_p_value(args[-1])
                    for i in range(num):
                        entry = self.entry_type(entry_list, i)
                        table.append((entry, vrid))

        self.del_arr_func(entry_list)
        delete_uint32_t_p(num_entries)
        return table

    def _get_table_count_(self, id_list=[0], version=4):
        entry_list = self.arr_type(64)
        num_entries = new_uint32_t_p()

        args_map = { 'handle': self.handle,
                     'addr': self.addr_type,
                     'filter': self.filter_type,
                     'list': entry_list }
        table = []
        num = 0
        for vrid in id_list:

            # Set the number of entries to 0 to just get the count
            uint32_t_p_assign(num_entries, 0)

            cmd_indx, args = self._pack_args_(vrid, num_entries, args_map)

            rc = self.func_type(*args)
            if rc == SX_STATUS_SUCCESS:
                num += uint32_t_p_value(args[-1])

        self.del_arr_func(entry_list)
        delete_uint32_t_p(num_entries)
        return num

    def _reset_mac_addr_p_(self, table, args):
        entry, swid = table[-1]
        self.addr_type.fid_vid = entry.fid_vid
        self.addr_type.mac_addr = entry.mac_addr

        return self.addr_type, swid

    def get_fdb_table(self, swid=0, filter_type = (None, None)):
        self.func_type = sx_api_fdb_uc_mac_addr_get
        self.arr_type = new_sx_fdb_uc_mac_addr_params_t_arr
        self.del_arr_func = delete_sx_fdb_uc_mac_addr_params_t_arr
        self.addr_type = new_sx_fdb_uc_mac_addr_params_t_p()
        self.entry_type = sx_fdb_uc_mac_addr_params_t_arr_getitem
        self.get_first = SX_ACCESS_CMD_GET_FIRST
        self.get_next = SX_ACCESS_CMD_GETNEXT
        self.filter_type = new_sx_fdb_uc_key_filter_t_p()
        self.max_entry_size = 64
        self.element_type = SX_FDB_UC_ALL
        self.args = ['handle', 'id', 'cmd', 'elements', 'addr', 'filter', 'list', 'count']
        self.reset_args = {'addr': self._reset_mac_addr_p_}

        table = self._get_table_(id_list=[swid])
        fdb = {}

        for entry, swid in table:
            mac = entry.mac_addr.to_str()
            if mac in fdb:
                fdb[mac].append({})
            else:
                fdb[mac] = [{}]

            fdb[mac][-1]['swid'] = swid
            fdb[mac][-1]['action'] = fdb_action[entry.action]
            fdb[mac][-1]['entry_type'] = fdb_entry_type[entry.entry_type]
            fdb[mac][-1]['fid_vid'] = entry.fid_vid
            fdb[mac][-1]['logical_port'] = entry.log_port
            fdb[mac][-1]['lport_type'] = port_type_id_get(entry.log_port)
            if port_is_nve(entry.log_port):
                fdb[mac][-1]['port_id'] = 'nve'
            elif (port_phy_id_get(entry.log_port) > 0) or \
               (fdb[mac][-1]['lport_type'] != 0):
                fdb[mac][-1]['port_id'] = "%d" % (port_phy_id_get(entry.log_port))
            else:
                fdb[mac][-1]['port_id'] = 'cpu'
            fdb[mac][-1]['vlan_id'] = port_vlan_id_get(entry.log_port)

        delete_sx_fdb_uc_mac_addr_params_t_p(self.addr_type)
        delete_sx_fdb_uc_key_filter_t_p(self.filter_type)
        return fdb

    def get_fdb_age_time(self, swid_list=[0]):
        age = {}
        age_time = new_uint32_t_p()

        for swid in swid_list:
            rc = sx_api_fdb_age_time_get(self.handle, swid, age_time)

            if rc == SX_STATUS_SUCCESS:
                value = uint32_t_p_value(age_time)
                age[swid] = value
            else:
                delete_uint32_t_p(age_time)
                sys.stdout.close()
                exit(-1)

        delete_uint32_t_p(age_time)
        return age

    def set_fdb_age_time(self, value, swid_list=[0]):
        for swid in swid_list:
            rc = sx_api_fdb_age_time_set(self.handle, swid, value)

            if rc != SX_STATUS_SUCCESS:
                sys.stdout.close()
                exit(-1)

    def flush_uc_fdb_all(self, swid_list=[0]):
        for swid in swid_list:
            rc = sx_api_fdb_uc_flush_all_set(self.handle, swid)

            if rc != SX_STATUS_SUCCESS:
                sys.stdout.close()
                exit(-1)

    def flush_uc_fdb_fid(self, fid, swid=0):
        rc = sx_api_fdb_uc_flush_fid_set(self.handle, swid, fid)

        if rc != SX_STATUS_SUCCESS:
            sys.stdout.close()
            exit(-1)

    def flush_uc_fdb_log_port(self, log_port_id):
        rc = sx_api_fdb_uc_flush_port_set(self.handle, log_port_id)

        if rc != SX_STATUS_SUCCESS:
            sys.stdout.close()
            exit(-1)

    def get_vlan_ids(self, swid_list=[0]):
        vlan_id_in_use = []

        vlan_id_list = new_sx_vlan_id_t_arr(4096)

        for swid in swid_list:
            num_vlans = new_uint32_t_p()
            rc = sx_api_vlan_get(self.handle, 0, vlan_id_list, num_vlans)

            if rc != SX_STATUS_SUCCESS:
                delete_sx_vlan_id_t_arr(vlan_id_list)
                delete_uint32_t_p(num_vlans)
                sys.stdout.close()
                exit(-1)

            for i in range(uint32_t_p_value(num_vlans)):
                vid_entry = sx_vlan_id_t_arr_getitem(vlan_id_list, i)
                vlan_id_in_use.append((swid, vid_entry))

            delete_uint32_t_p(num_vlans)

        delete_sx_vlan_id_t_arr(vlan_id_list)
        return vlan_id_in_use

    def get_bridge_vports(self, swid=0, br_vid=None):
        ports = []

        if not br_vid:
            sys.stdout.close()
            exit(-1)

        br_vports = new_sx_port_log_id_t_arr(256)
        num_br_vports = new_uint32_t_p()
        uint32_t_p_assign(num_br_vports, 256)

        rc = sx_api_bridge_vport_get(self.handle, int(br_vid), br_vports, num_br_vports)

        if rc != SX_STATUS_SUCCESS:
            delete_sx_port_log_id_t_p(br_vports)
            delete_uint32_t_p(num_br_vports)
            sys.stdout.close()
            exit(-1)

        for i in range(uint32_t_p_value(num_br_vports)):
            port = sx_port_log_id_t_arr_getitem(br_vports, i)
            if port not in ports:
                ports.append(port)

        delete_sx_port_log_id_t_p(br_vports)
        delete_uint32_t_p(num_br_vports)

        vport_info = []
        for port in ports:
                logical_port = port
                log_port_type = (logical_port >> 28) & 0xF
                log_port_vlan_or_device = (logical_port >> 16) & 0xFFF
                log_port_id = (logical_port >> 8) & 0xFF
                log_port_sub_id = logical_port & 0xFF
                vport_info.append({'type': log_port_type,
                                           'device_vlan_id': log_port_vlan_or_device,
                                           'port_id': log_port_id,
                                           'sub_port_id': log_port_sub_id,
                                           'encoded_log_port': logical_port})
        return vport_info

    def get_vlan_ports(self, swid=0, vids=[]):
        vlan_port_map = {}

        if not vids:
            info = self.get_vlan_ids(swid_list=[swid])
            tmp = []
            for swid, vid in info:
                tmp.append(vid)

            vids = tmp

        vlan_port_list = new_sx_vlan_ports_t_arr(256)
        num_vlan_port = new_uint32_t_p()

        for vid in vids:
            vlan_port_map[vid] = []
            uint32_t_p_assign(num_vlan_port, 256)

            rc = sx_api_vlan_ports_get(self.handle, swid, vid,
                                       vlan_port_list, num_vlan_port)

            if rc != SX_STATUS_SUCCESS:
                delete_sx_vlan_ports_t_arr(vlan_port_list)
                delete_uint32_t_p(num_vlan_port)
                sys.stdout.close()
                exit(-1)

            for i in range(uint32_t_p_value(num_vlan_port)):
                vlan_port_entry = sx_vlan_ports_t_arr_getitem(vlan_port_list, i)
                is_untagged = vlan_port_entry.is_untagged
                logical_port = vlan_port_entry.log_port
                log_port_type = (logical_port >> 28) & 0xF
                log_port_vlan_or_device = (logical_port >> 16) & 0xFFF
                log_port_id = (logical_port >> 8) & 0xFF
                log_port_sub_id = logical_port & 0xFF
                vlan_port_map[vid].append({'type': log_port_type,
                                           'device_vlan_id': log_port_vlan_or_device,
                                           'port_id': log_port_id,
                                           'sub_port_id': log_port_sub_id,
                                           'untagged': is_untagged,
                                           'encoded_log_port': logical_port})

        delete_sx_vlan_ports_t_arr(vlan_port_list)
        delete_uint32_t_p(num_vlan_port)

        return vlan_port_map

    def get_port_counters(self, portid):

        ieee_p = new_sx_port_cntr_ieee_802_dot_3_t_p()
        rc = sx_api_port_counter_ieee_802_dot_3_get(self.handle,
                                                    SX_ACCESS_CMD_READ,
                                                    portid,
                                                    ieee_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_port_counter_ieee_get failed: %d' % rc
            delete_sx_port_cntr_ieee_802_dot_3_t_p(ieee_p)
            sys.stdout.close()
            exit(-1)
        ieee_data = sx_port_cntr_ieee_802_dot_3_t_p_value(ieee_p)
        delete_sx_port_cntr_ieee_802_dot_3_t_p(ieee_p)

        tc_data   = []
        for tc_value in range(SX_PORT_TC_ID_MIN, SX_PORT_TC_ID_MAX, 1) :
            tc_p = new_sx_port_traffic_cntr_t_p()
            rc = sx_api_port_counter_tc_get(self.handle,
                                            SX_ACCESS_CMD_READ,
                                            portid,
                                            tc_value,
                                            tc_p)
            if rc != SX_STATUS_SUCCESS:
                print 'sx_api_port_counter_tc_get failed'
                delete_sx_port_traffic_cntr_t_p(tc_p)
                sys.stdout.close()
                exit(-1)
            tc_data.append(sx_port_traffic_cntr_t_p_value(tc_p))
            delete_sx_port_traffic_cntr_t_p(tc_p)

        prio_data = []
        for idx, prio_value in enumerate(priority_value_list) :
            prio_p = new_sx_port_cntr_prio_t_p()
            rc = sx_api_port_counter_prio_get(self.handle,
                                              SX_ACCESS_CMD_READ,
                                              portid,
                                              prio_value,
                                              prio_p)
            if rc != SX_STATUS_SUCCESS:
                print 'sx_api_port_counter_prio_get failed: %d' % rc
                delete_sx_port_cntr_prio_t_p(prio_p)
                sys.stdout.close()
                exit(-1)
            prio_data.append(sx_port_cntr_prio_t_p_value(prio_p))
            delete_sx_port_cntr_prio_t_p(prio_p)

        perf_p = new_sx_port_cntr_perf_t_p()
        rc = sx_api_port_counter_perf_get(self.handle,
                                          SX_ACCESS_CMD_READ,
                                          portid,
                                          SX_PORT_PRIO_ID_0,
                                          perf_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_port_counter_perf_get failed: prio value %d, rc %d' % (prio_value, rc)
            delete_sx_port_cntr_perf_t_p(perf_p)
            sys.stdout.close()
            exit(-1)
        perf_data = sx_port_cntr_perf_t_p_value(perf_p)
        delete_sx_port_cntr_perf_t_p(perf_p)

        cntr_p = new_sx_port_cntr_cli_t_p()
        rc = sx_api_port_counter_cli_get(self.handle,
                                         SX_ACCESS_CMD_READ,
                                         portid,
                                         cntr_p)
        if rc != SX_STATUS_SUCCESS:
            delete_sx_port_cntr_cli_t_p(cntr_p)
            print 'sx_api_port_counter_cli_get failed'
            sys.stdout.close()
            exit(-1)
        data = sx_port_cntr_cli_t_p_value(cntr_p)
        delete_sx_port_cntr_cli_t_p(cntr_p)

        port_labels = [ 'rx_bytes',
                        'rx_frames',
                        'rx_bc',
                        'rx_mc',
                        'rx_uc',
                        'rx_jumbo',
                        'rx_buffer',
                        'rx_fcs',
                        'rx_runt',
                        'rx_other',
                        'tx_bytes',
                        'tx_frames',
                        'tx_bc',
                        'tx_mc',
                        'tx_uc',
                        'tx_jumbo',
                        'tx_errors' ]

        port_values = [{ 'rx_bytes'     : data.port_rx_octets,
                         'rx_frames'    : data.port_rx_frames,
                         'rx_bc'        : data.port_rx_broadcast,
                         'rx_mc'        : data.port_rx_multicast,
                         'rx_uc'        : data.port_rx_unicast,
                         'rx_jumbo'     : data.port_rx_jumbo,
                         'rx_buffer'    : data.port_rx_no_buffer,
                         'rx_fcs'       : data.port_rx_fcs_errors,
                         'rx_runt'      : data.port_rx_runt,
                         'rx_other'     : data.port_rx_other_errors,
                         'tx_bytes'     : data.port_tx_octets,
                         'tx_frames'    : data.port_tx_frames,
                         'tx_bc'        : data.port_tx_broadcast,
                         'tx_mc'        : data.port_tx_multicast,
                         'tx_uc'        : data.port_tx_unicast,
                         'tx_jumbo'     : data.port_tx_jumbo,
                         'tx_errors'    : data.port_tx_errors }]

        ieee_labels = [ 'tx_good_frames',
                        'tx_good_bytes',
                        'tx_good_mc',
                        'tx_good_bc',
                        'rx_good_frames',
                        'rx_good_bytes',
                        'rx_good_mc',
                        'rx_good_bc',
                        'fcs_errors',
                        'alignment_errors',
                        'in_range_length_errors',
                        'out_of_range_length_errors',
                        'frame_too_long_errors',
                        'symbol_error_during_carrier',
                        'tx_mac_control_frames',
                        'rx_mac_control_frames',
                        'unsupported_opcodes_received',
                        'tx_pause_mac_ctrl_frames',
                        'rx_pause_mac_ctrl_frames' ]

        ieee_values = [{ 'tx_good_frames' : ieee_data.a_frames_transmitted_ok,
                         'tx_good_bytes'  : ieee_data.a_octets_transmitted_ok,
                         'tx_good_mc'     : ieee_data.a_multicast_frames_xmitted_ok,
                         'tx_good_bc'     : ieee_data.a_broadcast_frames_xmitted_ok,
                         'rx_good_frames' : ieee_data.a_frames_received_ok,
                         'rx_good_bytes'  : ieee_data.a_octets_received_ok,
                         'rx_good_mc'     : ieee_data.a_multicast_frames_received_ok,
                         'rx_good_bc'     : ieee_data.a_broadcast_frames_recieved_ok,
                         'fcs_errors'   : ieee_data.a_frame_check_sequence_errors,
                         'alignment_errors'             : ieee_data.a_alignment_errors,
                         'in_range_length_errors'       : ieee_data.a_in_range_length_errors,
                         'out_of_range_length_errors'   : ieee_data.a_out_of_range_length_field,
                         'frame_too_long_errors'        : ieee_data.a_frame_too_long_errors,
                         'symbol_error_during_carrier'  : ieee_data.a_symbol_error_during_carrier,
                         'tx_mac_control_frames'        : ieee_data.a_mac_control_frames_transmitted,
                         'rx_mac_control_frames'        : ieee_data.a_mac_control_frames_received,
                         'unsupported_opcodes_received' : ieee_data.a_unsupported_opcodes_received,
                         'rx_pause_mac_ctrl_frames'     : ieee_data.a_pause_mac_ctrl_frames_received,
                         'tx_pause_mac_ctrl_frames'     : ieee_data.a_pause_mac_ctrl_frames_transmitted }]

        prio_labels = [ 'rx_frames',
                        'rx_pause',
                        'rx_pause_duration',
                        'tx_frames',
                        'tx_pause',
                        'tx_pause_duration',
                        'rx_discard' ]

        prio_values = []
        for prio_idx in range(len(priority_value_list)):
            prio_values.append({})
            prio_values[prio_idx]['switch priority'] = priority_value_list[prio_idx]
            prio_values[prio_idx]['rx_frames']     = prio_data[prio_idx].rx_frames
            prio_values[prio_idx]['rx_pause']      = prio_data[prio_idx].rx_pause
            prio_values[prio_idx]['rx_discard']    = prio_data[prio_idx].rx_discard
            prio_values[prio_idx]['rx_pause_duration']  = prio_data[prio_idx].rx_pause_duration
            prio_values[prio_idx]['tx_frames']     = prio_data[prio_idx].tx_frames
            prio_values[prio_idx]['tx_pause']      = prio_data[prio_idx].tx_pause
            prio_values[prio_idx]['tx_pause_duration']  = prio_data[prio_idx].tx_pause_duration

        perf_labels = ['tx_wait',
                       'ecn_marked',
                       'mc_no_buffer' ]

        perf_values = [{ 'tx_wait'      : perf_data.tx_wait,
                         'ecn_marked'   : perf_data.ecn_marked,
                         'mc_no_buffer' : perf_data.no_buffer_discard_mc }]

        tc_labels = [ 'tx_frames',
                      'tx_bytes',
                      'tx_uc_frames',
                      'tx_mc_frames',
                      'tx_bc_frames',
                      'tx_queue',
                      'tx_no_buffer_discard_uc',
                      'tx_wred_discard']
        tc_values = []
        for tc_idx in range(SX_PORT_TC_ID_MIN, SX_PORT_TC_ID_MAX, 1) :
            tc_values.append({})
            tc_values[tc_idx]['traffic class'] = tc_idx
            tc_values[tc_idx]['tx_frames']    = tc_data[tc_idx].tx_frames
            tc_values[tc_idx]['tx_bytes']     = tc_data[tc_idx].tx_octet
            tc_values[tc_idx]['tx_uc_frames'] = tc_data[tc_idx].tx_uc_frames
            tc_values[tc_idx]['tx_mc_frames'] = tc_data[tc_idx].tx_mc_frames
            tc_values[tc_idx]['tx_bc_frames'] = tc_data[tc_idx].tx_bc_frames
            tc_values[tc_idx]['tx_queue']     = tc_data[tc_idx].tx_queue
            tc_values[tc_idx]['tx_no_buffer_discard_uc'] = tc_data[tc_idx].tx_no_buffer_discard_uc
            tc_values[tc_idx]['tx_wred_discard'] = tc_data[tc_idx].tx_wred_discard

        # populate the output data
        counter_info = {'port' : portid}
        counter_info['per_port'] = { 'labels' : port_labels,
                                     'values' : port_values }
        counter_info['perf']     = { 'labels' : perf_labels,
                                     'values' : perf_values }
        counter_info['ieee']     = { 'labels' : ieee_labels,
                                     'values' : ieee_values }
        counter_info['per_prio'] = { 'labels' : prio_labels,
                                     'values' : prio_values }
        counter_info['per_tc']   = { 'labels' : tc_labels,
                                     'values' : tc_values }

        return counter_info

    def get_port_discards(self, portid):
        discard_info = {}
        discard_p = new_sx_port_cntr_discard_t_p()

        rc = sx_api_port_counter_discard_get(self.handle,
                                             SX_ACCESS_CMD_READ,
                                             portid,
                                             discard_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_port_counter_discard_get failed'
            delete_sx_port_cntr_discard_t_p(discard_p)
            sys.stdout.close()
            exit(-1)

        data = sx_port_cntr_discard_t_p_value(discard_p)

        discard_info = {'port'                    : portid,
                        'ingress general'         : data.ingress_general,
                        'ingress policy engine'   : data.ingress_policy_engine,
                        'ingress vlan membership' : data.ingress_vlan_membership,
                        'ingress tag frame type'  : data.ingress_tag_frame_type,
                        'egress vlan membership'  : data.egress_vlan_membership,
                        'loopback filter'         : data.loopback_filter,
                        'egress general'          : data.egress_general,
                        'egress link down'        : data.egress_link_down,
                        'egress hoq'              : data.egress_hoq,
                        'port isolation'          : data.port_isolation,
                        'egress policy engine'    : data.egress_policy_engine }

        delete_sx_port_cntr_discard_t_p(discard_p)
        return discard_info

    def get_port_pkt_size_counters(self, portid):
        counter_info = {}
        cntr_p = new_sx_port_cntr_rfc_2819_t_p()

        rc = sx_api_port_counter_rfc_2819_get(self.handle, SX_ACCESS_CMD_READ, portid,
                                         cntr_p)

        if rc != SX_STATUS_SUCCESS:
            delete_sx_port_cntr_rfc_2819_t_p(cntr_p)
            sys.stdout.close()
            exit(-1)

        data = sx_port_cntr_rfc_2819_t_p_value(cntr_p)

        counter_info = {'port': portid,
                        'pkts_64': data.ether_stats_pkts64octets,
                        'pkts_65_to_127': data.ether_stats_pkts65to127octets,
                        'pkts_128_to_255': data.ether_stats_pkts128to255octets,
                        'pkts_256_to_511': data.ether_stats_pkts256to511octets,
                        'pkts_512_to_1023': data.ether_stats_pkts512to1023octets,
                        'pkts_1024_to_1518': data.ether_stats_pkts1024to1518octets,
                        'pkts_1519_to_2047': data.ether_stats_pkts1519to2047octets,
                        'pkts_2048_to_4095': data.ether_stats_pkts2048to4095octets,
                        'pkts_4096_to_8191': data.ether_stats_pkts4096to8191octets,
                        'pkts_8192_to_10239': data.ether_stats_pkts8192to10239octets}

        delete_sx_port_cntr_rfc_2819_t_p(cntr_p)

        return counter_info

    def _set_port_buffer_param_item(self, param_list, param_idx, log_port_list, param_type) :
        param_item = sx_port_statistic_usage_params_t_arr_getitem(param_list, param_idx)
        param_item.log_port_list_p  = log_port_list
        param_item.port_cnt         = 1
        param_item.sx_port_params.port_params_type = param_type
        m_items = {}

        if param_type == SX_COS_INGRESS_PORT_ATTR_E:
            param_item.sx_port_params.port_params_cnt  = 4
            pool_list = new_sx_cos_pool_id_t_arr(4)
            for pool_id in range(4) :
                sx_cos_pool_id_t_arr_setitem(pool_list, pool_id, pool_id)
            param_item.sx_port_params.port_param.ingress_port_pool_list_p = pool_list
            m_items[param_type] = pool_list

        elif param_type == SX_COS_EGRESS_PORT_ATTR_E :
            param_item.sx_port_params.port_params_cnt  = 4
            pool_list = new_sx_cos_pool_id_t_arr(4)
            for pool_id in range(4,8) :
                print 'adding pool id %d to pool list' % pool_id
                sx_cos_pool_id_t_arr_setitem(pool_list, pool_id, pool_id)
            param_item.sx_port_params.port_param.egress_port_pool_list_p  = pool_list
            #print 'param list assigned'
            m_items[param_type] = pool_list
            
        elif param_type == SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E :
            param_item.sx_port_params.port_params_cnt  = 8
            pg_list = new_sx_cos_priority_group_t_arr(8)
            for pg in range(8) :
                sx_cos_priority_group_t_arr_setitem(pg_list, pg, pg)
            param_item.sx_port_params.port_param.port_pg_list_p = pg_list
            m_items[param_type] = pg_list

        elif param_type == SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E :
            param_item.sx_port_params.port_params_cnt  = 8
            tc_list = new_sx_cos_traffic_class_t_arr(8)
            for tc in range(8) :
                sx_cos_traffic_class_t_arr_setitem(tc_list, tc, tc)
            param_item.sx_port_params.port_param.port_tc_list_p = tc_list
            m_items[param_type] = tc_list

        elif param_type == SX_COS_MULTICAST_ATTR_E :
            param_item.sx_port_params.port_params_cnt  = 16
            prio_list = new_sx_cos_priority_t_arr(16)
            for prio in range(16) :
                sx_cos_traffic_class_t_arr_setitem(prio_list, prio, prio)
            param_item.sx_port_params.port_param.mc_switch_prio = prio_list
            m_items[param_type] = prio_list

        # copy the item into the list
        sx_port_statistic_usage_params_t_arr_setitem(param_list, param_idx, param_item)
        return param_list, m_items

    def _delete_port_buffer_param_items(self, m_items):
        for param_type in m_items:
            if param_type == SX_COS_INGRESS_PORT_ATTR_E:
                delete_sx_cos_pool_id_t_arr(m_items[param_type])
            elif param_type == SX_COS_EGRESS_PORT_ATTR_E:
                delete_sx_cos_pool_id_t_arr(m_items[param_type])
            elif param_type == SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E:
                delete_sx_cos_priority_group_t_arr(m_items[param_type])
            elif param_type == SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E:
                delete_sx_cos_traffic_class_t_arr(m_items[param_type])
            elif param_type == SX_COS_MULTICAST_ATTR_E:
                delete_sx_cos_priority_t_arr(m_items[param_type])

    def _get_port_occupancy_stats(self, port_id, counter_info, usage_counter) :

        counter_item = {'current'   : int(usage_counter.statistics.curr_occupancy),
                        'watermark' : int(usage_counter.statistics.watermark)}
        if port_id != usage_counter.log_port :
            return
        if usage_counter.sx_port_params.port_params_type == SX_COS_INGRESS_PORT_ATTR_E:
            counter_item['type']  = 'ingress port, pool'
            counter_item['index'] = int(usage_counter.sx_port_params.port_param.ingress_port_pool)
        elif usage_counter.sx_port_params.port_params_type == SX_COS_EGRESS_PORT_ATTR_E:
            counter_item['type']  = 'egress port, pool'
            counter_item['index'] = int(usage_counter.sx_port_params.port_param.egress_port_pool)
        elif usage_counter.sx_port_params.port_params_type == SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E :
            counter_item['type']  = 'priority_group'
            counter_item['index'] = int(usage_counter.sx_port_params.port_param.port_pg)
        elif usage_counter.sx_port_params.port_params_type == SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E :
            counter_item['type']  = 'traffic class'
            counter_item['index'] = int(usage_counter.sx_port_params.port_param.port_tc)
        elif usage_counter.sx_port_params.port_params_type == SX_COS_MULTICAST_ATTR_E :
            counter_item['type']  = 'MC switch priority'
            counter_item['index'] = int(usage_counter.sx_port_params.port_param.mc_switch_prio)
        elif usage_counter.sx_port_params.port_params_type == SX_COS_MULTICAST_PORT_ATTR_E :
            counter_item['type']  = 'MC per-port'
            counter_item['index'] = int(usage_counter.sx_port_params.port_param.mc_switch_prio)
        else :
            print 'usage counter type %d not recognized' % usage_counter.sx_port_params.port_params_type
        counter_info['counter_list'].append(counter_item)

    def get_port_buffer_counters(self, portid):
        counter_info = {'port': portid, 'counter_list' : []}
        log_port_list = new_sx_port_log_id_t_arr(1)
        sx_port_log_id_t_arr_setitem(log_port_list, 0, portid)
        log_port_id = sx_port_log_id_t_arr_getitem(log_port_list, 0)

        # configure the parameters for each buffer type
        param_type_list = [SX_COS_INGRESS_PORT_ATTR_E,
                           SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E,
                           #SX_COS_EGRESS_PORT_ATTR_E,
                           SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E]
        if portid == 0x40000000 :
            param_type_list = [SX_COS_MULTICAST_ATTR_E]

        param_list = new_sx_port_statistic_usage_params_t_arr(len(param_type_list))
        param_idx = 0
        for param_type in param_type_list:
            param_list, m_items = self._set_port_buffer_param_item(param_list, param_idx, log_port_list, param_type)
            new_param_item = sx_port_statistic_usage_params_t_arr_getitem(param_list, param_idx)
            param_idx += 1

        num_counters = 64
        num_counters_p  = new_uint32_t_p()
        uint32_t_p_assign(num_counters_p, num_counters)

        cntr_list = new_sx_port_occupancy_statistics_t_arr(num_counters)

        # call the API again to get the usage status
        rc = sx_api_cos_port_buff_type_statistic_get(self.handle,
                                                     SX_ACCESS_CMD_READ,
                                                     param_list,
                                                     len(param_type_list),
                                                     cntr_list,
                                                     num_counters_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_port_buf_type_statistic_get failed'
            self._delete_port_buffer_param_items(m_items)
            delete_sx_port_log_id_t_arr(log_port_list)
            delete_sx_port_statistic_usage_params_t_arr(param_list)
            delete_sx_port_occupancy_statistics_t_arr(cntr_list)
            delete_uint32_t_p(num_counters_p)
            return counter_info

        num_counters = uint32_t_p_value(num_counters_p)
        for idx in range(num_counters) :
            usage_counter = sx_port_occupancy_statistics_t_arr_getitem(cntr_list, idx)
            self._get_port_occupancy_stats(portid, counter_info, usage_counter)

        self._delete_port_buffer_param_items(m_items)
        delete_sx_port_log_id_t_arr(log_port_list)
        delete_sx_port_statistic_usage_params_t_arr(param_list)
        delete_sx_port_occupancy_statistics_t_arr(cntr_list)
        delete_uint32_t_p(num_counters_p)
        return counter_info

    def get_port_shared_buffer_limits(self,portid) :
        limit_info = []
        port_buffer_attr_cnt_p = new_uint32_t_p()
        uint32_t_p_assign(port_buffer_attr_cnt_p, 0)

        # read the pool buffer limits so we can calculate the shared buffer limits
        pool_buffer_limits  = self.get_pool_buffer_limits()

        # get the buffer attribute count
        rc = sx_api_cos_port_shared_buff_type_get(self.handle, portid, None, port_buffer_attr_cnt_p)
        if rc != 0:
            print 'sx_api_cos_port_shared_buff_type_get failed for port %d shared limits: rc %d' % (portid, rc)
            delete_uint32_t_p(port_buffer_attr_cnt_p)
            return limit_info

        # get the buffer attribute list
        attr_cnt = uint32_t_p_value(port_buffer_attr_cnt_p)
        port_buffer_attr_list_p = new_sx_cos_port_shared_buffer_attr_t_arr(attr_cnt)
        rc = sx_api_cos_port_shared_buff_type_get(self.handle, portid, port_buffer_attr_list_p, port_buffer_attr_cnt_p)
        if rc != 0:
            print 'sx_api_cos_port_shared_buff_type_get failed for port %d shared limits: rc %d' % (portid, rc)
            delete_uint32_t_p(port_buffer_attr_cnt_p)
            delete_sx_cos_port_shared_buffer_attr_t_arr(port_buffer_attr_list_p)
            return limit_info
        attr_cnt = uint32_t_p_value(port_buffer_attr_cnt_p)
        for attr_idx in range(0,attr_cnt):
            attr_item = sx_cos_port_shared_buffer_attr_t_arr_getitem(port_buffer_attr_list_p, attr_idx)
            if attr_item.type == SX_COS_INGRESS_PORT_ATTR_E:
                limit_info.append({'type'  : port_buffer_type_dict[attr_item.type],
                                   'mode'  : shared_pool_mode_dict[attr_item.attr.ingress_port_shared_buff_attr.max.mode],
                                   'basis_points' : attr_item.attr.ingress_port_shared_buff_attr.max.max.size,
                                   'pool'  : attr_item.attr.ingress_port_shared_buff_attr.pool_id })
            elif attr_item.type == SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E:
                limit_info.append({'type'  : port_buffer_type_dict[attr_item.type],
                                   'pg'    : attr_item.attr.ingress_port_pg_shared_buff_attr.pg,
                                   'mode'  : shared_pool_mode_dict[attr_item.attr.ingress_port_pg_shared_buff_attr.max.mode],
                                   'basis_points' : attr_item.attr.ingress_port_pg_shared_buff_attr.max.max.size,
                                   'pool'  : attr_item.attr.ingress_port_pg_shared_buff_attr.pool_id })
            elif attr_item.type == SX_COS_EGRESS_PORT_ATTR_E:
                limit_info.append({'type'  : port_buffer_type_dict[attr_item.type],
                                   'mode'  : shared_pool_mode_dict[attr_item.attr.egress_port_shared_buff_attr.max.mode],
                                   'basis_points' : attr_item.attr.egress_port_shared_buff_attr.max.max.size,
                                   'pool'  : attr_item.attr.egress_port_shared_buff_attr.pool_id })
            elif attr_item.type == SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E:
                limit_info.append({'type'  : port_buffer_type_dict[attr_item.type],
                                   'tc'    : attr_item.attr.egress_port_tc_shared_buff_attr.tc,
                                   'mode'  : shared_pool_mode_dict[attr_item.attr.egress_port_tc_shared_buff_attr.max.mode],
                                   'basis_points' : attr_item.attr.egress_port_tc_shared_buff_attr.max.max.size,
                                   'pool'  : attr_item.attr.egress_port_tc_shared_buff_attr.pool_id })
            elif attr_item.type == SX_COS_MULTICAST_ATTR_E:
                limit_info.append({'type'  : port_buffer_type_dict[attr_item.type],
                                   'sp'    : attr_item.attr.multicast_shared_buff_attr.sp,
                                   'mode'  : shared_pool_mode_dict[attr_item.attr.multicast_shared_buff_attr.max.mode],
                                   'basis_points' : attr_item.attr.multicast_shared_buff_attr.max.max.size,
                                   'pool'  : attr_item.attr.multicast_shared_buff_attr.pool_id })
            elif attr_item.type == SX_COS_MULTICAST_PORT_ATTR_E:
                limit_info.append({'type'  : port_buffer_type_dict[attr_item.type],
                                    'sp'   : '-',
                                   'mode'  : shared_pool_mode_dict[attr_item.attr.multicast_port_shared_buff_attr.max.mode],
                                   'basis_points' : attr_item.attr.multicast_port_shared_buff_attr.max.max.size,
                                   'pool'  : 4 })
            else :
                continue

            # convert the limit in basis points to the number of cells
            shared_cell_limit = 0
            basis_points      = float(limit_info[attr_idx]['basis_points'])
            if basis_points > 0:
                pool_cell_limit    = pool_buffer_limits[limit_info[attr_idx]['pool']]['cells']
                shared_cell_limit  = int(pool_cell_limit * (basis_points / 10000))
            limit_info[attr_idx]['cells'] = shared_cell_limit

        delete_uint32_t_p(port_buffer_attr_cnt_p)
        delete_sx_cos_port_shared_buffer_attr_t_arr(port_buffer_attr_list_p)
        return limit_info

    def get_port_reserved_buffer_limits(self,portid) :
        limit_info = []
        port_buffer_attr_cnt_p = new_uint32_t_p()
        uint32_t_p_assign(port_buffer_attr_cnt_p, 0)

        # get the buffer attribute count
        rc = sx_api_cos_port_buff_type_get(self.handle, portid,  None, port_buffer_attr_cnt_p)
        if rc != 0:
            print 'sx_api_cos_port_buff_type_get port %d reserved buffer count failed: rc %d' % (portid, rc)
            delete_uint32_t_p(port_buffer_attr_cnt_p)
            return limit_info

        # get the buffer attribute list
        attr_cnt = uint32_t_p_value(port_buffer_attr_cnt_p)
        port_buffer_attr_list_p = new_sx_cos_port_buffer_attr_t_arr(attr_cnt)
        rc = sx_api_cos_port_buff_type_get(self.handle,
                                           portid,
                                           port_buffer_attr_list_p,
                                           port_buffer_attr_cnt_p)
        if rc != 0:
            print 'sx_api_cos_port_buff_type_get port %d reserved buffer list failed for %d attributes: rc %d' % (portid,
                                                                                                                  attr_cnt,
                                                                                                                  rc)
            delete_uint32_t_p(port_buffer_attr_cnt_p)
            delete_sx_cos_port_buffer_attr_t_arr(port_buffer_attr_cnt_p)
            return limit_info
        attr_cnt = uint32_t_p_value(port_buffer_attr_cnt_p)
        for i in range(0,attr_cnt):
            attr_item = sx_cos_port_buffer_attr_t_arr_getitem(port_buffer_attr_list_p, i)
            if attr_item.type == SX_COS_INGRESS_PORT_ATTR_E:
                limit_info.append({'type'  : port_buffer_type_dict[attr_item.type],
                                   'cells' : attr_item.attr.ingress_port_buff_attr.size,
                                   'pool'  : attr_item.attr.ingress_port_buff_attr.pool_id })
            elif attr_item.type == SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E:
                limit_info.append({'type'  : port_buffer_type_dict[attr_item.type],
                                   'pg'    : attr_item.attr.ingress_port_pg_buff_attr.pg,
                                   'lossy' : attr_item.attr.ingress_port_pg_buff_attr.is_lossy,
                                   'xon'   : attr_item.attr.ingress_port_pg_buff_attr.xon,
                                   'xoff'  : attr_item.attr.ingress_port_pg_buff_attr.xoff,
                                   'cells' : attr_item.attr.ingress_port_pg_buff_attr.size,
                                   'pool'  : attr_item.attr.ingress_port_pg_buff_attr.pool_id })
            elif attr_item.type == SX_COS_EGRESS_PORT_ATTR_E:
                limit_info.append({'type'  : port_buffer_type_dict[attr_item.type],
                                   'cells' : attr_item.attr.egress_port_buff_attr.size,
                                   'pool'  : attr_item.attr.egress_port_buff_attr.pool_id })
            elif attr_item.type == SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E:
                limit_info.append({'type'  : port_buffer_type_dict[attr_item.type],
                                   'tc'    : attr_item.attr.egress_port_tc_buff_attr.tc,
                                   'cells' : attr_item.attr.egress_port_tc_buff_attr.size,
                                   'pool'  : attr_item.attr.egress_port_tc_buff_attr.pool_id })
            elif attr_item.type == SX_COS_MULTICAST_ATTR_E:
                limit_info.append({'type'  : port_buffer_type_dict[attr_item.type],
                                   'sp'    : attr_item.attr.multicast_buff_attr.sp,
                                   'cells' : attr_item.attr.multicast_buff_attr.size,
                                   'pool'  : attr_item.attr.multicast_buff_attr.pool_id })
            elif attr_item.type == SX_COS_MULTICAST_PORT_ATTR_E:
                limit_info.append({'type'  : port_buffer_type_dict[attr_item.type],
                                   'sp'    : '-',
                                   'cells' : attr_item.attr.multicast_port_buff_attr.size,
                                   'pool'  : 4 })
            else:
                continue

        delete_uint32_t_p(port_buffer_attr_cnt_p)
        delete_sx_cos_port_buffer_attr_t_arr(port_buffer_attr_list_p)
        return limit_info

    def create_reserved_buffer(self, port_id, pool_id, buffer_type, buffer_id, cell_count) :
        count = 1
        cmd   = SX_ACCESS_CMD_SET
        port_buffer_attr_list_p = new_sx_cos_port_buffer_attr_t_arr(count)
        attr_item = sx_cos_port_buffer_attr_t_arr_getitem(port_buffer_attr_list_p, 0)
        if buffer_type == 'pg reserved' :
            attr_item.type = SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E
            attr_item.attr.ingress_port_pg_buff_attr.size     = cell_count
            attr_item.attr.ingress_port_pg_buff_attr.pg       = buffer_id
            attr_item.attr.ingress_port_pg_buff_attr.pool_id  = pool_id
            attr_item.attr.ingress_port_pg_buff_attr.is_lossy = 1
            attr_item.attr.ingress_port_pg_buff_attr.xon      = 0
            attr_item.attr.ingress_port_pg_buff_attr.xoff     = 0
        elif buffer_type == 'tc reserved' :
            attr_item.type = SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E
            attr_item.attr.egress_port_tc_buff_attr.size    = cell_count
            attr_item.attr.egress_port_tc_buff_attr.tc      = buffer_id
            attr_item.attr.egress_port_tc_buff_attr.pool_id = pool_id
        elif buffer_type == 'multicast' :
            attr_item.type = SX_COS_MULTICAST_ATTR_E
            attr_item.attr.multicast_buff_attr.size    = cell_count
            attr_item.attr.multicast_buff_attr.sp      = buffer_id
            attr_item.attr.multicast_buff_attr.pool_id = mc_pool
        else :
            print 'buffer type %s not recognized' % buffer_type
            delete_sx_cos_port_buffer_attr_t_arr(port_buffer_attr_list_p)
            return
        
        rc = sx_api_cos_port_buff_type_set(self.handle, cmd, port_id, attr_item, count)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_cos_port_buff_type_set (create) failed for port 0x%x: %s[%d] %d cells: %d\n' % (port_id,
                                                                                                          buffer_type,
                                                                                                          buffer_id,
                                                                                                          cell_count,
                                                                                                          rc)
            delete_sx_cos_port_buffer_attr_t_arr(port_buffer_attr_list_p)
            sys.stdout.close()
            exit(-1)

    def get_reserved_buffer_list(self, port_id) :
        cmd   = SX_ACCESS_CMD_GET
        buffer_count_p = new_uint32_t_p()
        uint32_t_p_assign(buffer_count_p, 0)

        # get the buffer attribute count
        rc = sx_api_cos_port_buff_type_get(self.handle,
                                           port_id,
                                           None,
                                           buffer_count_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_cos_port_buff_type_get failed for port %d reserved buffer count: %d\n' % (port_id, rc)
            delete_uint32_t_p(buffer_count_p)
            sys.stdout.close()
            exit(-1)

        # get the buffer attribute list
        count = uint32_t_p_value(buffer_count_p)
        port_buffer_attr_list_p = new_sx_cos_port_buffer_attr_t_arr(count)
        rc = sx_api_cos_port_buff_type_get(self.handle,
                                           port_id,
                                           port_buffer_attr_list_p,
                                           buffer_count_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_cos_port_buff_type_get failed for port %d reserved buffer list, count %d: %d\n' % (port_id,
                                                                                                             count,
                                                                                                             rc)
            delete_uint32_t_p(buffer_count_p)
            delete_sx_cos_port_buffer_attr_t_arr(port_buffer_attr_list_p)
            sys.stdout.close()
            exit(-1)
        count = uint32_t_p_value(buffer_count_p)
        buffer_list  = []
        for idx in range(0,count) :
            port_buffer_attr_item = sx_cos_port_buffer_attr_t_arr_getitem(port_buffer_attr_list_p, idx)
            buffer_list.append(port_buffer_attr_item)

        delete_uint32_t_p(buffer_count_p)
        delete_sx_cos_port_buffer_attr_t_arr(port_buffer_attr_list_p)
        return buffer_list

    def delete_reserved_buffer(self, port_id, buffer_item) :
        count = 1
        cmd   = SX_ACCESS_CMD_DELETE
        port_buffer_attr_list_p = new_sx_cos_port_buffer_attr_t_arr(count)
        sx_cos_port_buffer_attr_t_arr_setitem(port_buffer_attr_list_p, 0, buffer_item)
        rc = sx_api_cos_port_buff_type_set(self.handle, cmd, port_id, port_buffer_attr_list_p, count)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_cos_port_buff_type_set (delete) failed for port 0x%x: %d\n' % (port_id, rc)
            sys.stdout.close()
            exit(-1)
        delete_sx_cos_port_buffer_attr_t_arr(port_buffer_attr_list_p)

    def get_shared_buffer_list(self, port_id) :
        cmd   = SX_ACCESS_CMD_GET
        count = 34
        buffer_count_p = new_uint32_t_p()
        uint32_t_p_assign(buffer_count_p, count)

        # get the buffer attribute count
        rc = sx_api_cos_port_shared_buff_type_get(self.handle,
                                                  port_id,
                                                  None,
                                                  buffer_count_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_cos_port_shared_buff_type_get failed for port %d attribute count: %d\n' % (port_id, rc)
            delete_sx_cos_port_shared_buffer_attr_t_arr(port_shared_buffer_attr_list_p)
            delete_uint32_t_p(buffer_count_p)
            sys.stdout.close()
            exit(-1)

        # get the buffer attribute list
        count = uint32_t_p_value(buffer_count_p)
        port_shared_buffer_attr_list_p = new_sx_cos_port_shared_buffer_attr_t_arr(count)
        rc = sx_api_cos_port_shared_buff_type_get(self.handle,
                                                  port_id,
                                                  port_shared_buffer_attr_list_p,
                                                  buffer_count_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_cos_port_shared_buff_type_get failed for port 0x%x, %d items: %d\n' % (port_id, count, rc)
            delete_sx_cos_port_shared_buffer_attr_t_arr(port_shared_buffer_attr_list_p)
            delete_uint32_t_p(buffer_count_p)
            sys.stdout.close()
            exit(-1)
        count = uint32_t_p_value(buffer_count_p)
        buffer_list  = []
        for idx in range(0,count) :
            attr_item = sx_cos_port_shared_buffer_attr_t_arr_getitem(port_shared_buffer_attr_list_p, idx)
            buffer_list.append(attr_item)

        delete_uint32_t_p(buffer_count_p)
        delete_sx_cos_port_shared_buffer_attr_t_arr(port_shared_buffer_attr_list_p)
        return buffer_list

    def delete_shared_buffer(self, port_id, buffer_item) :
        count = 1
        cmd   = SX_ACCESS_CMD_DELETE
        port_shared_buffer_attr_list_p = new_sx_cos_port_shared_buffer_attr_t_arr(count)
        sx_cos_port_shared_buffer_attr_t_arr_setitem(port_shared_buffer_attr_list_p, 0, buffer_item)
        rc = sx_api_cos_port_shared_buff_type_set(self.handle, cmd, port_id, port_shared_buffer_attr_list_p, count)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_cos_port_shared_buff_type_set failed: %d\n' % rc
            delete_sx_cos_port_shared_buffer_attr_t_arr(port_shared_buffer_attr_list_p)
            sys.stdout.close()
            exit(-1)
        delete_sx_cos_port_shared_buffer_attr_t_arr(port_shared_buffer_attr_list_p)

    def create_shared_buffer(self, port_id, pool_id, buffer_type, buffer_id, cell_count) :
        pass
        count = 1
        cmd   = SX_ACCESS_CMD_CREATE
        port_buffer_attr_list_p = new_sx_cos_port_buffer_attr_t_arr(count)
        attr_item = sx_cos_port_buffer_attr_t_arr_getitem(port_buffer_attr_list_p, 0)
        if buffer_type == 'pg_reserved' :
            attr_item.type = SX_COS_INGRESS_PORT_PRIORITY_GROUP_ATTR_E
            attr_item.attr.ingress_port_pg_buff_attr.size     = cell_count
            attr_item.attr.ingress_port_pg_buff_attr.pg       = buffer_id
            attr_item.attr.ingress_port_pg_buff_attr.pool_id  = pool_id
            attr_item.attr.ingress_port_pg_buff_attr.is_lossy = 1
            attr_item.attr.ingress_port_pg_buff_attr.xon      = 0
            attr_item.attr.ingress_port_pg_buff_attr.xoff     = 0
        elif buffer_type == 'tc reserved' :
            attr_item.type = SX_COS_EGRESS_PORT_TRAFFIC_CLASS_ATTR_E
            attr_item.attr.egress_port_tc_buff_attr.size    = cell_count
            attr_item.attr.egress_port_tc_buff_attr.tc      = buffer_id
            attr_item.attr.egress_port_tc_buff_attr.pool_id = pool_id
        elif buffer_type == 'multicast' :
            attr_item.type = SX_COS_MULTICAST_ATTR_E
            attr_item.attr.multicast_buff_attr.size    = cell_count
            attr_item.attr.multicast_buff_attr.sp      = buffer_id
            attr_item.attr.multicast_buff_attr.pool_id = mc_pool
        else :
            print 'buffer type %d not recognized' % buffer_type
            delete_sx_cos_port_buffer_attr_t_arr(port_buffer_attr_list_p)
            return
        
        rc = sx_api_cos_port_buff_type_set(self.handle, cmd, port_id, attr_item, count)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_cos_port_buff_type_set (create) failed for port 0x%x: %d\n' % (port_id, rc)
            delete_sx_cos_port_buffer_attr_t_arr(port_buffer_attr_list_p)
            sys.stdout.close()
            exit(-1)

        delete_sx_cos_port_buffer_attr_t_arr(port_buffer_attr_list_p)

    def delete_pool_buffer(self, pool_id) :
        cpu_pool = 7
        cmd      = SX_ACCESS_CMD_DESTROY
        if pool_id == cpu_pool :
            return
        pool_id_p = new_uint32_t_p()
        uint32_t_p_assign(pool_id_p, pool_id)
        pool_attr_p = new_sx_cos_pool_attr_t_p()
        rc = sx_api_cos_shared_buff_pool_set(self.handle, cmd, pool_attr_p, pool_id_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_cos_shared_buff_pool_set (delete) failed for pool %d: %d\n' % (pool_id, rc)
            delete_uint32_t_p(pool_id_p)
            delete_sx_cos_pool_attr_t_p(pool_attr_p)
            sys.stdout.close()
            exit(-1)
        delete_uint32_t_p(pool_id_p)
        delete_sx_cos_pool_attr_t_p(pool_attr_p)

    def create_pool_buffer(self, pool_id, mode, cell_count) :
        cpu_pool = 7
        cmd      = SX_ACCESS_CMD_CREATE
        count    = 1
        if pool_id == cpu_pool:
            print 'skipping CPU pool create'
            return
        direction = SX_COS_PORT_BUFF_POOL_DIRECTION_INGRESS_E
        if pool_id > 3 :
            direction = SX_COS_PORT_BUFF_POOL_DIRECTION_EGRESS_E
        pool_id_p = new_uint32_t_p()
        uint32_t_p_assign(pool_id_p, pool_id)
        pool_attr_p = new_sx_cos_pool_attr_t_arr(count)
        pool_attr   = sx_cos_pool_attr_t_arr_getitem(pool_attr_p, 0)
        pool_attr.direction = direction
        pool_attr.pool_size = cell_count;
        if mode == 0 :
            pool_attr.mode = SX_COS_BUFFER_MAX_MODE_STATIC_E
        elif mode == 1 :
            pool_attr.mode = SX_COS_BUFFER_MAX_MODE_DYNAMIC_E
        else :
            print 'mode %d not recognized' % mode
            delete_uint32_t_p(pool_id_p)
            delete_sx_cos_pool_attr_t_p(pool_attr_p)
            return
        rc = sx_api_cos_shared_buff_pool_set(self.handle, cmd, pool_attr_p, pool_id_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_cos_shared_buff_pool_set failed (create) for pool %d, size %d: %d\n' % (pool_id, cell_count, rc)
            delete_uint32_t_p(pool_id_p)
            delete_sx_cos_pool_attr_t_p(pool_attr_p)
            sys.stdout.close()
            exit(-1)
        #print 'created pool %d with %d cells in mode %d' % (pool_id, cell_count, mode)
        delete_uint32_t_p(pool_id_p)
        delete_sx_cos_pool_attr_t_p(pool_attr_p)

    def get_pool_list(self) :
        cmd          = SX_ACCESS_CMD_GET
        pool_list    = []
        count        = 8
        pool_count_p = new_uint32_t_p()
        uint32_t_p_assign(pool_count_p, count)
        pool_list_p = new_sx_cos_pool_id_t_arr(count)
        rc = sx_api_cos_pools_list_get(self.handle,
                                       pool_count_p,
                                       pool_list_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_cos_pools_list_get failed for %d items: %d\n' % (count, rc)
            delete_uint32_t_p(pool_count_p)
            delete_sx_cos_pool_id_t_arr(pool_list_p)
            return pool_list
        count = uint32_t_p_value(pool_count_p)

        for idx in range(0,count) :
            pool_id = sx_cos_pool_id_t_arr_getitem(pool_list_p, idx)
            pool_list.append(pool_id)

        delete_uint32_t_p(pool_count_p)
        delete_sx_cos_pool_id_t_arr(pool_list_p)
        return pool_list

    def get_pool_buffer_counters(self):
        buffer_info = {}
        pool_id_list   = self.get_pool_list()
        num_pools      = len(pool_id_list)
        pool_id_list_p = new_sx_cos_pool_id_t_arr(num_pools)
        usage_list_p   = new_sx_cos_pool_occupancy_statistics_t_arr(len(pool_id_list))
        for pool_idx,pool_id in enumerate(pool_id_list) :
            attr_item = sx_cos_pool_id_t_arr_setitem(pool_id_list_p, pool_idx, pool_id)
        rc = sx_api_cos_pool_statistic_get(self.handle,
                                           SX_ACCESS_CMD_READ,
                                           pool_id_list_p,
                                           num_pools,
                                           usage_list_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_cos_pool_statistic_get failed'
            delete_sx_cos_pool_id_t_arr(pool_id_list_p)
            delete_sx_cos_pool_occupancy_statistics_t_arr(usage_list_p)
            return buffer_info

        for pool_idx,pool_id in enumerate(pool_id_list) :
            usage_stat = sx_cos_pool_occupancy_statistics_t_arr_getitem(usage_list_p, pool_idx)
            buffer_info[pool_idx] = {}
            buffer_info[pool_idx]['id']        = pool_id
            buffer_info[pool_idx]['type']      = pool_label[pool_id]
            buffer_info[pool_idx]['current']   = int(usage_stat.statistics.curr_occupancy)
            buffer_info[pool_idx]['watermark'] = int(usage_stat.statistics.watermark)

        delete_sx_cos_pool_id_t_arr(pool_id_list_p)
        delete_sx_cos_pool_occupancy_statistics_t_arr(usage_list_p)
        return buffer_info

    def get_free_buffer_counters(self):
        buffer_status_p = new_sx_buffer_status_t_p()

        rc = sx_api_cos_buff_status_get(self.handle, buffer_status_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_cos_buff_status_get_failed: %d\n' % rc
            delete_sx_buffer_status_t_p(buffer_status_p)
            return {'none'}
        buffer_status = sx_buffer_status_t_p_value(buffer_status_p)
        
        delete_sx_buffer_status_t_p(buffer_status_p)
        return {'ingress' : buffer_status.free_size_ingress,
                'egress'  : buffer_status.free_size_egress }

    def get_pool_buffer_limits(self):

        buffer_info  = {}
        pool_id_list = self.get_pool_list()
        pool_id_p    = new_uint32_t_p()
        for pool_id in pool_id_list :
            uint32_t_p_assign(pool_id_p, pool_id)
            shared_pool_attr_p = new_sx_cos_pool_attr_t_arr(1)
            rc = sx_api_cos_shared_buff_pool_get(self.handle, pool_id, shared_pool_attr_p)
            if rc != 0:
                print 'sx_api_cos_shared_buff_pool_get failed for pool %d: %d\n' % (pool_id, rc)
                delete_sx_cos_pool_attr_t_arr(shared_pool_attr_p)
                break
            pool_attr_item = sx_cos_pool_attr_t_arr_getitem(shared_pool_attr_p, 0)
            buffer_info[pool_id] = {}
            buffer_info[pool_id]['type']  = pool_label[pool_id]
            buffer_info[pool_id]['mode']  = shared_pool_mode_dict[pool_attr_item.mode]
            buffer_info[pool_id]['cells'] = pool_attr_item.pool_size
            delete_sx_cos_pool_attr_t_arr(shared_pool_attr_p)

        delete_uint32_t_p(pool_id_p)
        return buffer_info

    def get_port_priority_trust(self,portid):
        trust_info = {'dscp' : 'no', 'pcpdei' : 'no'}
        trust_level_p = new_sx_cos_trust_level_t_p()
        rc = sx_api_cos_port_trust_get(self.handle, portid, trust_level_p)
        if rc != 0:
            print 'sx_api_cos_port_trust_get failed: rc %d' % rc
            delete_sx_cos_trust_level_t_p(trust_level_p)
            return trust_info
        trust_level = sx_cos_trust_level_t_p_value(trust_level_p)
        if trust_level == SX_COS_TRUST_LEVEL_L2 or trust_level == SX_COS_TRUST_LEVEL_BOTH:
            trust_info['pcpdei'] = 'yes'
        if trust_level == SX_COS_TRUST_LEVEL_L3 or trust_level == SX_COS_TRUST_LEVEL_BOTH:
            trust_info['dscp'] = 'yes'

        delete_sx_cos_trust_level_t_p(trust_level_p)
        return trust_info

    def get_port_priority_rewrite(self,portid) :
        rewrite_info = {'dscp' : 'no', 'pcpdei' : 'no', 'exp': 'no'}
        rewrite_enable_p = new_sx_cos_rewrite_enable_t_p()
        rc = sx_api_cos_port_rewrite_enable_get(self.handle, portid, rewrite_enable_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_cos_port_rewrite_enable_get failed: rc %d' % rc
            delete_sx_cos_rewrite_enable_t_p(rewrite_enable_p)
            return rewrite_info
        rewrite_enable = sx_cos_rewrite_enable_t_p_value(rewrite_enable_p)
        if rewrite_enable.rewrite_pcp_dei:
            rewrite_info['pcpdei'] = 'yes'
        if rewrite_enable.rewrite_dscp:
            rewrite_info['dscp'] = 'yes'
            if rewrite_enable.rewrite_exp:
                rewrite_info['exp'] = 'yes'
                
        delete_sx_cos_rewrite_enable_t_p(rewrite_enable_p)
        return rewrite_info

    def get_pg_prio_map(self,portid) :
        pg_list = []
        pg_p = new_sx_cos_port_prio_buff_t_p()
        rc = sx_api_cos_port_prio_buff_map_get(self.handle,
                                               portid,
                                               pg_p)
        if rc != 0:
            print 'sx_api_cos_port_prio_buf_map_get failed: rc %d' % rc
            delete_sx_cos_port_prio_buff_t_p(pg_p)
            return pg_list

        for cos_id in range(0,switch_priority_count):
            pg = sx_cos_port_buff_t_arr_getitem(pg_p.prio_to_buff,cos_id)
            pg_list.append({'sp' : cos_id,
                            'pg' : pg })
        delete_sx_cos_port_prio_buff_t_p(pg_p)
        return pg_list

    def get_pcpdei_prio_map(self, portid) :
        prio_list = []
        element_count = 16;
        element_count_p = new_uint32_t_p()
        uint32_t_p_assign(element_count_p, element_count)
        pcpdei_p = new_sx_cos_pcp_dei_t_arr(element_count)
        switch_priority_p = new_sx_cos_priority_color_t_arr(element_count)
        rc = sx_api_cos_port_pcpdei_to_prio_get(self.handle,
                                                portid,
                                                pcpdei_p,
                                                switch_priority_p,
                                                element_count_p)
        if rc != 0:
            print 'sx_api_cos_pcp_dei_to_prio_get failed: rc %d' % rc
            delete_uint32_t_p(element_count_p)
            delete_sx_cos_pcp_dei_t_arr(pcpdei_p)
            delete_sx_cos_priority_color_t_arr(switch_priority_p)
            return prio_list
        element_count = uint32_t_p_value(element_count_p)
        for idx in range(0,element_count):
            pcpdei          = sx_cos_pcp_dei_t_arr_getitem(pcpdei_p, idx)
            switch_priority = sx_cos_priority_color_t_arr_getitem(switch_priority_p, idx)
            prio_list.append({'pcp'   : pcpdei.pcp,
                              'dei'   : pcpdei.dei,
                              'sp'    : switch_priority.priority,
                              'color' : switch_priority.color })

        delete_uint32_t_p(element_count_p)
        delete_sx_cos_pcp_dei_t_arr(pcpdei_p)
        delete_sx_cos_priority_color_t_arr(switch_priority_p)
        return prio_list

    def get_pcpdei_rewrite_map(self, portid) :
        pcpdei_list = []
        element_count = 48;
        element_count_p = new_uint32_t_p()
        uint32_t_p_assign(element_count_p, element_count)
        pcpdei_p = new_sx_cos_pcp_dei_t_arr(element_count)
        switch_priority_p = new_sx_cos_priority_color_t_arr(element_count)
        rc = sx_api_cos_port_prio_to_pcpdei_rewrite_get(self.handle,
                                                        portid,
                                                        switch_priority_p,
                                                        pcpdei_p,
                                                        element_count_p)
        if rc != 0:
            print 'sx_api_cos_port_prio_to_pcpdei_rewrite_get failed: rc %d' % rc
            delete_uint32_t_p(element_count_p)
            delete_sx_cos_pcp_dei_t_arr(pcpdei_p)
            delete_sx_cos_priority_color_t_arr(switch_priority_p)
            return pcpdei_list
        element_count = uint32_t_p_value(element_count_p)
        for idx in range(0,element_count):
            pcpdei          = sx_cos_pcp_dei_t_arr_getitem(pcpdei_p, idx)
            switch_priority = sx_cos_priority_color_t_arr_getitem(switch_priority_p, idx)
            pcpdei_list.append({'pcp'   : pcpdei.pcp,
                                'dei'   : pcpdei.dei,
                                'sp'    : switch_priority.priority,
                                'color' : switch_priority.color })

        delete_uint32_t_p(element_count_p)
        delete_sx_cos_pcp_dei_t_arr(pcpdei_p)
        delete_sx_cos_priority_color_t_arr(switch_priority_p)
        return pcpdei_list

    def get_dscp_prio_map(self, portid) :
        prio_list = []
        element_count = 64
        element_count_p = new_uint32_t_p()
        uint32_t_p_assign(element_count_p, element_count)
        dscp_p = new_sx_cos_dscp_t_arr(element_count)
        switch_priority_p = new_sx_cos_priority_color_t_arr(element_count)
        rc = sx_api_cos_port_dscp_to_prio_get(self.handle,
                                              portid,
                                              dscp_p,
                                              switch_priority_p,
                                              element_count_p)
        if rc != 0:
            print 'sx_api_cos_dscp_to_prio_get failed: rc %d' % rc
            delete_uint32_t_p(element_count_p)
            delete_sx_cos_dscp_t_arr(dscp_p)
            delete_sx_cos_priority_color_t_arr(switch_priority_p)
            return prio_list
        element_count = uint32_t_p_value(element_count_p)
        for idx in range(0,element_count):
            dscp            = sx_cos_dscp_t_arr_getitem(dscp_p, idx)
            switch_priority = sx_cos_priority_color_t_arr_getitem(switch_priority_p, idx)
            prio_list.append({'dscp'   : dscp,
                              'sp'     : switch_priority.priority,
                              'color'  : switch_priority.color })

        delete_uint32_t_p(element_count_p)
        delete_sx_cos_dscp_t_arr(dscp_p)
        delete_sx_cos_priority_color_t_arr(switch_priority_p)
        return prio_list

    def get_dscp_rewrite_map(self, portid):
        dscp_list = []
        element_count = 64
        element_count_p = new_uint32_t_p()
        uint32_t_p_assign(element_count_p, element_count)
        dscp_p = new_sx_cos_dscp_t_arr(element_count)
        switch_priority_p = new_sx_cos_priority_color_t_arr(element_count)
        rc = sx_api_cos_port_prio_to_dscp_rewrite_get(self.handle,
                                                      portid,
                                                      switch_priority_p,
                                                      dscp_p,
                                                      element_count_p)
        if rc != 0:
            print 'sx_api_cos_port_prio_to_dscp_get failed: rc %d' % rc
            delete_uint32_t_p(element_count_p)
            delete_sx_cos_dscp_t_arr(dscp_p)
            delete_sx_cos_priority_color_t_arr(switch_priority_p)
            return dscp_list
        element_count = uint32_t_p_value(element_count_p)
        for idx in range(0,element_count):
            switch_priority = sx_cos_priority_color_t_arr_getitem(switch_priority_p, idx)
            dscp            = sx_cos_dscp_t_arr_getitem(dscp_p, idx)
            dscp_list.append({'dscp'   : dscp,
                              'sp'     : switch_priority.priority,
                              'color'  : switch_priority.color })

        delete_uint32_t_p(element_count_p)
        delete_sx_cos_dscp_t_arr(dscp_p)
        delete_sx_cos_priority_color_t_arr(switch_priority_p)
        return dscp_list

    def get_prio_ieee_map(self) :
        ieee_list = []
        element_count = switch_priority_count
        element_count_p = new_uint32_t_p()
        uint32_t_p_assign(element_count_p, element_count)
        ieee_p = new_sx_cos_ieee_prio_t_arr(element_count)
        switch_priority_p = new_sx_cos_priority_t_arr(element_count)
        rc = sx_api_cos_prio_to_ieeeprio_get(self.handle,
                                             switch_priority_p,
                                             ieee_p,
                                             element_count_p)
        if rc != 0:
            print 'sx_api_cos_prio_to_ieeeprio_get failed: rc %d' % rc
            delete_uint32_t_p(element_count_p)
            delete_sx_cos_ieee_prio_t_arr(ieee_p)
            delete_sx_cos_priority_t_arr(switch_priority_p)
            return ieee_list
        
        element_count = uint32_t_p_value(element_count_p)
        for idx in range(0,element_count):
            switch_priority = sx_cos_priority_t_arr_getitem(switch_priority_p, idx)
            ieee            = sx_cos_ieee_prio_t_arr_getitem(ieee_p, idx)
            ieee_list.append({'ieee'   : ieee,
                              'sp'     : switch_priority })

        delete_uint32_t_p(element_count_p)
        delete_sx_cos_ieee_prio_t_arr(ieee_p)
        delete_sx_cos_priority_t_arr(switch_priority_p)
        return ieee_list
        
    def get_prio_tc_map(self, portid) :
        tc_list = []
        tc_p = new_sx_cos_traffic_class_t_p()
        for priority in range(0,switch_priority_count) :
            rc =  sx_api_cos_port_tc_prio_map_get(self.handle,
                                                  portid,
                                                  priority,
                                                  tc_p);
            if rc != 0:
                print 'sx_api_cos_port_tc_prio_map_get failed for switch priority %d: rc %d' % (priority, rc)
                delete_sx_cos_traffic_class_t_p(tc_p)
                return tc_list

            tc = sx_cos_traffic_class_t_p_value(tc_p)
            tc_list.append({'tc' : tc,
                            'sp' : priority })
        delete_sx_cos_traffic_class_t_p(tc_p)
        return tc_list

    def get_mc_aware(self, portid):
        mc_aware_dict = {'mc_aware' : 'unknown' }
        mc_aware_p = new_boolean_t_p()
        rc =  sx_api_cos_port_tc_mcaware_get(self.handle, portid, mc_aware_p)
        if rc != 0:
            print 'sx_api_cos_port_tc_mcaware_get failed: rc %d' % rc
            delete_boolean_t_p(mc_aware_p)
            return mc_aware_dict

        mc_aware = boolean_t_p_value(mc_aware_p)
        if mc_aware :
            mc_aware_dict['mc_aware'] = 'yes'
        else:
            mc_aware_dict['mc_aware'] = 'no'
        delete_boolean_t_p(mc_aware_p)
        return mc_aware_dict

    def get_port_scheduler(self, portid) :
        element_list = []
        element_count = 128;
        element_count_p = new_uint32_t_p()
        uint32_t_p_assign(element_count_p, element_count)
        element_p = new_sx_cos_ets_element_config_t_arr(element_count)

        rc = sx_api_cos_port_ets_element_get(self.handle,
                                             portid,
                                             element_p,
                                             element_count_p)
        if rc != 0:
            print 'sx_api_cos_port_ets_element_get failed: rc %d' % rc
            delete_uint32_t_p(element_count_p)
            delete_sx_cos_ets_element_config_t_arr(element_p)
            return element_list

        element_count = uint32_t_p_value(element_count_p)
        for idx in range(0,element_count) :
            element = sx_cos_ets_element_config_t_arr_getitem(element_p, idx)
            dwrr_mode = 'dwrr'
            if element.dwrr == 0 :
                dwrr_mode = 'strict priority'
            min_shaper_mode = 'bytes'
            if element.packets_mode == 1 :
                min_shaper_mode = 'packets'
            element_list.append({'level' : element.element_hierarchy,
                                 'index' : element.element_index,
                                 'parent_index'      : element.next_element_index,
                                 'min_shaper_enable' : element.min_shaper_enable,
                                 'min_shaper_rate'   : element.min_shaper_rate,
                                 'min_shaper_mode'   : min_shaper_mode,
                                 'dwrr_enable'       : element.dwrr_enable,
                                 'dwrr_mode'         : dwrr_mode,
                                 'dwrr_weight'       : element.dwrr_weight})

        delete_uint32_t_p(element_count_p)
        delete_sx_cos_ets_element_config_t_arr(element_p)
        return element_list

    def get_flow_control(self, portid) :
        fc_dict   = {}
        fc_mode_p = new_sx_port_flow_ctrl_mode_t_p()
        rc = sx_api_port_global_fc_enable_get(self.handle,
                                              portid,
                                              fc_mode_p);
        if rc != 0:
            print 'sx_api_cos_port_global_fc_enable_get failed: rc %d' % rc
            delete_sx_port_flow_ctrl_mode_t_p(fc_mode_p)
            return fc_dict
        fc_mode = sx_port_flow_ctrl_mode_t_p_value(fc_mode_p)
        fc_dict['link_tx_enabled'] = 'no'
        fc_dict['link_rx_enabled'] = 'no'

        if fc_mode == SX_PORT_FLOW_CTRL_MODE_TX_EN_RX_DIS or \
           fc_mode == SX_PORT_FLOW_CTRL_MODE_TX_EN_RX_EN:
            fc_dict['link_tx_enabled'] = 'yes'
        if fc_mode == SX_PORT_FLOW_CTRL_MODE_TX_DIS_RX_EN or \
           fc_mode == SX_PORT_FLOW_CTRL_MODE_TX_EN_RX_EN:
            fc_dict['link_rx_enabled'] = 'yes'

        delete_sx_port_flow_ctrl_mode_t_p(fc_mode_p)

        fc_dict['pfc'] = {}
        pfc_mode_p = new_sx_port_flow_ctrl_mode_t_p()
        for priority in pfc_priority_list :
            rc = sx_api_port_pfc_enable_get(self.handle,
                                            portid,
                                            priority,
                                            pfc_mode_p);
            if rc != 0:
                print 'sx_api_cos_port_pfc_fc_enable_get failed: rc %d' % rc
                continue
            pfc_mode = sx_port_flow_ctrl_mode_t_p_value(pfc_mode_p)
            fc_dict['pfc'][priority] = { 'tx_enabled' : 'no',
                                         'rx_enabled' : 'no' }
            if pfc_mode == SX_PORT_FLOW_CTRL_MODE_TX_EN_RX_DIS or \
               pfc_mode == SX_PORT_FLOW_CTRL_MODE_TX_EN_RX_EN:
                fc_dict['pfc'][priority]['tx_enabled'] = 'yes'
            if pfc_mode == SX_PORT_FLOW_CTRL_MODE_TX_DIS_RX_EN or \
               pfc_mode == SX_PORT_FLOW_CTRL_MODE_TX_EN_RX_EN:
                fc_dict['pfc'][priority]['rx_enabled'] = 'yes'
                
        delete_sx_port_flow_ctrl_mode_t_p(pfc_mode_p)
        return fc_dict
        
    def clear_port_counters(self, portid, group_type=63):
        if group_type not in valid_group_types:
            sys.stdout.close()
            exit(-1)

        rc = sx_api_port_counter_clear_set(self.handle, portid, False, group_type)

        if rc != SX_STATUS_SUCCESS:
            sys.stdout.close()
            exit(-1)

        return

    def clear_all_port_counters(self, group_type=63):
        if group_type not in valid_group_types:
            sys.stdout.close()
            exit(-1)

        rc = sx_api_port_counter_clear_set(self.handle, 0, True, group_type)

        if rc != SX_STATUS_SUCCESS:
            sys.stdout.close()
            exit(-1)

        return

    def get_lag_member_ids(self, swid, logical_bond_id):
        member_ports = new_sx_port_log_id_t_arr(64)
        num_members = new_uint32_t_p()
        uint32_t_p_assign(num_members, 64)

        for i in range(64):
            sx_port_log_id_t_arr_setitem(member_ports, i, 0)

        rc = sx_api_lag_port_group_get(self.handle, swid, logical_bond_id,
                                       member_ports, num_members)

        if rc != SX_STATUS_SUCCESS:
            delete_sx_port_log_id_t_arr(member_ports)
            delete_uint32_t_p(num_members)
            sys.stdout.close()
            exit(-1)

        logical_port_ids = []
        for port_idx in range(uint32_t_p_value(num_members)):
            port_id = sx_port_log_id_t_arr_getitem(member_ports, port_idx)
            logical_port_ids.append(port_id)

        delete_sx_port_log_id_t_arr(member_ports)
        delete_uint32_t_p(num_members)

        return logical_port_ids

    def get_vport_port(self, vport) :
        vlan_id_p      = new_sx_vlan_id_t_p()
        logical_port_p = new_sx_port_log_id_t_p()
        rc = sx_api_port_vport_base_get(self.handle,
                                        vport,
                                        vlan_id_p,
                                        logical_port_p)
        if rc != SX_STATUS_SUCCESS:
            delete_sx_vlan_id_t_p(vlan_id_p)
            delete_sx_port_log_id_t_p(logical_port_p)
            print 'sx_api_port_vport_base_get failed using vport %d: %d' % (vport, rc)
            return 0

        logical_port = sx_port_log_id_t_p_value(logical_port_p)
        delete_sx_vlan_id_t_p(vlan_id_p)
        delete_sx_port_log_id_t_p(logical_port_p)
        return logical_port

    def get_lag_parameters(self):
        params = new_sx_lag_hash_param_t_p()

        rc = sx_api_lag_hash_flow_params_get(self.handle, params)

        if rc != SX_STATUS_SUCCESS:
            delete_sx_lag_hash_param_t_p(params)
            sys.stdout.close()
            exit(-1)

        lag_params = sx_lag_hash_param_t_p_value(params)
        lag_hash_params = {}

        lag_hash_params['type'] = lag_params.lag_hash_type
        lag_hash_params['flow_params'] = lag_params.lag_hash
        lag_hash_params['seed'] = lag_params.lag_seed

        delete_sx_lag_hash_param_t_p(params)

        return lag_hash_params

    def get_all_mstp_instances(self, swid):
        mstp_ids = []

        vlan_list = new_sx_vlan_id_t_arr(4095)
        num_vlans = new_uint32_t_p()

        for i in range(4095):
            uint32_t_p_assign(num_vlans, 4095)
            rc = sx_api_mstp_inst_vlan_list_get(self.handle, swid, i, vlan_list, num_vlans)

            if rc == SX_STATUS_SUCCESS:
                mstp_ids.append(i)

        delete_sx_vlan_id_t_arr(vlan_list)
        delete_uint32_t_p(num_vlans)

        return mstp_ids

    def get_mstp_instance_vlans(self, swid, mstp_inst):
        vlan_list = new_sx_vlan_id_t_arr(4095)
        num_vlans = new_uint32_t_p()
        uint32_t_p_assign(num_vlans, 4095)

        rc = sx_api_mstp_inst_vlan_list_get(self.handle, swid,
                                            mstp_inst, vlan_list, num_vlans)
        if rc != SX_STATUS_SUCCESS:
            delete_sx_vlan_id_t_arr(vlan_list)
            delete_uint32_t_p(num_vlans)
            sys.stdout.close()
            exit(-1)

        vlan_ids = []
        for i in range(uint32_t_p_value(num_vlans)):
            vlan_ids.append(sx_vlan_id_t_arr_getitem(vlan_list, i))

        delete_sx_vlan_id_t_arr(vlan_list)
        delete_uint32_t_p(num_vlans)

        return vlan_ids

    def get_mstp_mode(self, swid):
        mode = new_sx_mstp_mode_t_p()

        rc = sx_api_mstp_mode_get(self.handle, swid, mode)
        if rc != SX_STATUS_SUCCESS:
            sys.stdout.close()
            exit(-1)

        mode_value =  sx_mstp_mode_t_p_value(mode)

        delete_sx_mstp_mode_t_p(mode)
        return mode_value

    def get_mstp_port_state(self, swid, mstp_inst, port_id):
        # Get the MSTP mode
        mode = self.get_mstp_mode(swid)
        state = new_sx_mstp_inst_port_state_t_p()

        if mode == SX_MSTP_MODE_MSTP:
            rc = sx_api_mstp_inst_port_state_get(self.handle, swid,
                                                 mstp_inst, port_id, state)
        elif mode == SX_MSTP_MODE_RSTP:
            rc = sx_api_rstp_port_state_get(self.handle, port_id, state)

        port_state = sx_mstp_inst_port_state_t_p_value(state)
        delete_sx_mstp_inst_port_state_t_p(state)

        if port_state == SX_MSTP_INST_PORT_STATE_DISCARDING:
            return "discarding"
        elif port_state == SX_MSTP_INST_PORT_STATE_LEARNING:
            return "learning"
        elif port_state == SX_MSTP_INST_PORT_STATE_FORWARDING:
            return "forwarding"
        else:
            return "unknown"

    def get_port_ids(self, swid_list=[0]):
        port_id_dict = {}

        for swid in swid_list:
            # Get the number of ports
            num_ports_p = new_uint32_t_p()
            rc = sx_api_port_swid_port_list_get(self.handle, swid, None, num_ports_p)

            if rc != SX_STATUS_SUCCESS:
                delete_uint32_t_p(num_ports_p)
                sys.stdout.close()
                exit(-1)

            # Get the list of the port IDs
            num_ports = uint32_t_p_value(num_ports_p)
            if num_ports:
                port_list_p = new_sx_port_log_id_t_arr(num_ports)
                rc = sx_api_port_swid_port_list_get(self.handle, swid, port_list_p, num_ports_p)

                if rc != SX_STATUS_SUCCESS:
                    delete_sx_port_log_id_t_arr(port_list_p)
                    delete_uint32_t_p(num_ports_p)
                    sys.stdout.close()
                    exit(-1)

                # Format the dictionary to return to the caller:
                # { swid1 : [ portId1, portId2, ... ], swid2 : [ portId3, ... ], ... }
                port_ids = []
                for i in range(uint32_t_p_value(num_ports_p)):
                    port_id = sx_port_log_id_t_arr_getitem(port_list_p, i)
                    port_ids.append(port_id)
                port_id_dict[swid] = port_ids

                delete_sx_port_log_id_t_arr(port_list_p)

            delete_uint32_t_p(num_ports_p)

        return port_id_dict

    def get_port_isolate(self, swid_list=[0]):
        port_iso_dict = {}

        for swid in swid_list:
            # Get the number of ports
            num_ports_p = new_uint32_t_p()
            rc = sx_api_port_swid_port_list_get(self.handle, swid, None, num_ports_p)

            if rc != SX_STATUS_SUCCESS:
                delete_uint32_t_p(num_ports_p)
                sys.stdout.close()
                exit(-1)

            # Get the list of the port IDs
            num_ports = uint32_t_p_value(num_ports_p)
            if num_ports:
                port_list_p = new_sx_port_log_id_t_arr(num_ports)
                rc = sx_api_port_swid_port_list_get(self.handle, swid, port_list_p, num_ports_p)

                if rc != SX_STATUS_SUCCESS:
                    delete_sx_port_log_id_t_arr(port_list_p)
                    delete_uint32_t_p(num_ports_p)
                    sys.stdout.close()
                    exit(-1)

                # Get the port isolation for each port    
                for i in range(uint32_t_p_value(num_ports_p)):
                    port_id = sx_port_log_id_t_arr_getitem(port_list_p, i)

                    # Get the number of ports in the isolation group
                    log_port_cnt_p = new_uint32_t_p()
                    rc = sx_api_port_isolate_get(self.handle, port_id, None, log_port_cnt_p)

                    if rc != SX_STATUS_SUCCESS:
                        delete_uint32_t_p(log_port_cnt_p)
                        delete_sx_port_log_id_t_arr(port_list_p)
                        delete_uint32_t_p(num_ports_p)
                        sys.stdout.close()
                        exit(-1)

                    num_iso_ports = uint32_t_p_value(log_port_cnt_p)
                    if num_iso_ports:
                        log_port_list_p = new_sx_port_log_id_t_arr(num_iso_ports)
                        rc = sx_api_port_isolate_get(self.handle, port_id, log_port_list_p, log_port_cnt_p)

                        if rc != SX_STATUS_SUCCESS:
                            delete_sx_port_log_id_t_arr(log_port_list_p)
                            delete_uint32_t_p(log_port_cnt_p)
                            delete_sx_port_log_id_t_arr(port_list_p)
                            delete_uint32_t_p(num_ports_p)
                            sys.stdout.close()
                            exit(-1)

                        # Format the dictionary to return to the caller:
                        # { portid1 : [ iosportId1, isoportId2, ... ], portid2 : [ isoportId2, ... ], ... }
                        port_iso_dict[port_id] = []
                        for i in range(uint32_t_p_value(log_port_cnt_p)):
                            iso_port_id = sx_port_log_id_t_arr_getitem(log_port_list_p, i)
                            port_iso_dict[port_id].append(iso_port_id)

                        delete_sx_port_log_id_t_arr(log_port_list_p)

                    delete_uint32_t_p(log_port_cnt_p)

                delete_sx_port_log_id_t_arr(port_list_p)

            delete_uint32_t_p(num_ports_p)

        return port_iso_dict

    def set_port_isolate(self, portid, iso_port_list):
        num_iso_ports = len(iso_port_list)
        log_port_list_p = new_sx_port_log_id_t_arr(num_iso_ports)
        for i in range(num_iso_ports):
            sx_port_log_id_t_arr_setitem(log_port_list_p, i, iso_port_list[i])
        cmd = SX_ACCESS_CMD_SET if num_iso_ports else SX_ACCESS_CMD_DELETE_ALL
        rc = sx_api_port_isolate_set(self.handle, cmd, portid, log_port_list_p, num_iso_ports)

        if rc != SX_STATUS_SUCCESS:
            delete_sx_port_log_id_t_arr(log_port_list_p)
            sys.stdout.close()
            exit(-1)

        delete_sx_port_log_id_t_arr(log_port_list_p)

    def get_port_status(self, portid):
        oper_state_p = new_sx_port_oper_state_t_p()
        admin_state_p = new_sx_port_admin_state_t_p()
        module_state_p = new_sx_port_module_state_t_p()

        rc = sx_api_port_state_get(self.handle, portid, oper_state_p, admin_state_p, module_state_p)

        if rc != SX_STATUS_SUCCESS:
            delete_sx_port_oper_state_t_p(oper_state_p)
            delete_sx_port_admin_state_t_p(admin_state_p)
            delete_sx_port_module_state_t_p(module_state_p)
            sys.stdout.close()
            exit(-1)

        oper_state = "None"
        oper_state_enum = sx_port_oper_state_t_p_value(oper_state_p)
        if oper_state_enum == SX_PORT_OPER_STATUS_UP:
            oper_state = "Up"
        elif oper_state_enum == SX_PORT_OPER_STATUS_DOWN:
            oper_state = "Down"
        elif oper_state_enum == SX_PORT_OPER_STATUS_DOWN_BY_FAIL:
            oper_state = "Failed"

        delete_sx_port_oper_state_t_p(oper_state_p)
        delete_sx_port_admin_state_t_p(admin_state_p)
        delete_sx_port_module_state_t_p(module_state_p)
        return oper_state
    
    def get_port_mtu(self, portid):
        max_mtu_size_p = new_sx_port_mtu_t_p()
        oper_mtu_size_p = new_sx_port_mtu_t_p()
        sx_port_mtu_t_p_assign(max_mtu_size_p, 9000)
        rc = sx_api_port_mtu_get(self.handle,
                                 portid,
                                 max_mtu_size_p,
                                 oper_mtu_size_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_port_mtu_get failed'
            sys.stdout.close()
            exit(-1)
        mtu_value = sx_port_mtu_t_p_value(oper_mtu_size_p)
        return mtu_value

    def get_port_speed(self, portid):
        admin_speed_p = new_sx_port_speed_capability_t_p()
        oper_speed_p = new_sx_port_oper_speed_t_p()

        rc = sx_api_port_speed_get(self.handle, portid, admin_speed_p, oper_speed_p)

        if rc != SX_STATUS_SUCCESS:
            delete_sx_port_speed_capability_t_p(admin_speed_p)
            delete_sx_port_oper_speed_t_p(oper_speed_p)
            sys.stdout.close()
            exit(-1)

        oper_speed = 0
        oper_speed_enum = sx_port_oper_speed_t_p_value(oper_speed_p)
        if oper_speed_enum in [SX_PORT_SPEED_1GB_CX_SGMII, SX_PORT_SPEED_1GB_KX]:
            oper_speed = 1 * 1000 * 1000 * 1000
        elif oper_speed_enum in [SX_PORT_SPEED_10GB_CX4_XAUI, SX_PORT_SPEED_10GB_KX4, \
                                 SX_PORT_SPEED_10GB_KR, SX_PORT_SPEED_10GB_CR, \
                                 SX_PORT_SPEED_10GB_SR, SX_PORT_SPEED_10GB_ER_LR]:
            oper_speed = 10 * 1000 * 1000 * 1000
        elif oper_speed_enum in [SX_PORT_SPEED_20GB_KR2]:
            oper_speed = 20 * 1000 * 1000 * 1000
        elif oper_speed_enum in [SX_PORT_SPEED_25GB_CR, SX_PORT_SPEED_25GB_KR, \
                                 SX_PORT_SPEED_25GB_SR]:
            oper_speed = 25 * 1000 * 1000 * 1000
        elif oper_speed_enum in [SX_PORT_SPEED_40GB_CR4, SX_PORT_SPEED_40GB_KR4, \
                                 SX_PORT_SPEED_40GB_SR4, SX_PORT_SPEED_40GB_LR4_ER4]:
            oper_speed = 40 * 1000 * 1000 * 1000
        elif oper_speed_enum in [SX_PORT_SPEED_50GB_CR2, SX_PORT_SPEED_50GB_KR2]:
            oper_speed = 50 * 1000 * 1000 * 1000
        elif oper_speed_enum in [SX_PORT_SPEED_56GB_KR4, SX_PORT_SPEED_56GB_KX4]:
            oper_speed = 56 * 1000 * 1000 * 1000
        elif oper_speed_enum in [SX_PORT_SPEED_100GB_CR4, SX_PORT_SPEED_100GB_SR4, \
                                 SX_PORT_SPEED_100GB_KR4, SX_PORT_SPEED_100GB_LR4_ER4]:
            oper_speed = 100 * 1000 * 1000 * 1000

        delete_sx_port_speed_capability_t_p(admin_speed_p)
        delete_sx_port_oper_speed_t_p(oper_speed_p)
        return oper_speed

    def get_port_fec(self, portid):
        gig_speed = self.get_port_speed(portid) / (1000*1000*1000)
        if gig_speed == 10:
            phy_speed = SX_PORT_PHY_SPEED_10GB
        elif gig_speed == 25:
            phy_speed = SX_PORT_PHY_SPEED_25GB
        elif gig_speed == 40:
            phy_speed = SX_PORT_PHY_SPEED_40GB
        elif gig_speed == 50:
            phy_speed = SX_PORT_PHY_SPEED_50GB
        elif gig_speed == 100:
            phy_speed = SX_PORT_PHY_SPEED_100GB
        else:
            return "Unknown"

        admin_mode_p = new_sx_port_phy_mode_t_p()
        oper_mode_p = new_sx_port_phy_mode_t_p()

        rc = sx_api_port_phy_mode_get(self.handle, portid, phy_speed, admin_mode_p, oper_mode_p)

        if rc != SX_STATUS_SUCCESS:
            delete_sx_port_phy_mode_t_p(admin_mode_p)
            delete_sx_port_phy_mode_t_p(oper_mode_p)
            sys.stdout.close()
            exit(-1)

        oper_mode_enum = sx_port_phy_mode_t_p_value(oper_mode_p)
        if oper_mode_enum.fec_mode == SX_PORT_FEC_MODE_AUTO:
            fec_mode = "Auto"
        elif oper_mode_enum.fec_mode == SX_PORT_FEC_MODE_NONE:
            fec_mode = "None"
        elif oper_mode_enum.fec_mode == SX_PORT_FEC_MODE_FC:
            fec_mode = "FC"
        elif oper_mode_enum.fec_mode == SX_PORT_FEC_MODE_RS:
            fec_mode = "RS"
        else:
            fec_mode = "Other (0x%08x)" % (oper_mode_enum.fec_mode,)

        delete_sx_port_phy_mode_t_p(admin_mode_p)
        delete_sx_port_phy_mode_t_p(oper_mode_p)
        return fec_mode

    def set_port_sflow(self, portid, ing_rate, egr_rate=0, deviation=0, 
                       uc=True, mc=True, bc=True, uuc=True, umc=True):
        if egr_rate > 0:
            print 'sflow non-zero egr_rate not supported'
            sys.stdout.close()
            exit(-1)

        sflow_params_p = new_sx_port_sflow_params_t_p()
        sflow_params = sx_port_sflow_params_t_p_value(sflow_params_p)
        sflow_params.ratio = ing_rate
        sflow_params.deviation = deviation

        sflow_params.packet_types.uc = uc
        sflow_params.packet_types.mc = mc
        sflow_params.packet_types.bc = bc
        sflow_params.packet_types.uuc = uuc
        sflow_params.packet_types.umc = umc

        sx_port_sflow_params_t_p_assign(sflow_params_p, sflow_params)

        if sflow_params.ratio > 0:
            cmd = SX_ACCESS_CMD_ADD
        else:
            cmd = SX_ACCESS_CMD_DELETE

        rc = sx_api_port_sflow_set(self.handle,
                                   cmd,
                                   portid,
                                   sflow_params_p)
        if rc != SX_STATUS_SUCCESS:
            if cmd == SX_ACCESS_CMD_ADD and rc == SX_STATUS_ENTRY_ALREADY_EXISTS:
                rc = sx_api_port_sflow_set(self.handle,
                                           SX_ACCESS_CMD_EDIT,
                                           portid,
                                           sflow_params_p)
            elif cmd == SX_ACCESS_CMD_DELETE and rc == SX_STATUS_ENTRY_NOT_FOUND:
                rc = SX_STATUS_SUCCESS

        delete_sx_port_sflow_params_t_p(sflow_params_p)

        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_port_sflow_set failed'
            sys.stdout.close()
            exit(-1)

    def get_port_sflow(self, portid):
        sflow_params = {}
        sflow_params_p = new_sx_port_sflow_params_t_p()

        rc = sx_api_port_sflow_get(self.handle,
                                   portid,
                                   sflow_params_p)
        if rc != SX_STATUS_SUCCESS and rc != SX_STATUS_ENTRY_NOT_FOUND:
            print 'sx_api_port_sflow_get failed'
            delete_sx_port_sflow_params_t_p(sflow_params_p)
            sys.stdout.close()
            exit(-1)

        if rc == SX_STATUS_ENTRY_NOT_FOUND:
            sflow_params = {'port'         : portid,
                            'ing_rate'     : 0,
                            'egr_rate'     : 0,
                            'deviation'    : 0,
                            'uc'           : False,
                            'mc'           : False,
                            'bc'           : False,
                            'uuc'          : False,
                            'umc'          : False }
        else:
            data = sx_port_sflow_params_t_p_value(sflow_params_p)
            sflow_params = {'port'         : portid,
                            'ing_rate'     : data.ratio,
                            'egr_rate'     : 0,
                            'deviation'    : data.deviation,
                            'uc'           : data.packet_types.uc,
                            'mc'           : data.packet_types.mc,
                            'bc'           : data.packet_types.bc,
                            'uuc'          : data.packet_types.uuc,
                            'umc'          : data.packet_types.umc }

        delete_sx_port_sflow_params_t_p(sflow_params_p)
        return sflow_params

    def get_port_sflow_statistics(self, portid):
        sflow_statistics = {}
        sflow_statistics_p = new_sx_port_sflow_statistics_t_p()

        rc = sx_api_port_sflow_statistics_get(self.handle,
                                              SX_ACCESS_CMD_READ,
                                              portid,
                                              sflow_statistics_p)
        if rc != SX_STATUS_SUCCESS and rc != SX_STATUS_ENTRY_NOT_FOUND:
            print 'sx_api_port_sflow_statistics_get failed'
            delete_sx_port_sflow_statistics_t_p(sflow_statistics_p)
            sys.stdout.close()
            exit(-1)

        if rc == SX_STATUS_ENTRY_NOT_FOUND:
            sflow_statistics = {'port'              : portid,
                                'count_sample_drop' : 0 }
        else:
            data = sx_port_sflow_statistics_t_p_value(sflow_statistics_p)
            sflow_statistics = {'port'              : portid,
                                'count_sample_drop' : data.count_sample_drop }

        delete_sx_port_sflow_statistics_t_p(sflow_statistics_p)
        return sflow_statistics

    def clear_port_sflow_statistics(self, portid):
        rc = sx_api_port_sflow_statistics_get(self.handle,
                                              SX_ACCESS_CMD_CLEAR,
                                              portid,
                                              0)
        if rc != SX_STATUS_SUCCESS and rc != SX_STATUS_ENTRY_NOT_FOUND:
            print 'sx_api_port_sflow_statistics_get clear failed'
            sys.stdout.close()
            exit(-1)

    def get_port_sflow_sample_rate_max(self):
        return SX_PORT_SFLOW_SAMPLING_RATE_MAX


    def set_port_priority_rewrite(self,portid,dscp='no',pcpdei='no',exp='no') :
        rewrite_enable_p = new_sx_cos_rewrite_enable_t_p()
        rewrite_info = sx_cos_rewrite_enable_t_p_value(rewrite_enable_p)
        rewrite_info.rewrite_dscp = 1 if dscp == 'yes' else 0
        rewrite_info.rewrite_pcp_dei = 1 if pcpdei == 'yes' else 0
        rewrite_info.rewrite_exp = 1 if exp == 'yes' else 0
        rc = sx_api_cos_port_rewrite_enable_set(self.handle, portid, rewrite_info)
        delete_sx_cos_rewrite_enable_t_p(rewrite_enable_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_cos_port_rewrite_enable_set failed: rc %d' % rc
            sys.stdout.close()
            exit(-1)


    def set_port_dscp_remark(self, portid, priority, color, dscp):
        dscp_list = []
        element_count = 64
        element_count_p = new_uint32_t_p()
        uint32_t_p_assign(element_count_p, element_count)
        dscp_p = new_sx_cos_dscp_t_arr(element_count)
        switch_priority_p = new_sx_cos_priority_color_t_arr(element_count)
        rc = sx_api_cos_port_prio_to_dscp_rewrite_get(self.handle,
                                                      portid,
                                                      switch_priority_p,
                                                      dscp_p,
                                                      element_count_p)
        if rc != SX_STATUS_SUCCESS:
            print 'sx_api_cos_port_prio_to_dscp_get failed: rc %d' % rc
        else:
            element_count = 45
            for idx in range(0,element_count):
                sp = sx_cos_priority_color_t_arr_getitem(switch_priority_p, idx)
                if sp.priority == priority and sp.color == color:
                    sx_cos_dscp_t_arr_setitem(dscp_p, idx, dscp)
                    rc = sx_api_cos_port_prio_to_dscp_rewrite_set(self.handle,
                                                              portid,
                                                              switch_priority_p,
                                                              dscp_p,
                                                              element_count)
                    if rc != SX_STATUS_SUCCESS:
                        print 'sx_api_cos_port_prio_to_dscp_rewrite_set failed: rc %d' % rc
                    break


        delete_uint32_t_p(element_count_p)
        delete_sx_cos_dscp_t_arr(dscp_p)
        delete_sx_cos_priority_color_t_arr(switch_priority_p)

        if rc != SX_STATUS_SUCCESS:
            sys.stdout.close()
            exit(-1)

    def get_reg_recr_v2(self, portid):
        reg_recrv2 = sxd_api.ku_recr_v2_reg()
        meta = sxd_api.sxd_reg_meta_t()

        reg_recrv2.per_port_configuration = 0
        if portid != SX_INVALID_PORT:
            reg_recrv2.per_port_configuration = 1
            reg_recrv2.local_port = port_phy_id_get(portid)

        meta.dev_id = 1
        meta.swid = 0

        meta.access_cmd = sxd_api.SXD_ACCESS_CMD_GET
        rc = sxd_api.sxd_access_reg_recr_v2(reg_recrv2, meta, 1, None, None)
        if rc != SX_STATUS_SUCCESS:
            print 'sxd_access_reg_recr_v2 failed'
            sys.stdout.close()
            exit(-1)

        recrv2 = {'per_port_configuration'      : reg_recrv2.per_port_configuration,
                  'local_port'                  : reg_recrv2.local_port,
                  'symmetric_hash'              : reg_recrv2.symmetric_hash,
                  'hash_type'                   : reg_recrv2.hash_type,
                  'seed'                        : reg_recrv2.seed,
                  'general_fields'              : reg_recrv2.general_fields,
                  'outer_header_enables'        : reg_recrv2.outer_header_enables,
                  'outer_header_field_enables0' : uint32_t_arr_getitem(reg_recrv2.outer_header_field_enables, 0),
                  'outer_header_field_enables1' : uint32_t_arr_getitem(reg_recrv2.outer_header_field_enables, 1),
                  'outer_header_field_enables2' : uint32_t_arr_getitem(reg_recrv2.outer_header_field_enables, 2),
                  'outer_header_field_enables3' : uint32_t_arr_getitem(reg_recrv2.outer_header_field_enables, 3),
                  'outer_header_field_enables4' : uint32_t_arr_getitem(reg_recrv2.outer_header_field_enables, 4),
                  'inner_header_enables'        : reg_recrv2.inner_header_enables,
                  'inner_header_field_enables'  : reg_recrv2.inner_header_field_enables}

        return recrv2

class SX_Mgr (object) :
    def __init__(self) :
        pass

    def open_connection(self) :
        # Open the connection to the MLX SDK
        rc, self.handle = sx_api_open(None)
        if rc != SX_STATUS_SUCCESS:
            print "Error connection to the MLX SDK"
            sx_api_close(self.handle)
            exit()

        rc = sxd_api.sxd_access_reg_init(0, None, SX_VERBOSITY_LEVEL_ERROR)
        if (rc != SX_STATUS_SUCCESS):
            print "Failed to initializing register access."
            sx_api_close(self.handle)
            exit()

        self.cli = mlxSdk(self.handle)

    def close_connection(self) :
        sxd_api.sxd_access_reg_deinit()
        sx_api_close(self.handle)

sx_mgr = SX_Mgr()

def mlx_open_connection() :
    sx_mgr.open_connection()

def mlx_close_connection() :
    sx_mgr.close_connection()

def mlx_invalid_port_id() :
    return sx_mgr.cli.invalid_port_id()

def mlx_get_ecmp(ecmp_id) :
    return sx_mgr.cli.get_ecmp_table(ecmp_id)

def mlx_get_operational_ecmp(ecmp_id) :
    return sx_mgr.cli.get_operational_ecmp_table(ecmp_id)

def mlx_get_intf(rif=None) :
    return sx_mgr.cli.get_intf_table(rif)

def mlx_get_neighbor(version=4, rif=0) :
    return sx_mgr.cli.get_neighbor_table(version, rif)

def mlx_get_uc_route(version=4, vrid=None) :
    return sx_mgr.cli.get_uc_route_table(version, vrid)

def mlx_get_uc_route_count(version=4, vrid=None) :
    return sx_mgr.cli.get_uc_route_count(version, vrid)

def mlx_get_ecmp_attributes(ecmp_id) :
    return sx_mgr.cli.get_hw_ecmp_attributes(ecmp_id)

def mlx_get_mc_route(version=4, vrid= None) :
    return sx_mgr.cli.get_mc_route_table(version, vrid)

def mlx_get_fdb(swid=None, filter_type = (None, None)) :
    return sx_mgr.cli.get_fdb_table(swid, filter_type)

def mlx_get_fdb_age_time(swid_list = [0]) :
    return sx_mgr.cli.get_fdb_age_time(swid_list)

def mlx_set_fdb_age_time(fid, swid_list = [0]) :
    return sx_mgr.cli.set_fdb_age_time(fid, swid_list)

def mlx_flush_uc_fdb_all(swid_list = [0]) :
    return sx_mgr.cli.flush_uc_fdb_all(swid_list)

def mlx_flush_uc_fdb_fid(fid, swid_list = [0]) :
    return sx_mgr.cli.flush_uc_fdb_fid(fid, swid_list)

def mlx_flush_uc_fdb_log_port(log_port_id) :
    return sx_mgr.cli.flush_uc_fdb_log_port(log_port_id)

def mlx_get_vlan_ids(swid_list) :
    return sx_mgr.cli.get_vlan_ids(swid_list)

def mlx_get_bridge_vports(swid, br_vid):
    return sx_mgr.cli.get_bridge_vports(swid, br_vid)

def mlx_get_vlan_ports(swid, vids) :
    return sx_mgr.cli.get_vlan_ports(swid, vids)

def mlx_get_port_counters(portid) :
    return sx_mgr.cli.get_port_counters(portid)

def mlx_get_port_discards(portid) :
    return sx_mgr.cli.get_port_discards(portid)

def mlx_get_port_pkt_size_counters(portid) :
    return sx_mgr.cli.get_port_pkt_size_counters(portid)

def mlx_clear_port_counters(portid, group_type) :
    return sx_mgr.cli.clear_port_counters(portid, group_type=group_type)

def mlx_get_port_buffer_counters(portid) :
    return sx_mgr.cli.get_port_buffer_counters(portid)

def mlx_get_port_reserved_buffer_limits(portid) :
    return sx_mgr.cli.get_port_reserved_buffer_limits(portid)

def mlx_get_port_shared_buffer_limits(portid) :
    return sx_mgr.cli.get_port_shared_buffer_limits(portid)

def mlx_get_pool_buffer_limits() :
    return sx_mgr.cli.get_pool_buffer_limits()

def mlx_get_pool_buffer_counters() :
    return sx_mgr.cli.get_pool_buffer_counters()

def mlx_get_lag_member_ids(swid, logical_bond_id):
    return sx_mgr.cli.get_lag_member_ids(swid, logical_bond_id)

def mlx_get_lag_parameters():
    return sx_mgr.cli.get_lag_parameters()

def mlx_get_all_mstp_instances(swid):
    return sx_mgr.cli.get_all_mstp_instances(swid)

def mlx_get_mstp_instance_vlans(swid, mstp_inst):
    return sx_mgr.cli.get_mstp_instance_vlans(swid, mstp_inst)

def mlx_get_mstp_port_state(swid, mstp_inst, port_id):
    return sx_mgr.cli.get_mstp_port_state(swid, mstp_inst, port_id)

def mlx_get_port_ids(swid_list) :
    return sx_mgr.cli.get_port_ids(swid_list)

def mlx_get_port_isolate(swid_list) :
    return sx_mgr.cli.get_port_isolate(swid_list)

def mlx_set_port_isolate(portid, iso_port_list) :
    return sx_mgr.cli.set_port_isolate(portid, iso_port_list)

def mlx_get_port_status(portid) :
    return sx_mgr.cli.get_port_status(portid)

def mlx_get_port_speed(portid) :
    return sx_mgr.cli.get_port_speed(portid)

def mlx_get_port_mtu(portid) :
    return sx_mgr.cli.get_port_mtu(portid)

def mlx_get_port_fec(portid) :
    return sx_mgr.cli.get_port_fec(portid)

def mlx_get_interface() :
    return sx_mgr.cli.get_router_interface()

def mlx_set_interface(rif) :
    return sx_mgr.cli.set_router_interface(rif)

def mlx_get_reserved_buffer_list(port_id):
    return sx_mgr.cli.get_reserved_buffer_list(port_id)

def mlx_print_reserved_buffer_list(port_id):
    return sx_mgr.cli.print_reserved_buffer_list(port_id)

def mlx_create_reserved_buffer(port_id, pool_id, buffer_type, buffer_id, cell_count) :
    return sx_mgr.cli.create_reserved_buffer(port_id, pool_id, buffer_type, buffer_id, cell_count)

def mlx_delete_reserved_buffer(port_id, buffer_item) :
    return sx_mgr.cli.delete_reserved_buffer(port_id, buffer_item)

def mlx_get_shared_buffer_list(port_id):
    return sx_mgr.cli.get_shared_buffer_list(port_id)

def mlx_create_shared_buffer(port_id, pool_id, buffer_type, buffer_id, cell_count) :
    return sx_mgr.cli.create_shared_buffer(port_id, pool_id, buffer_type, buffer_id, cell_count)

def mlx_delete_shared_buffer(port_id, buffer_item) :
    return sx_mgr.cli.delete_shared_buffer(port_id, buffer_item)

def mlx_create_pool_buffer(pool_id, mode, size) :
    return sx_mgr.cli.create_pool_buffer(pool_id, mode, size)

def mlx_get_pool_list():
    return sx_mgr.cli.get_pool_list()

def mlx_get_pool_buffer_limits() :
    return sx_mgr.cli.get_pool_buffer_limits()

def mlx_delete_pool_buffer(pool_id) :
    return sx_mgr.cli.delete_pool_buffer(pool_id)

def mlx_get_free_buffer_counters() :
    return sx_mgr.cli.get_free_buffer_counters()

def mlx_get_pcpdei_prio_map(port_id):
    return sx_mgr.cli.get_pcpdei_prio_map(port_id)

def mlx_get_dscp_prio_map(port_id):
    return sx_mgr.cli.get_dscp_prio_map(port_id)

def mlx_get_port_priority_trust(port_id):
    return sx_mgr.cli.get_port_priority_trust(port_id)

def mlx_get_pg_prio_map(port_id):
    return sx_mgr.cli.get_pg_prio_map(port_id)
                                
def mlx_get_port_scheduler(port_id):
    return sx_mgr.cli.get_port_scheduler(port_id)

def mlx_get_pcpdei_rewrite_map(port_id) :
    return sx_mgr.cli.get_pcpdei_rewrite_map(port_id)

def mlx_get_dscp_rewrite_map(port_id) :
    return sx_mgr.cli.get_dscp_rewrite_map(port_id)

def mlx_get_port_priority_rewrite(port_id):
    return sx_mgr.cli.get_port_priority_rewrite(port_id)

def mlx_get_prio_tc_map(port_id):
    return sx_mgr.cli.get_prio_tc_map(port_id)

def mlx_get_prio_ieee_map():
    return sx_mgr.cli.get_prio_ieee_map()

def mlx_get_mc_aware(port_id):
    return sx_mgr.cli.get_mc_aware(port_id)

def mlx_get_flow_control(port_id):
    return sx_mgr.cli.get_flow_control(port_id)

def mlx_get_port_sflow(port_id) :
    return sx_mgr.cli.get_port_sflow(port_id)

def mlx_get_port_sflow_sample_rate_max():
    return sx_mgr.cli.get_port_sflow_sample_rate_max()

def mlx_set_port_sflow(portid, ing_rate, egr_rate=0, deviation=0,
                       uc=True, mc=True, bc=True, uuc=True, umc=True):
    return sx_mgr.cli.set_port_sflow(portid, ing_rate, egr_rate, deviation,
                                     uc, mc, bc, uuc, umc)

def mlx_get_port_sflow_statistics(port_id) :
    return sx_mgr.cli.get_port_sflow_statistics(port_id)

def mlx_clear_port_sflow_statistics(port_id) :
    return sx_mgr.cli.clear_port_sflow_statistics(port_id)

def mlx_get_vport_port(vport):
    return  sx_mgr.cli.get_vport_port(vport)

def mlx_set_port_priority_rewrite(port_id, dscp, pcpdei, exp):
    return sx_mgr.cli.set_port_priority_rewrite(port_id, dscp, pcpdei, exp)

def mlx_set_port_dscp_remark(portid, priority, color, dscp):
    return sx_mgr.cli.set_port_dscp_remark(portid, priority, color, dscp)

def mlx_get_reg_recr_v2(port_id) :
    return sx_mgr.cli.get_reg_recr_v2(port_id)

