#
# Copyright 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class Accton4654SFPp(cumulus.platform.Apollo10GPort):
    def __init__(self, label, hw_port, phy_addr, leds):
        cumulus.platform.Apollo10GPort.__init__(self,
                                                label=label,
                                                hw_port=hw_port,
                                                phy_type="84754",
                                                phy_addr=phy_addr,
                                                phy_layer='SFP+',
                                                leds=leds,
                                                unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')
    # one LED per port, two bits per LED:
    #   0, 1 -> green
    #   1, 0 -> orange
    _set_leds = '''
; Accton4654SFPp - activity
set_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX    ;; This bit indicates linkup
        tor
        tinv    ; active zero
        pack

        pushst  PORTSTATUS_ONE
        pack

        ret
    '''
    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call_set_leds = (fn_name, self._set_leds, self.led_port_num(mode, 0))
        set_calls[led_index_list[0]] = call_set_leds
        set_calls[led_index_list[0] + 1] = None

        return set_calls

class Accton4654LED0(cumulus.ledup.TriumphLEDController):
    num_leds = 48*4

class Accton4654LED1(cumulus.ledup.TriumphLEDController):
    num_leds = 12*2

class Accton4654GigE(cumulus.platform.Apollo1GPort):
    def __init__(self, label, hw_port, phy_addr, prim_offset, leds=None):
        cumulus.platform.Apollo1GPort.__init__(self,
                                               label=label,
                                               hw_port=hw_port,
                                               phy_type="54280",
                                               phy_addr=phy_addr,
                                               prim_offset=prim_offset,
                                               leds=leds,
                                               unit=0)
    # two LEDs per port, two bits per LED:
    #   left
    #   0, 1 -> green
    #   1, 0 -> orange
    #
    #   right
    #   0, 1 -> green
    #   1, 0 -> orange
    _set_leds = '''
; Accton4654GigE - link/activity
set_led:
        port a

        pushst  PORTSTATUS_ONE
        pack

        pushst  PORTSTATUS_LINKEN
        tinv    ; active low
        pack

        pushst  PORTSTATUS_ONE
        pack

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv    ; active low
        pack

        ret
    '''
    _set_leds_orange = '''
; Accton4654GigE - link/activity
set_led:
        port 1

        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ONE
        pack
        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ONE
        pack

        ret
    '''
    _set_leds_green = '''
; Accton4654GigE - link/activity
set_led:
        port 1

        pushst  PORTSTATUS_ONE
        pack
        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ONE
        pack
        pushst  PORTSTATUS_ZERO
        pack

        ret
    '''
    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        led_index = led_index_list[0]
        fn_name = '%s' % self.__class__.__name__
#	fn_name = fn_name + '_swp' + self.label + '_phy0x%x' % self.phy_addrs[0] + '_' + self.hw_ports[0]
#        set_calls[led_index_list] = ('orange', self._set_leds_orange, None)
        set_calls[led_index] = (fn_name, self._set_leds, self.led_port_num(mode, 0))
        set_calls[led_index + 1] = None
        set_calls[led_index + 2] = None
        set_calls[led_index + 3] = None


#	for x in range(0*4, 48*4, 4):
#            set_calls[x] = ('green', self._set_leds_green, None)
#            set_calls[x + 1] = None
#            set_calls[x + 2] = None
#            set_calls[x + 3] = None
#
#        import os
#        set_calls[int(os.getenv('FOO'))*4] = ('orange', self._set_leds_orange, None)

        return set_calls

class Accton4654CPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/ff705000.localbus/ea000000.cpld'

class Accton4654Switch(cumulus.platform.BCMSwitch):
    ledup = (Accton4654LED0, Accton4654LED1)
    _gige = Accton4654GigE
    _sfp = Accton4654SFPp
    has_phy_ucode_mdio = True
    has_phy_ucode_spi_rom = False

    ports = (
        _gige(label="1",  hw_port='ge1',  phy_addr=0x2,  prim_offset=0x0101, leds=(0, 6*4)),
        _gige(label="2",  hw_port='ge0',  phy_addr=0x1,  prim_offset=0x0100, leds=(0, 7*4)),
        _gige(label="3",  hw_port='ge3',  phy_addr=0x4,  prim_offset=0x0103, leds=(0, 4*4)),
        _gige(label="4",  hw_port='ge2',  phy_addr=0x3,  prim_offset=0x0102, leds=(0, 5*4)),
        _gige(label="5",  hw_port='ge5',  phy_addr=0x6,  prim_offset=0x0105, leds=(0, 2*4)),
        _gige(label="6",  hw_port='ge4',  phy_addr=0x5,  prim_offset=0x0104, leds=(0, 3*4)),
        _gige(label="7",  hw_port='ge7',  phy_addr=0x8,  prim_offset=0x0107, leds=(0, 0*4)),
        _gige(label="8",  hw_port='ge6',  phy_addr=0x7,  prim_offset=0x0106, leds=(0, 1*4)),
        _gige(label="9",  hw_port='ge33', phy_addr=0xa,  prim_offset=0x0901, leds=(0, 14*4)),
        _gige(label="10", hw_port='ge32', phy_addr=0x9,  prim_offset=0x0900, leds=(0, 15*4)),
        _gige(label="11", hw_port='ge35', phy_addr=0xc,  prim_offset=0x0903, leds=(0, 12*4)),
        _gige(label="12", hw_port='ge34', phy_addr=0xb,  prim_offset=0x0902, leds=(0, 13*4)),
        _gige(label="13", hw_port='ge37', phy_addr=0xe,  prim_offset=0x0905, leds=(0, 10*4)),
        _gige(label="14", hw_port='ge36', phy_addr=0xd,  prim_offset=0x0904, leds=(0, 11*4)),
        _gige(label="15", hw_port='ge39', phy_addr=0x10, prim_offset=0x0907, leds=(0, 8*4)),
        _gige(label="16", hw_port='ge38', phy_addr=0xf,  prim_offset=0x0906, leds=(0, 9*4)),
        _gige(label="17", hw_port='ge41', phy_addr=0x12, prim_offset=0x1101, leds=(0, 22*4)),
        _gige(label="18", hw_port='ge40', phy_addr=0x11, prim_offset=0x1100, leds=(0, 23*4)),
        _gige(label="19", hw_port='ge43', phy_addr=0x14, prim_offset=0x1103, leds=(0, 20*4)),
        _gige(label="20", hw_port='ge42', phy_addr=0x13, prim_offset=0x1102, leds=(0, 21*4)),
        _gige(label="21", hw_port='ge45', phy_addr=0x16, prim_offset=0x1105, leds=(0, 18*4)),
        _gige(label="22", hw_port='ge44', phy_addr=0x15, prim_offset=0x1104, leds=(0, 19*4)),
        _gige(label="23", hw_port='ge47', phy_addr=0x18, prim_offset=0x1107, leds=(0, 16*4)),
        _gige(label="24", hw_port='ge46', phy_addr=0x17, prim_offset=0x1106, leds=(0, 17*4)),
        _gige(label="25", hw_port='ge9' , phy_addr=0x22, prim_offset=0x2101, leds=(0, 30*4)),
        _gige(label="26", hw_port='ge8',  phy_addr=0x21, prim_offset=0x2100, leds=(0, 31*4)),
        _gige(label="27", hw_port='ge11', phy_addr=0x24, prim_offset=0x2103, leds=(0, 28*4)),
        _gige(label="28", hw_port='ge10', phy_addr=0x23, prim_offset=0x2102, leds=(0, 29*4)),
        _gige(label="29", hw_port='ge13', phy_addr=0x26, prim_offset=0x2105, leds=(0, 26*4)),
        _gige(label="30", hw_port='ge12', phy_addr=0x25, prim_offset=0x2104, leds=(0, 27*4)),
        _gige(label="31", hw_port='ge15', phy_addr=0x28, prim_offset=0x2107, leds=(0, 24*4)),
        _gige(label="32", hw_port='ge14', phy_addr=0x27, prim_offset=0x2106, leds=(0, 25*4)),
        _gige(label="33", hw_port='ge17', phy_addr=0x2a, prim_offset=0x2a01, leds=(0, 38*4)),
        _gige(label="34", hw_port='ge16', phy_addr=0x29, prim_offset=0x2a00, leds=(0, 39*4)),
        _gige(label="35", hw_port='ge19', phy_addr=0x2c, prim_offset=0x2a03, leds=(0, 36*4)),
        _gige(label="36", hw_port='ge18', phy_addr=0x2b, prim_offset=0x2a02, leds=(0, 37*4)),
        _gige(label="37", hw_port='ge21', phy_addr=0x2e, prim_offset=0x2a05, leds=(0, 34*4)),
        _gige(label="38", hw_port='ge20', phy_addr=0x2d, prim_offset=0x2a04, leds=(0, 35*4)),
        _gige(label="39", hw_port='ge23', phy_addr=0x30, prim_offset=0x2a07, leds=(0, 32*4)),
        _gige(label="40", hw_port='ge22', phy_addr=0x2f, prim_offset=0x2a06, leds=(0, 33*4)),
        _gige(label="41", hw_port='ge25', phy_addr=0x32, prim_offset=0x3201, leds=(0, 46*4)),
        _gige(label="42", hw_port='ge24', phy_addr=0x31, prim_offset=0x3200, leds=(0, 47*4)),
        _gige(label="43", hw_port='ge27', phy_addr=0x34, prim_offset=0x3203, leds=(0, 44*4)),
        _gige(label="44", hw_port='ge26', phy_addr=0x33, prim_offset=0x3202, leds=(0, 45*4)),
        _gige(label="45", hw_port='ge29', phy_addr=0x36, prim_offset=0x3205, leds=(0, 42*4)),
        _gige(label="46", hw_port='ge28', phy_addr=0x35, prim_offset=0x3204, leds=(0, 43*4)),
        _gige(label="47", hw_port='ge31', phy_addr=0x38, prim_offset=0x3207, leds=(0, 40*4)),
        _gige(label="48", hw_port='ge30', phy_addr=0x37, prim_offset=0x3206, leds=(0, 41*4)),
        _sfp(label="49",  hw_port='hg0',  phy_addr=0x40, leds=(1,11*2)),
        _sfp(label="50",  hw_port='hg1',  phy_addr=0x41, leds=(1,10*2)),
        _sfp(label="51",  hw_port='hg2',  phy_addr=0x42, leds=(1,9*2)),
        _sfp(label="52",  hw_port='hg3',  phy_addr=0x43, leds=(1,8*2)),
    )

class Accton4654PSU1(cumulus.sensors.PSU_Unit):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class Accton4654PSU2(cumulus.sensors.PSU_Unit):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

class Accton4654Temp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = '/sys/devices/soc.0/ff703000.i2c/i2c-0/i2c-9/9-0049'
    driver_hwmon = ['temp1']
    temp_fan_min = 55
    temp_max = 75
    temp_crit = 85

class Accton4654Temp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = cumulus.sensors.Temp_Unit.p2020_cpu_sensor_name
    driver_path = '/sys/devices/soc.0/ff703100.i2c/i2c-1/1-002e'
    driver_hwmon = ['temp1']
    temp_fan_min = 65
    temp_max = 100
    #P2020 (Tj) Temperature Operating range is 0-125 C
    temp_crit = 120

# Accton has suggested a minpwm of 0xBF(191). Seems to high to
# start with. Sticking to our minpwm 64.
class Accton4654Fan1(cumulus.sensors.Fan_Unit):
    name = 'Fan1'
    cpld_hwmon = ['fan0']
    pwm_path = '/sys/devices/soc.0/ff703100.i2c/i2c-1/1-002e'
    pwm_hwmon = ['pwm1']
    minpwm = 64

class Accton4654Fan2(cumulus.sensors.Fan_Unit):
    name = 'Fan2'
    cpld_hwmon = ['fan1']
    pwm_path = '/sys/devices/soc.0/ff703100.i2c/i2c-1/1-002e'
    pwm_hwmon = ['pwm3']
    minpwm = 64

class Accton4654Sensors():
    units = ( Accton4654PSU1(),
              Accton4654PSU2(),
              Accton4654Temp1(),
              Accton4654Temp2(),
              Accton4654Fan1(),
              Accton4654Fan2(),
              )
class Accton4654PWM1(cumulus.sensors.PWM_Unit):
    fans = (Accton4654Fan1(),
            Accton4654Fan2(),
            )
    temps = ( Accton4654Temp1(),
              Accton4654Temp2(),
              )

class Accton4654PWMs():
    units = (Accton4654PWM1(),
             )

class Accton4654PSU1LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU1'
    sources = ['PSU1']
    led_location = 'led_psu1'

class Accton4654PSU2LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU2'
    sources = ['PSU2']
    led_location = 'led_psu2'

class Accton4654Fan1LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan1'
    sources = ['Fan1']
    led_location = 'led_fan0'

class Accton4654Fan2LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan2'
    sources = ['Fan2']
    led_location = 'led_fan1'

class Accton4654SystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_diag'

class Accton4654LEDs():
    units = (Accton4654PSU1LED(),
             Accton4654PSU2LED(),
             Accton4654Fan1LED(),
             Accton4654Fan2LED(),
             Accton4654SystemLED(),
             )

class Accton4654(cumulus.platform.Platform):
    name = 'accton,as4600_54t'
    cpld = Accton4654CPLD()
    chip = cumulus.platform.ApolloChip()
    switch = Accton4654Switch(chip)
    sensors = Accton4654Sensors()
    pwms = Accton4654PWMs()
    statusleds = Accton4654LEDs()

Platform = Accton4654
