#
# Copyright 2017.  Cumulus Networks, Inc.
#

import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import cumulus.platforms.mlnx_SN2xxx

AcctonAS7312_54X_cpld_path = '/sys/bus/platform/devices/accton_as7312_54x_cpld.0'

class AcctonAS7312_54XCPLD(cumulus.cpld.SysFS):
    path = AcctonAS7312_54X_cpld_path

class AcctonAS7312_54XFanOdd(cumulus.sensors.Fan_Unit):
    fan_min = 8700
    fan_max = 21000
    driver_path = AcctonAS7312_54X_cpld_path
    pwm_path = driver_path

class AcctonAS7312_54XFanEven(AcctonAS7312_54XFanOdd):
    fan_min = 7300
    fan_max = 17700

class AcctonAS7312_54XFan1(AcctonAS7312_54XFanOdd):
    name = 'Fan1'
    description = 'Fan Tray 1'
    cpld_hwmon = ['fan1_ok']
    pwm_hwmon = ['pwm1']

class AcctonAS7312_54XFan2(AcctonAS7312_54XFanEven):
    name = 'Fan2'
    description = 'Fan Tray 1'
    cpld_hwmon = ['fan2_ok']
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm1']

class AcctonAS7312_54XFan3(AcctonAS7312_54XFanOdd):
    name = 'Fan3'
    description = 'Fan Tray 2'
    cpld_hwmon = ['fan3_ok']
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm2']

class AcctonAS7312_54XFan4(AcctonAS7312_54XFanEven):
    name = 'Fan4'
    description = 'Fan Tray 2'
    cpld_hwmon = ['fan4_ok']
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm2']

class AcctonAS7312_54XFan5(AcctonAS7312_54XFanOdd):
    name = 'Fan5'
    description = 'Fan Tray 3'
    cpld_hwmon = ['fan5_ok']
    driver_hwmon = ['fan5']
    pwm_hwmon = ['pwm3']

class AcctonAS7312_54XFan6(AcctonAS7312_54XFanEven):
    name = 'Fan6'
    description = 'Fan Tray 3'
    cpld_hwmon = ['fan6_ok']
    driver_hwmon = ['fan6']
    pwm_hwmon = ['pwm3']

class AcctonAS7312_54XFan7(AcctonAS7312_54XFanOdd):
    name = 'Fan7'
    description = 'Fan Tray 4'
    cpld_hwmon = ['fan7_ok']
    driver_hwmon = ['fan7']
    pwm_hwmon = ['pwm4']

class AcctonAS7312_54XFan8(AcctonAS7312_54XFanEven):
    name = 'Fan8'
    description = 'Fan Tray 4'
    cpld_hwmon = ['fan8_ok']
    driver_hwmon = ['fan8']
    pwm_hwmon = ['pwm4']

class AcctonAS7312_54XFan9(AcctonAS7312_54XFanOdd):
    name = 'Fan9'
    description = 'Fan Tray 5'
    cpld_hwmon = ['fan9_ok']
    driver_hwmon = ['fan9']
    pwm_hwmon = ['pwm5']

class AcctonAS7312_54XFan10(AcctonAS7312_54XFanEven):
    name = 'Fan10'
    description = 'Fan Tray 5'
    cpld_hwmon = ['fan10_ok']
    driver_hwmon = ['fan10']
    pwm_hwmon = ['pwm5']

class AcctonAS7312_54XFan11(AcctonAS7312_54XFanOdd):
    name = 'Fan11'
    description = 'Fan Tray 6'
    cpld_hwmon = ['fan11_ok']
    driver_hwmon = ['fan11']
    pwm_hwmon = ['pwm6']

class AcctonAS7312_54XFan12(AcctonAS7312_54XFanEven):
    name = 'Fan12'
    description = 'Fan Tray 6'
    cpld_hwmon = ['fan12_ok']
    driver_hwmon = ['fan12']
    pwm_hwmon = ['pwm6']

class AcctonAS7312_54XTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = 'Power VCORE'
    driver_path = '/sys/bus/i2c/devices/21-0048'
    temp_max  = 60
    temp_crit = 70

class AcctonAS7312_54XTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'Switch Board'
    driver_path = '/sys/bus/i2c/devices/21-0049'
    temp_max  = 60
    temp_crit = 70

class AcctonAS7312_54XTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = 'Ambient'
    driver_path = '/sys/bus/i2c/devices/21-004a'
    temp_max  = 60
    temp_crit = 70

class AcctonAS7312_54XTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    description = 'CPU'
    driver_path = '/sys/bus/i2c/devices/21-004b'
    temp_max  = 60
    temp_crit = 70

# Intel CPU die sensors
AcctonAS7312_54X_core_path = '/sys/devices/platform/coretemp.0'
class AcctonAS7312_54XTempCPU(cumulus.sensors.Temp_Unit):
    description = 'Intel CPU die sensor'
    driver_path = AcctonAS7312_54X_core_path
    temp_crit = 98
    temp_max  = 80
    temp_fan_max = 80
    temp_fan_min = 54

class AcctonAS7312_54XTemp5(AcctonAS7312_54XTempCPU):
    name = 'Temp5'
    driver_hwmon = ['temp2']

class AcctonAS7312_54XTemp6(AcctonAS7312_54XTempCPU):
    name = 'Temp6'
    driver_hwmon = ['temp3']

class AcctonAS7312_54XTemp7(AcctonAS7312_54XTempCPU):
    name = 'Temp7'
    driver_hwmon = ['temp4']

class AcctonAS7312_54XTemp8(AcctonAS7312_54XTempCPU):
    name = 'Temp8'
    driver_hwmon = ['temp5']

class AcctonAS7312_54XTHTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path = '/cumulus/switchd/run'
    temp_max = 105
    temp_crit = 110

class AcctonAS7312_54XTHTemp1(AcctonAS7312_54XTHTemp):
    name = 'Temp9'
    driver_hwmon = ['temp1']

class AcctonAS7312_54XTHTemp2(AcctonAS7312_54XTHTemp):
    name = 'Temp10'
    driver_hwmon = ['temp2']

class AcctonAS7312_54XTHTemp3(AcctonAS7312_54XTHTemp):
    name = 'Temp11'
    driver_hwmon = ['temp3']

class AcctonAS7312_54XTHTemp4(AcctonAS7312_54XTHTemp):
    name = 'Temp12'
    driver_hwmon = ['temp4']

class AcctonAS7312_54XTHTemp5(AcctonAS7312_54XTHTemp):
    name = 'Temp13'
    driver_hwmon = ['temp5']

class AcctonAS7312_54XTHTemp6(AcctonAS7312_54XTHTemp):
    name = 'Temp14'
    driver_hwmon = ['temp6']

class AcctonAS7312_54XTHTemp7(AcctonAS7312_54XTHTemp):
    name = 'Temp15'
    driver_hwmon = ['temp7']

class AcctonAS7312_54XTHTemp8(AcctonAS7312_54XTHTemp):
    name = 'Temp16'
    driver_hwmon = ['temp8']

class AcctonAS7312_54X_PSU(cumulus.sensors.PSU_Unit):
    init_done = False
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            cmd = 'rmmod pmbus && modprobe pmbus'
            subprocess.call(cmd, shell=True)
            self.init_done = True

class AcctonAS7312_54XPSU1(AcctonAS7312_54X_PSU):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class AcctonAS7312_54XPSU2(AcctonAS7312_54X_PSU):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

AcctonAS7312_54X_psu1_path = '/sys/bus/i2c/devices/11-005b'
AcctonAS7312_54X_psu2_path = '/sys/bus/i2c/devices/10-0059'
class AcctonAS7312_54XPSUTemp(cumulus.sensors.Temp_Unit):
    driver_hwmon = ['temp1']
    cpld_path = AcctonAS7312_54X_cpld_path
    temp_max = 60
    temp_crit = 100

    def find_and_add_attrs(self):
        if self.get_cpld_value(self.cpld_psu).find('ok') >= 0:
            return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)
        else:
            return -1

class AcctonAS7312_54XPSUTemp1(AcctonAS7312_54XPSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp1'
    driver_hwmon = ['temp1']
    driver_path = AcctonAS7312_54X_psu1_path
    cpld_psu = 'psu_pwr1'

class AcctonAS7312_54XPSUTemp2(AcctonAS7312_54XPSUTemp):
    name = 'PSU1Temp2'
    description = 'PSU1 Temp2'
    driver_hwmon = ['temp2']
    driver_path = AcctonAS7312_54X_psu1_path
    cpld_psu = 'psu_pwr1'

class AcctonAS7312_54XPSUTemp3(AcctonAS7312_54XPSUTemp):
    name = 'PSU1Temp3'
    description = 'PSU1 Temp3'
    driver_hwmon = ['temp3']
    driver_path = AcctonAS7312_54X_psu1_path
    cpld_psu = 'psu_pwr1'

class AcctonAS7312_54XPSUTemp4(AcctonAS7312_54XPSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp1'
    driver_hwmon = ['temp1']
    driver_path = AcctonAS7312_54X_psu2_path
    cpld_psu = 'psu_pwr2'

class AcctonAS7312_54XPSUTemp5(AcctonAS7312_54XPSUTemp):
    name = 'PSU2Temp2'
    description = 'PSU2 Temp2'
    driver_hwmon = ['temp2']
    driver_path = AcctonAS7312_54X_psu2_path
    cpld_psu = 'psu_pwr2'

class AcctonAS7312_54XPSUTemp6(AcctonAS7312_54XPSUTemp):
    name = 'PSU2Temp3'
    description = 'PSU2 Temp3'
    driver_hwmon = ['temp3']
    driver_path = AcctonAS7312_54X_psu2_path
    cpld_psu = 'psu_pwr2'

class AcctonAS7312_54XPSUFan(cumulus.sensors.Fan_Unit):
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    cpld_path = AcctonAS7312_54X_cpld_path
    fan_max = 24500
    fan_min = 5000

    def find_and_add_attrs(self):
        if self.get_cpld_value(self.cpld_hwmon[0]).find('ok') >= 0:
            return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)
        else:
            return -1

class AcctonAS7312_54XFan13(AcctonAS7312_54XPSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    driver_path = AcctonAS7312_54X_psu1_path
    cpld_hwmon = ['psu_pwr1']

class AcctonAS7312_54XFan14(AcctonAS7312_54XPSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    driver_path = AcctonAS7312_54X_psu2_path
    cpld_hwmon = ['psu_pwr2']

class AcctonAS7312SFP28(cumulus.platform.Tomahawk25GPort):
    _set_led = '''
    ;;;
    ;;; AcctonAS7312SFP28
    ;;;
set_led:
        call    set_group_4x
        call    set_port_led
        call    set_port_led
        call    set_port_led
        call    set_port_led
        ret
        '''

    _set_led_common = '''
; sets the leds for a lane
; 4 channels/ports per group (one led each)
; 2 bits config for each group
; each channel/port gets
;   2 speed bits
;   1 bit of link/activity
; To keep things consistent with other platforms
; we do green/off for all leds

set_group_1x:
;
; group config
;
        pushst  PORTSTATUS_ONE
        pack
        pushst  PORTSTATUS_ZERO
        pack
        ret

set_group_2x:
;
; group config
;
        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ONE
        pack
        ret

set_group_4x:
;
; group config
;
        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ZERO
        pack
        ret

set_port_led:
        port a
;
; speed
;
        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ONE
        pack
;
; link/activity
;
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        pack
        inc     a

        ret

set_port_off:
;
; speed
;
        pushst  PORTSTATUS_ZERO
        pack
        pushst  PORTSTATUS_ONE
        pack
        pushst  PORTSTATUS_ZERO
        pack
        inc     a
        ret
'''

    def __init__(self, label, serdes, leds, rx_lane_map=None, tx_lane_map=None,
                 rx_polarity_flip=None, tx_polarity_flip=None, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp28')
        cumulus.platform.Tomahawk25GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=leds,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp
    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)
        call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        for idx in range(1, 14):
            set_calls[led_index_list[0]+idx] = None

        return set_calls

class AcctonAS7312QSFP28(cumulus.platform.Tomahawk100GPort):
    _set_led = '''
;;;
;;; AcctonAS7312QSFP28
;;;
'''
    _set_led_1x = '''
set_led:
        call      set_group_1x
        call      set_port_led
        call      set_port_off
        call      set_port_off
        call      set_port_off
        ret
    '''

    _set_led_2x = '''
set_led:
        call      set_group_2x
        call      set_port_led
        inc       a
        call      set_port_led
        call      set_port_off
        call      set_port_off
        ret
    '''

    _set_led_4x = '''
set_led:
        call      set_group_4x
        call      set_port_led
        call      set_port_led
        call      set_port_led
        call      set_port_led
        ret
    '''
    def __init__(self, label, serdes, leds, rx_lane_map, tx_lane_map,
                 rx_polarity_flip, tx_polarity_flip, pre_emphasis=None):
        sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp28')
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=sfp.type.upper(),
                                                   leds=leds,
                                                   rx_lane_map=rx_lane_map,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   pre_emphasis=pre_emphasis,
                                                   oversub=True,
                                                   unit=0)
        self.sfp = sfp
    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if sub != 0:
            # We only set a caller for the first sub-interface
            # regardless of what mode we're in.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, sub)

        num_lp = self.num_logical_ports(mode)
        if num_lp == 4:
            call = (fn_name + "_4x", self._set_led_4x, led_port)
        elif num_lp == 2:
            call = (fn_name + "_2x", self._set_led_2x, led_port)
        else:
            call = (fn_name + "_1x", self._set_led_1x, led_port)

        set_calls[led_index_list[0]] = call
        for idx in range(1, 14):
            set_calls[led_index_list[0]+idx] = None

        return set_calls

class AcctonAS7312_54XSensors():
    units = (
        AcctonAS7312_54XFan1(),
        AcctonAS7312_54XFan2(),
        AcctonAS7312_54XFan3(),
        AcctonAS7312_54XFan4(),
        AcctonAS7312_54XFan5(),
        AcctonAS7312_54XFan6(),
        AcctonAS7312_54XFan7(),
        AcctonAS7312_54XFan8(),
        AcctonAS7312_54XFan9(),
        AcctonAS7312_54XFan10(),
        AcctonAS7312_54XFan11(),
        AcctonAS7312_54XFan12(),
        AcctonAS7312_54XFan13(),
        AcctonAS7312_54XFan14(),
        AcctonAS7312_54XTemp1(),
        AcctonAS7312_54XTemp2(),
        AcctonAS7312_54XTemp3(),
        AcctonAS7312_54XTemp4(),
        AcctonAS7312_54XTemp5(),
        AcctonAS7312_54XTemp6(),
        AcctonAS7312_54XTemp7(),
        AcctonAS7312_54XTemp8(),
        AcctonAS7312_54XTHTemp1(),
        AcctonAS7312_54XTHTemp2(),
        AcctonAS7312_54XTHTemp3(),
        AcctonAS7312_54XTHTemp4(),
        AcctonAS7312_54XTHTemp5(),
        AcctonAS7312_54XTHTemp6(),
        AcctonAS7312_54XTHTemp7(),
        AcctonAS7312_54XTHTemp8(),
        AcctonAS7312_54XPSUTemp1(),
        AcctonAS7312_54XPSUTemp2(),
        AcctonAS7312_54XPSUTemp3(),
        AcctonAS7312_54XPSUTemp4(),
        AcctonAS7312_54XPSUTemp5(),
        AcctonAS7312_54XPSUTemp6(),
        AcctonAS7312_54XPSU1(),
        AcctonAS7312_54XPSU2(),
    )

class AcctonAS7312_54XPWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (
        AcctonAS7312_54XFan1(),
        AcctonAS7312_54XFan2(),
        AcctonAS7312_54XFan3(),
        AcctonAS7312_54XFan4(),
        AcctonAS7312_54XFan5(),
        AcctonAS7312_54XFan6(),
        AcctonAS7312_54XFan7(),
        AcctonAS7312_54XFan8(),
        AcctonAS7312_54XFan9(),
        AcctonAS7312_54XFan10(),
        AcctonAS7312_54XFan11(),
        AcctonAS7312_54XFan12(),
    )
    temps = (
        AcctonAS7312_54XTemp1(),
        AcctonAS7312_54XTemp2(),
        AcctonAS7312_54XTemp3(),
        AcctonAS7312_54XTemp4(),
        AcctonAS7312_54XTemp5(),
        AcctonAS7312_54XTemp6(),
        AcctonAS7312_54XTemp7(),
        AcctonAS7312_54XTemp8(),
        AcctonAS7312_54XTHTemp1(),
        AcctonAS7312_54XTHTemp2(),
        AcctonAS7312_54XTHTemp3(),
        AcctonAS7312_54XTHTemp4(),
        AcctonAS7312_54XTHTemp5(),
        AcctonAS7312_54XTHTemp6(),
        AcctonAS7312_54XTHTemp7(),
        AcctonAS7312_54XTHTemp8(),
    )

class AcctonAS7312_54XPWM2(cumulus.sensors.PWM_Unit):
    name = 'PSU1 PWM'
    fans = (
        AcctonAS7312_54XFan13(),
    )
    temps = (
        AcctonAS7312_54XPSUTemp1(),
        AcctonAS7312_54XPSUTemp2(),
        AcctonAS7312_54XPSUTemp3(),
    )

class AcctonAS7312_54XPWM3(cumulus.sensors.PWM_Unit):
    name = 'PSU2 PWM'
    fans = (
        AcctonAS7312_54XFan14(),
    )
    temps = (
        AcctonAS7312_54XPSUTemp4(),
        AcctonAS7312_54XPSUTemp5(),
        AcctonAS7312_54XPSUTemp6(),
    )

class AcctonAS7312_54XPWMs():
    units = (
        AcctonAS7312_54XPWM1(),
        AcctonAS7312_54XPWM2(),
        AcctonAS7312_54XPWM3(),
    )

class AcctonAS7312LED(cumulus.ledup.TomahawkLEDController):
    led_functions = (AcctonAS7312SFP28._set_led_common)
    num_leds = 126

class AcctonAS7312Switch(cumulus.platform.BCMSwitch):
    ledup = AcctonAS7312LED
    _sfp28 = AcctonAS7312SFP28
    _qsfp28 = AcctonAS7312QSFP28

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False
    ports = (
        _sfp28(label="1",   serdes=(40,), leds=(1, 0), tx_lane_map=0x0132,
              rx_lane_map=0x2310, tx_polarity_flip=True, rx_polarity_flip=False),
        _sfp28(label="2",   serdes=(41,), leds=None, tx_lane_map=0x0132,
              rx_lane_map=0x2310, tx_polarity_flip=True, rx_polarity_flip=False),
        _sfp28(label="3",   serdes=(42,), leds=None, tx_lane_map=0x0132,
              rx_lane_map=0x2310, tx_polarity_flip=True, rx_polarity_flip=False),
        _sfp28(label="4",   serdes=(43,), leds=None, tx_lane_map=0x0132,
              rx_lane_map=0x2310, tx_polarity_flip=True, rx_polarity_flip=False),
        _sfp28(label="5",   serdes=(48,), leds=(1, 14), tx_lane_map=0x3210,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="6",   serdes=(49,), leds=None, tx_lane_map=0x3210,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="7",   serdes=(50,), leds=None, tx_lane_map=0x3210,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="8",   serdes=(51,), leds=None, tx_lane_map=0x3210,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="9",   serdes=(52,), leds=(1, 28), tx_lane_map=0x3210,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="10",  serdes=(53,), leds=None, tx_lane_map=0x3210,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="11",  serdes=(54,), leds=None, tx_lane_map=0x3210,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="12",  serdes=(55,), leds=None, tx_lane_map=0x3210,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="13",  serdes=(64,), leds=(1, 42), tx_lane_map=0x0123,
               rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="14",  serdes=(65,), leds=None, tx_lane_map=0x0123,
               rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="15",  serdes=(66,), leds=None, tx_lane_map=0x0123,
               rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="16",  serdes=(67,), leds=None, tx_lane_map=0x0123,
               rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=False),
        _sfp28(label="17",  serdes=(32,), leds=(1, 56), tx_lane_map=0x0123,
               rx_lane_map=0x0123, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="18",  serdes=(33,), leds=None, tx_lane_map=0x0123,
               rx_lane_map=0x0123, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="19",  serdes=(34,), leds=None, tx_lane_map=0x0123,
               rx_lane_map=0x0123, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="20",  serdes=(35,), leds=None, tx_lane_map=0x0123,
               rx_lane_map=0x0123, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="21",  serdes=(36,), leds=(1, 70), tx_lane_map=0x0123,
               rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="22",  serdes=(37,), leds=None, tx_lane_map=0x0123,
               rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True,),
        _sfp28(label="23",  serdes=(38,), leds=None, tx_lane_map=0x0123,
               rx_lane_map=0x0123, tx_polarity_flip=False, rx_polarity_flip=True,),
        _sfp28(label="24",  serdes=(39,), leds=None, tx_lane_map=0x0123,
               rx_lane_map=0x0123, tx_polarity_flip=True,  rx_polarity_flip=True,),
        _sfp28(label="25",  serdes=(68,), leds=(1, 84),  tx_lane_map=0x0123,
               rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="26",  serdes=(69,), leds=None, tx_lane_map=0x0123,
               rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="27",  serdes=(70,), leds=None, tx_lane_map=0x0123,
               rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="28",  serdes=(71,), leds=None, tx_lane_map=0x0123,
               rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="29",  serdes=(80,), leds=(1, 98), tx_lane_map=0x0123,
               rx_lane_map=0x1032, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="30",  serdes=(81,), leds=None, tx_lane_map=0x0123,
               rx_lane_map=0x1032, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="31",  serdes=(82,), leds=None, tx_lane_map=0x0123,
               rx_lane_map=0x1032, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="32",  serdes=(83,), leds=None, tx_lane_map=0x0123,
               rx_lane_map=0x1032, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="33",  serdes=(84,), leds=(1, 112), tx_lane_map=0x2301,
               rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True,),
        _sfp28(label="34",  serdes=(85,), leds=None, tx_lane_map=0x2301,
               rx_lane_map=0x2301, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="35",  serdes=(86,), leds=None, tx_lane_map=0x2301,
               rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True,),
        _sfp28(label="36",  serdes=(87,), leds=None, tx_lane_map=0x2301,
               rx_lane_map=0x2301, tx_polarity_flip=True,  rx_polarity_flip=True,),
        _sfp28(label="37",  serdes=(96,), leds=(0, 0), tx_lane_map=0x0123,
               rx_lane_map=0x0123, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="38",  serdes=(97,), leds=None, tx_lane_map=0x0123,
               rx_lane_map=0x0123, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="39",  serdes=(98,), leds=None, tx_lane_map=0x0123,
               rx_lane_map=0x0123, tx_polarity_flip=True, rx_polarity_flip=True,),
        _sfp28(label="40",  serdes=(99,), leds=None, tx_lane_map=0x0123,
               rx_lane_map=0x0123, tx_polarity_flip=True, rx_polarity_flip=False,),
        _sfp28(label="41",  serdes=(100,), leds=(0, 14), tx_lane_map=0x3210,
               rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="42",  serdes=(101,), leds=None, tx_lane_map=0x3210,
               rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="43",  serdes=(102,), leds=None, tx_lane_map=0x3210,
               rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="44",  serdes=(103,), leds=None, tx_lane_map=0x3210,
               rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="45",  serdes=(104,), leds=(0, 28), tx_lane_map=0x1032,
               rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="46",  serdes=(105,), leds=None, tx_lane_map=0x1032,
               rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="47",  serdes=(106,), leds=None, tx_lane_map=0x1032,
               rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),
        _sfp28(label="48",  serdes=(107,), leds=None, tx_lane_map=0x1032,
               rx_lane_map=0x1032, tx_polarity_flip=False, rx_polarity_flip=False,),

        _qsfp28(label="49", serdes=(  4,   5,   6,   7), leds=(0, 42),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
        _qsfp28(label="50", serdes=(  0,   1,   2,   3), leds=(0, 56),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
        _qsfp28(label="51", serdes=(108, 109, 110, 111), leds=(0, 70),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x0123, rx_lane_map=0x0123),
        _qsfp28(label="52", serdes=( 20,  21,  22,  23), leds=(0, 84),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x0213),
        _qsfp28(label="53", serdes=(  8,   9,  10,  11), leds=(0, 98),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
        _qsfp28(label="54", serdes=( 116, 117, 118, 119), leds=(0, 112),
              tx_polarity_flip=0x000b, rx_polarity_flip=0x000f,
              tx_lane_map=0x3210, rx_lane_map=0x1230),
    )

class AcctonAS7312(cumulus.platform.Platform):
    name = 'act,as7312_54x'
    chip = cumulus.platform.TomahawkChip()
    switch = AcctonAS7312Switch(chip)
    cpld = AcctonAS7312_54XCPLD()
    sensors = AcctonAS7312_54XSensors()
    pwms = AcctonAS7312_54XPWMs()
    statusleds = None

Platform = AcctonAS7312
