#
# Copyright 2017.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class AcctonOMP800Fabric(cumulus.platform.Tomahawk100GPort):
    '''
    Fabric ports are 100G Tomahawk ports using KR4 serdes and
    no LEDs.
    '''
    is_fabric = True
    create_fabric = True
    def __init__(self, label, serdes, tx_polarity_flip,
                 rx_polarity_flip, tx_lane_map, rx_lane_map):
        cumulus.platform.Tomahawk100GPort.__init__(self,
                                                   label=label,
                                                   serdes=serdes,
                                                   leds=None,
                                                   phy_types=None,
                                                   phy_addrs=None,
                                                   phy_layer=None,
                                                   tx_polarity_flip=tx_polarity_flip,
                                                   rx_polarity_flip=rx_polarity_flip,
                                                   tx_lane_map=tx_lane_map,
                                                   rx_lane_map=rx_lane_map,
                                                   oversub=True,
                                                   unit=0)
        self._serdes_intf_map = { 100 : 'KR4' }

    @property
    def max_logical_ports(self):
        return 1

class AcctonOMP800Switch(cumulus.platform.BCMSwitch):
    _fab = AcctonOMP800Fabric
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (

        # Fabric Card Tomahawk B Backplane Left
        _fab(label="0",  serdes=(  0,   1,   2,   3),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x1032),
        _fab(label="1",  serdes=(  4,   5,   6,   7),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x1032),
        _fab(label="2",  serdes=(  8,   9,  10,  11),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x1032),
        _fab(label="3",  serdes=( 12,  13,  14,  15),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x1032),

        # Fabric Card Tomahawk B Whisper Cable Left
        _fab(label="4",  serdes=( 16,  17,  18,  19),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x3201, rx_lane_map=0x3201),
        _fab(label="5",  serdes=( 20,  21,  22,  23),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000c,
              tx_lane_map=0x1032, rx_lane_map=0x3210),
        _fab(label="6",  serdes=( 24,  25,  26,  27),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x1032, rx_lane_map=0x0123),
        _fab(label="7",  serdes=( 28,  29,  30,  31),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x2301, rx_lane_map=0x3210),

        # Fabric Card Tomahawk B Whisper Cable Right
        _fab(label="8",  serdes=( 32,  33,  34,  35),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
        _fab(label="9",  serdes=( 36,  37,  38,  39),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x1032, rx_lane_map=0x1032),
        _fab(label="10", serdes=( 40,  41,  42,  43),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x0123, rx_lane_map=0x0123),
        _fab(label="11", serdes=( 44,  45,  46,  47),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x2301, rx_lane_map=0x2301),

        # Fabric Card Tomahawk B Backplane Right
        _fab(label="12", serdes=( 48,  49,  50,  51),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x1032, rx_lane_map=0x3210),
        _fab(label="13", serdes=( 52,  53,  54,  55),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x1032, rx_lane_map=0x3210),
        _fab(label="14", serdes=( 56,  57,  58,  59),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x1032, rx_lane_map=0x3210),
        _fab(label="15", serdes=( 60,  61,  62,  63),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x1032, rx_lane_map=0x3210),

        # Fabric Card Tomahawk B Backplane Mid-Right
        _fab(label="16", serdes=( 64,  65,  66,  67),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x0132),
        _fab(label="17", serdes=( 68,  69,  70,  71),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x1032, rx_lane_map=0x3210),
        _fab(label="18", serdes=( 72,  73,  74,  75),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x1032, rx_lane_map=0x3210),
        _fab(label="19", serdes=( 76,  77,  78,  79),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0003,
              tx_lane_map=0x3210, rx_lane_map=0x3210),

        # Fabric Card Tomahawk B Whisper Cable Mid-Right
        _fab(label="20", serdes=( 80,  81,  82,  83),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x0123, rx_lane_map=0x1032),
        _fab(label="21", serdes=( 84,  85,  86,  87),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
        _fab(label="22", serdes=( 88,  89,  90,  91),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0003,
              tx_lane_map=0x3210, rx_lane_map=0x3201),
        _fab(label="23", serdes=( 92,  93,  94,  95),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x1032, rx_lane_map=0x1032),

        # Fabric Card Tomahawk B Whisper Cable Mid-Left
        _fab(label="24", serdes=( 96,  97,  98,  99),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x0123, rx_lane_map=0x1032),
        _fab(label="25", serdes=(100, 101, 102, 103),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
        _fab(label="26", serdes=(104, 105, 106, 107),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0006,
              tx_lane_map=0x3210, rx_lane_map=0x0231),
        _fab(label="27", serdes=(108, 109, 110, 111),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x1032, rx_lane_map=0x1032),

        # Fabric Card Tomahawk B Backplane Mid-Left
        _fab(label="28", serdes=(112, 113, 114, 115),
              tx_polarity_flip=0x000a, rx_polarity_flip=0x0005,
              tx_lane_map=0x3201, rx_lane_map=0x3012),
        _fab(label="29", serdes=(116, 117, 118, 119),
              tx_polarity_flip=0x0008, rx_polarity_flip=0x0004,
              tx_lane_map=0x3201, rx_lane_map=0x1320),
        _fab(label="30", serdes=(120, 121, 122, 123),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x1032),
        _fab(label="31", serdes=(124, 125, 126, 127),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x1032, rx_lane_map=0x3210),
    )


class AcctonOMP800(cumulus.platform.Platform):
    name = 'accton,omp800_fcb'
    chip = cumulus.platform.TomahawkChip()
    switch = AcctonOMP800Switch(chip)
    cpld = None
    statusleds = None
    pwms = None
    sensors = None

Platform = AcctonOMP800
