#
# Copyright 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class CelKennisisGigE(cumulus.platform.Triumph1GPort):
    def __init__(self, label, hw_port, phy_addr, prim_offset):
        cumulus.platform.Triumph1GPort.__init__(self,
                                                label=label,
                                                hw_port=hw_port,
                                                phy_type="54680",
                                                phy_addr=phy_addr,
                                                phy_layer='SFP+',
                                                prim_offset=prim_offset,
                                                leds=None,
                                                unit=0)

class CelKennisisSFPp(cumulus.platform.Triumph10GPort):
    _set_led = '''
;;; CelKennisisSFPp
;;; Two bits per port: yellow(active high), green(active high)
set_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack

        port 1
        pushst  PORTSTATUS_ZERO ; disable yellow LED
        pack

        ret
'''
    def __init__(self, label, hw_port, phy_addr, leds):
        cumulus.platform.Triumph10GPort.__init__(self,
                                                 label=label,
                                                 hw_port=hw_port,
                                                 phy_type="8747",
                                                 phy_addr=phy_addr,
                                                 phy_layer='SFP+',
                                                 leds=leds,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMBCM(phy_addr, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, self.hw_port_num(mode, 0))
        for led in led_index_list:
            set_calls[led] = call
            set_calls[led + 1] = None

        return set_calls

class CelKennisisCPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/ffe05000.localbus/ffb00000.CPLD'

class CelKennisisLED(cumulus.ledup.TriumphLEDController):
    num_leds = 8

class CelKennisisSwitch(cumulus.platform.BCMSwitch):
    ledup = CelKennisisLED
    _gige = CelKennisisGigE
    _sfp = CelKennisisSFPp
    has_phy_ucode_mdio = False # XXX unverified
    has_phy_ucode_spi_rom = False # XXX unverified

    ports = (
        _gige(label="1",  hw_port='ge1',  phy_addr=0x2,  prim_offset=0x1d02),
        _gige(label="2",  hw_port='ge0',  phy_addr=0x1,  prim_offset=0x1d01),
        _gige(label="3",  hw_port='ge3',  phy_addr=0x4,  prim_offset=0x1d04),
        _gige(label="4",  hw_port='ge2',  phy_addr=0x3,  prim_offset=0x1d03),
        _gige(label="5",  hw_port='ge5',  phy_addr=0x6,  prim_offset=0x1d06),
        _gige(label="6",  hw_port='ge4',  phy_addr=0x5,  prim_offset=0x1d05),
        _gige(label="7",  hw_port='ge7',  phy_addr=0x8,  prim_offset=0x2500),
        _gige(label="8",  hw_port='ge6',  phy_addr=0x7,  prim_offset=0x1d07),
        _gige(label="9",  hw_port='ge9',  phy_addr=0xa,  prim_offset=0x0102),
        _gige(label="10", hw_port='ge8' , phy_addr=0x9,  prim_offset=0x0101),
        _gige(label="11", hw_port='ge11', phy_addr=0xc,  prim_offset=0x0104),
        _gige(label="12", hw_port='ge10', phy_addr=0xb,  prim_offset=0x0103),
        _gige(label="13", hw_port='ge13', phy_addr=0xe,  prim_offset=0x0106),
        _gige(label="14", hw_port='ge12', phy_addr=0xd,  prim_offset=0x0105),
        _gige(label="15", hw_port='ge15', phy_addr=0x10, prim_offset=0x0900),
        _gige(label="16", hw_port='ge14', phy_addr=0xf,  prim_offset=0x0107),
        _gige(label="17", hw_port='ge17', phy_addr=0x12, prim_offset=0x2502),
        _gige(label="18", hw_port='ge16', phy_addr=0x11, prim_offset=0x2501),
        _gige(label="19", hw_port='ge19', phy_addr=0x14, prim_offset=0x2504),
        _gige(label="20", hw_port='ge18', phy_addr=0x13, prim_offset=0x2503),
        _gige(label="21", hw_port='ge21', phy_addr=0x16, prim_offset=0x0506),
        _gige(label="22", hw_port='ge20', phy_addr=0x15, prim_offset=0x0505),
        _gige(label="23", hw_port='ge23', phy_addr=0x18, prim_offset=0x0d00),
        _gige(label="24", hw_port='ge22', phy_addr=0x17, prim_offset=0x0507),
        _gige(label="25", hw_port='ge25', phy_addr=0x22, prim_offset=0x0d02),
        _gige(label="26", hw_port='ge24', phy_addr=0x21, prim_offset=0x0d01),
        _gige(label="27", hw_port='ge27', phy_addr=0x24, prim_offset=0x0d04),
        _gige(label="28", hw_port='ge26', phy_addr=0x23, prim_offset=0x0d03),
        _gige(label="29", hw_port='ge29', phy_addr=0x26, prim_offset=0x2506),
        _gige(label="30", hw_port='ge28', phy_addr=0x25, prim_offset=0x2505),
        _gige(label="31", hw_port='ge31', phy_addr=0x28, prim_offset=0x2d00),
        _gige(label="32", hw_port='ge30', phy_addr=0x27, prim_offset=0x2507),
        _gige(label="33", hw_port='ge33', phy_addr=0x2a, prim_offset=0x1102),
        _gige(label="34", hw_port='ge32', phy_addr=0x29, prim_offset=0x1101),
        _gige(label="35", hw_port='ge35', phy_addr=0x2c, prim_offset=0x1104),
        _gige(label="36", hw_port='ge34', phy_addr=0x2b, prim_offset=0x1103),
        _gige(label="37", hw_port='ge37', phy_addr=0x2e, prim_offset=0x1106),
        _gige(label="38", hw_port='ge36', phy_addr=0x2d, prim_offset=0x1105),
        _gige(label="39", hw_port='ge39', phy_addr=0x30, prim_offset=0x1900),
        _gige(label="40", hw_port='ge38', phy_addr=0x2f, prim_offset=0x1107),
        _gige(label="41", hw_port='ge41', phy_addr=0x32, prim_offset=0x2d02),
        _gige(label="42", hw_port='ge40', phy_addr=0x31, prim_offset=0x2d01),
        _gige(label="43", hw_port='ge43', phy_addr=0x34, prim_offset=0x2d04),
        _gige(label="44", hw_port='ge42', phy_addr=0x33, prim_offset=0x2d03),
        _gige(label="45", hw_port='ge45', phy_addr=0x36, prim_offset=0x2d06),
        _gige(label="46", hw_port='ge44', phy_addr=0x35, prim_offset=0x2d05),
        _gige(label="47", hw_port='ge47', phy_addr=0x38, prim_offset=0x3500),
        _gige(label="48", hw_port='ge46', phy_addr=0x37, prim_offset=0x2d07),
        _sfp(label="49", hw_port='hg0', phy_addr=0x47, leds=(0, 0)),
        _sfp(label="50", hw_port='hg1', phy_addr=0x46, leds=(0, 2)),
        _sfp(label="51", hw_port='hg2', phy_addr=0x45, leds=(0, 4)),
        _sfp(label="52", hw_port='hg3', phy_addr=0x44, leds=(0, 6)),
    )

class CelKennisisPSU1(cumulus.sensors.PSU_Unit):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class CelKennisisPSU2(cumulus.sensors.PSU_Unit):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

class CelKennisisTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/0-004d'
    driver_hwmon = ['temp1']
    temp_fan_min = 25
    temp_max = 65
    temp_crit = 90

class CelKennisisTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'Front Right Corner Ambient Temp Sensor'
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/0-004b'
    driver_hwmon = ['temp1']
    temp_fan_min = 25
    temp_max = 45
    temp_crit = 65

class CelKennisisTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = 'System Rear Outlet Temp Sensor'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-004e'
    driver_hwmon = ['temp1']
    temp_fan_min = 25
    temp_max = 45
    temp_crit = 60

class CelKennisisTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    description = 'P2020 Temp Sensor'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-0048'
    driver_hwmon = ['temp1']
    temp_fan_min = 25
    temp_max = 65
    temp_crit = 90

class CelKennisisTemp5(cumulus.sensors.Temp_Unit):
    name = 'Temp5'
    description = 'Front Left Corner Ambient Temp Sensor'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-004f'
    driver_hwmon = ['temp1']
    temp_fan_min = 25
    temp_max = 45
    temp_crit = 65

class CelKennisisFan1(cumulus.sensors.Fan_Unit):
    name = 'Fan1'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-004d'
    cpld_hwmon = ['fan1']
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']

class CelKennisisFan2(cumulus.sensors.Fan_Unit):
    name = 'Fan2'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-004d'
    cpld_hwmon = ['fan2']
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']

class CelKennisisFan3(cumulus.sensors.Fan_Unit):
    name = 'Fan3'
    driver_path = '/sys/devices/soc.0/ffe03100.i2c/i2c-1/1-004d'
    cpld_hwmon = ['fan3']
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']

class CelKennisisPWM1(cumulus.sensors.PWM_Unit):
    fans = (CelKennisisFan1(),
            CelKennisisFan2(),
            CelKennisisFan3(),
            )
    temps = (CelKennisisTemp1(),
             CelKennisisTemp2(),
             CelKennisisTemp3(),
             CelKennisisTemp4(),
             CelKennisisTemp5(),
             )

class CelKennisisSensors():
    units = ( CelKennisisPSU1(),
              CelKennisisPSU2(),
              CelKennisisFan1(),
              CelKennisisFan2(),
              CelKennisisFan3(),
              CelKennisisTemp1(),
              CelKennisisTemp2(),
              CelKennisisTemp3(),
              CelKennisisTemp4(),
              CelKennisisTemp5(),
              )

class CelKennisisPWMs():
    units = (CelKennisisPWM1(),
             )

class CelKennisisSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_status'
    bad_led_color = 'green_blinking'

class CelKennisisLEDs():
    units = (CelKennisisSystemLED(),
             )

class CelKennisis(cumulus.platform.Platform):
    name = 'cel,kennisis'
    chip = cumulus.platform.TriumphChip()
    switch = CelKennisisSwitch(chip)
    cpld = CelKennisisCPLD()
    statusleds = CelKennisisLEDs()
    pwms = CelKennisisPWMs()
    sensors = CelKennisisSensors()

Platform = CelKennisis
