#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class DellS4048TCPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/dell_s4048t_cpld.0'

'''
   Dell S4048T LEDS
     System:
       green - all systems ok
       yellow blinking - fan missing or broken, psu missing or broken
     Fan_Tray_X (0 - 2):
       green - fan X installed and working
       yellow - fan X malfunction
'''

DellS4048T_i2c_path = '/sys/bus/i2c/devices/'

''' tmp75: temp sensor 1 '''
DellS4048T_i2c_11_49_path = DellS4048T_i2c_path + '11-0049'

''' tmp75: temp sensor 2 '''
DellS4048T_i2c_11_4a_path = DellS4048T_i2c_path + '11-004a'

''' tmp75: temp sensor 3 '''
DellS4048T_i2c_11_4b_path = DellS4048T_i2c_path + '11-004b'

''' tmp75: temp sensor 4 '''
DellS4048T_i2c_11_4c_path = DellS4048T_i2c_path + '11-004c'

''' tmp75: temp sensor 5 '''
DellS4048T_i2c_11_4e_path = DellS4048T_i2c_path + '11-004e'

''' tmp75: temp sensor 6 '''
DellS4048T_i2c_11_4f_path = DellS4048T_i2c_path + '11-004f'

''' tmp75: temp sensor 7 '''
DellS4048T_i2c_10_4d_path = DellS4048T_i2c_path + '10-004d'

''' dps460: PSU1 Temp/Fan '''
DellS4048T_i2c_14_58_path = DellS4048T_i2c_path + '14-0058'

''' dps460: PSU2 Temp/Fan '''
DellS4048T_i2c_14_59_path = DellS4048T_i2c_path + '14-0059'

''' max6620: Fan Tray 1, 2 Controller '''
DellS4048T_i2c_11_4d_path = DellS4048T_i2c_path + '11-004d'

''' max6620: Fan Tray 3, 4 Controller '''
DellS4048T_i2c_13_4d_path = DellS4048T_i2c_path + '13-004d'

''' ltc4215: PSU1 '''
DellS4048T_i2c_11_40_path = DellS4048T_i2c_path + '11-0040'

''' ltc4215: PSU2 '''
DellS4048T_i2c_11_42_path = DellS4048T_i2c_path + '11-0042'

class DellS4048TSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_system'
    bad_led_color = 'yellow_blinking'

class DellS4048TPowerLED(cumulus.sensors.StatusLED_Unit):
    name = 'Power'
    sources = ['PSU1', 'PSU2']
    led_location = 'led_power'
    bad_led_color = 'yellow_blinking'

class DellS4048TFanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan'
    sources = ['Fan1', 'Fan2','Fan3', 'Fan4','Fan5', 'Fan6','Fan7', 'Fan8']
    led_location = 'led_fan'
    bad_led_color = 'yellow_blinking'
    
class DellS4048TFan1LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 1'
    sources = ['Fan1', 'Fan5']
    led_location = 'led_fan_tray_1'
    bad_led_color = 'yellow'

class DellS4048TFan2LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 2'
    sources = ['Fan2', 'Fan6']
    led_location = 'led_fan_tray_2'
    bad_led_color = 'yellow'

class DellS4048TFan3LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 3'
    sources = ['Fan3', 'Fan7']
    led_location = 'led_fan_tray_3'
    bad_led_color = 'yellow'

class DellS4048TFan4LED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan Tray 4'
    sources = ['Fan4', 'Fan8']
    led_location = 'led_fan_tray_4'
    bad_led_color = 'yellow'

class DellS4048TTemp1(cumulus.sensors.Temp_Unit):
    # U278, Hardware shutdown, Fan Controlled
    name = 'Temp1'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = DellS4048T_i2c_11_4b_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True
    
class DellS4048TTemp1F2B(DellS4048TTemp1):
    temp_fan_min = 34
    temp_fan_max = 56    
    temp_max = 63
    temp_crit = 67

class DellS4048TTemp1B2F(DellS4048TTemp1):
    temp_fan_min = 39
    temp_fan_max = 61    
    temp_max = 66
    temp_crit = 70

class DellS4048TTemp2(cumulus.sensors.Temp_Unit):
    # U245, Hardware shutdown, Fan Controlled
    name = 'Temp2'
    description = 'Board Sensor near Left PHY'
    driver_path = DellS4048T_i2c_11_4e_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True
            
class DellS4048TTemp2F2B(DellS4048TTemp2):
    temp_fan_min = 46
    temp_fan_max = 65    
    temp_max = 68
    temp_crit = 72
        
class DellS4048TTemp2B2F(DellS4048TTemp2):
    temp_fan_min = 46
    temp_fan_max = 69    
    temp_max = 73
    temp_crit = 77
        
class DellS4048TTemp3(cumulus.sensors.Temp_Unit):
    # U244, Hardware shutdown, Fan Controlled
    name = 'Temp3'
    description = 'Board Sensor near Right PHY'
    driver_path = DellS4048T_i2c_11_4a_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True
        
class DellS4048TTemp3F2B(DellS4048TTemp3):
    temp_fan_min = 46
    temp_fan_max = 65
    temp_max = 68
    temp_crit = 72
    
class DellS4048TTemp3B2F(DellS4048TTemp3):
    temp_fan_min = 43
    temp_fan_max = 66    
    temp_max = 73
    temp_crit = 77
            
class DellS4048TTemp4(cumulus.sensors.Temp_Unit):
    # U254, Hardware shutdown, Fan Controlled
    name = 'Temp4'
    description = 'Board Sensor near Middle PHY'
    driver_path = DellS4048T_i2c_11_4c_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True
        
class DellS4048TTemp4F2B(DellS4048TTemp4):
    temp_fan_min = 46
    temp_fan_max = 65    
    temp_max = 68
    temp_crit = 72

class DellS4048TTemp4B2F(DellS4048TTemp4):
    temp_fan_min = 43
    temp_fan_max = 66    
    temp_max = 73
    temp_crit = 77

class DellS4048TTemp5(cumulus.sensors.Temp_Unit):
    # U288, Hardware shutdown, Fan Controlled
    name = 'Temp5'
    description = 'Board Sensor near QSFP'
    driver_path = DellS4048T_i2c_11_49_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True
        
class DellS4048TTemp5F2B(DellS4048TTemp5):
    temp_fan_min = 32
    temp_fan_max = 54    
    temp_max = 59
    temp_crit = 62

class DellS4048TTemp5B2F(DellS4048TTemp5):
    temp_fan_min = 34
    temp_fan_max = 56    
    temp_max = 59
    temp_crit = 62
    
class DellS4048TTemp6(cumulus.sensors.Temp_Unit):
    # U277, Hardware shutdown, Fan Controlled
    name = 'Temp6'
    description = 'Board Sensor near Fan'
    driver_path = DellS4048T_i2c_11_4f_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True
        
class DellS4048TTemp6F2B(DellS4048TTemp6):
    temp_fan_min = 35
    temp_fan_max = 63    
    temp_max = 69
    temp_crit = 73

class DellS4048TTemp6B2F(DellS4048TTemp6):
    temp_fan_min = 35
    temp_fan_max = 58    
    temp_max = 63
    temp_crit = 67

class DellS4048TTemp7(cumulus.sensors.Temp_Unit):
    # U57, Hardware shutdown, Fan Controlled
    name = 'Temp7'
    description = 'Board Sensor near CPU'
    driver_path = DellS4048T_i2c_10_4d_path
    driver_hwmon = ['temp1']
    temp_otp_hw = True
        
class DellS4048TTemp7F2B(DellS4048TTemp7):
    temp_fan_min = 26
    temp_fan_max = 48    
    temp_max = 60
    temp_crit = 65

class DellS4048TTemp7B2F(DellS4048TTemp7):
    temp_fan_min = 34
    temp_fan_max = 56    
    temp_max = 60
    temp_crit = 65

class DellS4048TCT(cumulus.sensors.Temp_Unit):
    # U16
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_fan_min = 75
    temp_fan_max = 90    
    temp_max = 104
    temp_crit = 125
    
class DellS4048TTemp8(DellS4048TCT):
    name = 'Temp8'
    driver_hwmon = ['temp2']

class DellS4048TTemp9(DellS4048TCT):
    name = 'Temp9'
    driver_hwmon = ['temp3']
        
class DellS4048TPSUTemp(cumulus.sensors.Temp_Unit):
    temp_fan_min = 30
    temp_fan_max = 52
    temp_max = 55
    temp_crit = 65
    psu = None
    temp_otp_hw = True

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state == 'ABSENT'):
            return -1
        return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)

class DellS4048TTemp10(DellS4048TPSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp Sensor'
    driver_path = DellS4048T_i2c_14_59_path
    driver_hwmon = ['temp1']
    psu = 'PSU1'
        
class DellS4048TTemp11(DellS4048TPSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp Sensor'
    driver_path = DellS4048T_i2c_14_58_path
    driver_hwmon = ['temp1']
    psu = 'PSU2'
            
class DellS4048T_T2PlusTemp(cumulus.sensors.Temp_Unit):
    description  = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path  = '/cumulus/switchd/run'
    temp_max     = 99
    temp_crit    = 104
    temp_fan_min = 57
    temp_fan_max = 79

class DellS4048T_T2PlusTemp1(DellS4048T_T2PlusTemp):
    name = 'Temp10'
    driver_hwmon = ['temp1']

class DellS4048T_T2PlusTemp2(DellS4048T_T2PlusTemp):
    name = 'Temp11'
    driver_hwmon = ['temp2']

class DellS4048T_T2PlusTemp3(DellS4048T_T2PlusTemp):
    name = 'Temp12'
    driver_hwmon = ['temp3']

class DellS4048T_T2PlusTemp4(DellS4048T_T2PlusTemp):
    name = 'Temp13'
    driver_hwmon = ['temp4']

class DellS4048T_T2PlusTemp5(DellS4048T_T2PlusTemp):
    name = 'Temp14'
    driver_hwmon = ['temp5']

class DellS4048T_T2PlusTemp6(DellS4048T_T2PlusTemp):
    name = 'Temp15'
    driver_hwmon = ['temp6']

class DellS4048T_T2PlusTemp7(DellS4048T_T2PlusTemp):
    name = 'Temp16'
    driver_hwmon = ['temp7']

class DellS4048T_T2PlusTemp8(DellS4048T_T2PlusTemp):
    name = 'Temp17'
    driver_hwmon = ['temp8']

class DellS4048T_T2PlusTemp9(DellS4048T_T2PlusTemp):
    name = 'Temp18'
    driver_hwmon = ['temp9']

class DellS4048TFan(cumulus.sensors.Fan_Unit):
    fan_max = 16500 # Corresponds to duty 50 % and RPM 7700/8000
    fan_min = 10000
    minpwm = 128
    hwmon_search = False

class DellS4048TPSUFan(cumulus.sensors.Fan_Unit):
    minpwm = 128 # Corresponds to duty 50 % and RPM 11500
    fan_max = 23500
    fan_min = 11500
    psu = None

    def find_and_add_attrs(self):
        state = 'OK'
        try:
            path = os.path.join(self.status_path, self.psu, 'state')
            with open(path) as fp:
                state = fp.read().rsplit('\n')[0]
        except:
            return -1
        if (state == 'ABSENT'):
            return -1
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class DellS4048TFan1(DellS4048TFan):
    name = 'Fan1'
    description = 'Fan Tray 1, Fan 1'
    driver_path = DellS4048T_i2c_11_4d_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    cpld_hwmon = ['fan_1']
    
class DellS4048TFan2(DellS4048TFan):
    name = 'Fan2'
    description = 'Fan Tray 2, Fan 1'
    driver_path = DellS4048T_i2c_11_4d_path
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']
    cpld_hwmon = ['fan_2']
    
class DellS4048TFan3(DellS4048TFan):
    name = 'Fan3'
    description = 'Fan Tray 3, Fan 1'
    driver_path = DellS4048T_i2c_11_4d_path
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']    
    cpld_hwmon = ['fan_3']
    
class DellS4048TFan4(DellS4048TFan):
    name = 'Fan4'
    description = 'Fan Tray 4, Fan 1'
    driver_path = DellS4048T_i2c_11_4d_path
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']
    cpld_hwmon = ['fan_4']
    
class DellS4048TFan5(DellS4048TFan):
    name = 'Fan5'
    description = 'Fan Tray 1, Fan 2'
    fan_min = 7000
    driver_path = DellS4048T_i2c_13_4d_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    cpld_hwmon = ['fan_1']
    
class DellS4048TFan6(DellS4048TFan):
    name = 'Fan6'
    description = 'Fan Tray 2, Fan 2'
    fan_min = 7000
    driver_path = DellS4048T_i2c_13_4d_path
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm2']    
    cpld_hwmon = ['fan_2']
    
class DellS4048TFan7(DellS4048TFan):
    name = 'Fan7'
    description = 'Fan Tray 3, Fan 2'
    fan_min = 7000
    driver_path = DellS4048T_i2c_13_4d_path
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm3']
    cpld_hwmon = ['fan_3']
    
class DellS4048TFan8(DellS4048TFan):
    name = 'Fan8'
    description = 'Fan Tray 4, Fan 2'
    fan_min = 7000
    driver_path = DellS4048T_i2c_13_4d_path
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm4']    
    cpld_hwmon = ['fan_4']
    
class DellS4048TFan9(DellS4048TPSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    driver_path = DellS4048T_i2c_14_59_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    psu = 'PSU1'
        
class DellS4048TFan10(DellS4048TPSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    driver_path = DellS4048T_i2c_14_58_path
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']    
    psu = 'PSU2'
    
class DellS4048TPSU(cumulus.sensors.PSU_Unit):
    init_done = False
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            subprocess.call(["/sbin/rmmod", "dps460"])
            subprocess.call(["/sbin/modprobe", "dps460"])
            self.init_done = True

class DellS4048TPSU1(DellS4048TPSU):
    name = 'PSU1'
    driver_path = DellS4048T_i2c_11_42_path
    driver_hwmon = ['power1', 'in1', 'in2', 'curr1']
    cpld_hwmon = ['psu_pwr1']
    
class DellS4048TPSU2(DellS4048TPSU):
    name = 'PSU2'
    driver_path = DellS4048T_i2c_11_40_path
    driver_hwmon = ['power1', 'in1', 'in2', 'curr1']
    cpld_hwmon = ['psu_pwr2']
    
def dell_s4048t_airflow():
    f2b_part_nums = ['0YVCK0', '0DC9DH', '08D1MG']
    b2f_part_nums = ['0J1P3P', '05JM5X', '0Y1K3R']

    plat = os.popen('platform-detect -m').read().strip()
    if plat != 'dell,s4048t_c2338':
       return None

    try:
       b_info = subprocess.check_output('/usr/cumulus/bin/decode-syseeprom')
    except:
       b_info = ''
    # Part Number Format
    # TLV Name             Code Len Value
    # -------------------- ---- --- -----
    # Part Number          0x22   6 0J09D3    
    part_num = ''
    for l in b_info.split('\n'):
       if 'Part Number' in l:
           part_num = l.split()[4]
           break
    if part_num in b2f_part_nums:
       return 'B2F'        
    if part_num in f2b_part_nums:
       return 'F2B'        
    return None

class DellS4048TPWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (DellS4048TFan1(),
            DellS4048TFan2(),
            DellS4048TFan3(),
            DellS4048TFan4(),
            DellS4048TFan5(),
            DellS4048TFan6(),
            DellS4048TFan7(),
            DellS4048TFan8(),
            DellS4048TFan9(),
            DellS4048TFan10(),
    )

    temps = (DellS4048TTemp8(),
             DellS4048TTemp9(),
             DellS4048TTemp10(),
             DellS4048TTemp11(),
             DellS4048T_T2PlusTemp1(),
             DellS4048T_T2PlusTemp2(),
             DellS4048T_T2PlusTemp3(),
             DellS4048T_T2PlusTemp4(),
             DellS4048T_T2PlusTemp5(),
             DellS4048T_T2PlusTemp6(),
             DellS4048T_T2PlusTemp7(),
             DellS4048T_T2PlusTemp8(),
             DellS4048T_T2PlusTemp9(),
             )
    f2b_temps = (DellS4048TTemp1F2B(),
                 DellS4048TTemp2F2B(),
                 DellS4048TTemp3F2B(),
                 DellS4048TTemp4F2B(),
                 DellS4048TTemp5F2B(),
                 DellS4048TTemp6F2B(),
                 DellS4048TTemp7F2B(),
                )
    b2f_temps = (DellS4048TTemp1B2F(),
                 DellS4048TTemp2B2F(),
                 DellS4048TTemp3B2F(),
                 DellS4048TTemp4B2F(),
                 DellS4048TTemp5B2F(),
                 DellS4048TTemp6B2F(),
                 DellS4048TTemp7B2F(),
                )
    # Default to B2F if airflow cant be figured out.
    if dell_s4048t_airflow() == 'F2B':
        temps = temps + f2b_temps
    else:
        temps = temps + b2f_temps
        
class DellS4048TSensors():
    units = (DellS4048TPSU1(),
             DellS4048TPSU2(),
             DellS4048TTemp8(),
             DellS4048TTemp9(),
             DellS4048TTemp10(),
             DellS4048TTemp11(),
             DellS4048TFan1(),
             DellS4048TFan2(),
             DellS4048TFan3(),
             DellS4048TFan4(),
             DellS4048TFan5(),
             DellS4048TFan6(),
             DellS4048TFan7(),
             DellS4048TFan8(),
             DellS4048TFan9(),
             DellS4048TFan10(),
             DellS4048T_T2PlusTemp1(),
             DellS4048T_T2PlusTemp2(),
             DellS4048T_T2PlusTemp3(),
             DellS4048T_T2PlusTemp4(),
             DellS4048T_T2PlusTemp5(),
             DellS4048T_T2PlusTemp6(),
             DellS4048T_T2PlusTemp7(),
             DellS4048T_T2PlusTemp8(),
             DellS4048T_T2PlusTemp9(),
    )

    f2b_temps = (DellS4048TTemp1F2B(),
                 DellS4048TTemp2F2B(),
                 DellS4048TTemp3F2B(),
                 DellS4048TTemp4F2B(),
                 DellS4048TTemp5F2B(),
                 DellS4048TTemp6F2B(),
                 DellS4048TTemp7F2B(),
                )
    b2f_temps = (DellS4048TTemp1B2F(),
                 DellS4048TTemp2B2F(),
                 DellS4048TTemp3B2F(),
                 DellS4048TTemp4B2F(),
                 DellS4048TTemp5B2F(),
                 DellS4048TTemp6B2F(),
                 DellS4048TTemp7B2F(),
                )
    # Default to B2F if airflow cant be figured out.
    if dell_s4048t_airflow() == 'F2B':
        units = units + f2b_temps
    else:
        units = units + b2f_temps
        
class DellS4048TPWMs():
    units = (DellS4048TPWM1(),
             )

class DellS4048TLEDs():
    units = (
             DellS4048TSystemLED(),
             DellS4048TPowerLED(),
             DellS4048TFanLED(),
             DellS4048TFan1LED(),
             DellS4048TFan2LED(),
             DellS4048TFan3LED(),
             DellS4048TFan4LED(),             
        )

class DellS4048T10GT(cumulus.platform.TridentTwoPlus10GPort):
    _set_led = '''
;;;
;;; Each 10GT port has one LED for link status and activity.
;;; The link LED has 3 colors, green, amber and blue.
;;; We are using one color: Green, for link status and activity.
;;;
;;; 3 bits per port. Set bit as 1 for color.
;;; 0 -> Green
;;; 1 -> Amber
;;; 2 -> Blue
;;;

set_led:
        port    a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        ;tinv
        pack

        pushst PORTSTATUS_ZERO
        pack

        pushst PORTSTATUS_ZERO
        pack
                
        ret
'''
    def __init__(self, label, serdes, phy_addrs, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, leds):
        cumulus.platform.TridentTwoPlus10GPort.__init__(self,
                                                    label=label,
                                                    serdes=serdes,
                                                    phy_types=("84858",),
                                                    phy_addrs=phy_addrs,
                                                    phy_layer='10GBaseT',
                                                    tx_lane_map=tx_lane_map,
                                                    rx_lane_map=rx_lane_map,
                                                    tx_polarity_flip=tx_polarity_flip,
                                                    rx_polarity_flip=rx_polarity_flip,
                                                    leds=leds,
                                                    unit=0)

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, led_port)
        for led in led_index_list:
            set_calls[led] = call
            set_calls[led+1] = None
            set_calls[led+2] = None

        return set_calls

class DellS4048TQSFPp(cumulus.platform.TridentTwoPlus40GPort):
    _set_led = '''
;;; DellS4048TQSFPp
;;;
;;; 40G ports have 4 green/yellow LEDs, two bits per LED.
;;; For 40G mode, we use only the first LED to indicate link/activity.
;;;    0: link_green   (active high)
;;;    1-7: off
;;;  We use the link_green led for link and activity
;;; The other 3 LEDs will be off.
;;;
set_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor                     ; on with link
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ld      a, 7
qsfp_pack_loop:
        port 1
        pushst  PORTSTATUS_ZERO
        pack
        sub     a, 1
        jnz     qsfp_pack_loop

        ret
'''
    _set_led_4x10 = '''
;;; DellS4048TQSFPp_4x10
;;;
;;;  For 4x10G mode, we use an LED for each port. Each port has
;;;  a single green/yellow LED:
;;;    0: link_green   (active high)
;;;    1: link_yellow  (active high)
;;;  We use the link_green led for link and activity
;;;  link_yellow is always off
;;
set_led:
        call    set_qsfp_4x10_led
        inc     a
        call    set_qsfp_4x10_led
        inc     a
        call    set_qsfp_4x10_led
        inc     a
        call    set_qsfp_4x10_led

        ret

set_qsfp_4x10_led:
        port a

        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor                     ; link_green on with link
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        pushst  PORTSTATUS_ZERO ; link_yellow off
        pack

        ret
'''
    def __init__(self, label, serdes, tx_lane_map, rx_lane_map,
                 tx_polarity_flip, rx_polarity_flip, leds):
        cumulus.platform.TridentTwoPlus40GPort.__init__(self,
                                                    label=label,
                                                    serdes=serdes,
                                                    phy_addrs = None,
                                                    phy_types = None,
                                                    tx_lane_map=tx_lane_map,
                                                    rx_lane_map=rx_lane_map,
                                                    tx_polarity_flip=tx_polarity_flip,
                                                    rx_polarity_flip=rx_polarity_flip,
                                                    leds=leds,
                                                    unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode in ('4x1G', '4x10G')) and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        if mode in ('4x1G', '4x10G'):
            call = (fn_name + "_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0]+1] = None
        set_calls[led_index_list[0]+2] = None
        set_calls[led_index_list[0]+3] = None
        set_calls[led_index_list[0]+4] = None
        set_calls[led_index_list[0]+5] = None
        set_calls[led_index_list[0]+6] = None
        set_calls[led_index_list[0]+7] = None
        return set_calls

class DellS4048TLED_1_36(cumulus.ledup.TridentTwoPlusLEDController):
    default_pad = 0
    num_leds = 112

class DellS4048TLED_37_54(cumulus.ledup.TridentTwoPlusLEDController):
    default_pad = 0
    num_leds = 84

class DellS4048TSwitch(cumulus.platform.BCMSwitch):
    ledup = (DellS4048TLED_1_36, DellS4048TLED_37_54)
    _10GT = DellS4048T10GT
    _QSFP = DellS4048TQSFPp
    has_phy_ucode_spi_rom = False
    has_phy_ucode_mdio = True

    ports = (
        _10GT(label="1",  serdes=(13,), phy_addrs=(0x01,), tx_lane_map=0x0132,
              rx_lane_map=0x3201, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(0, 0),),
        _10GT(label="2",  serdes=(12,), phy_addrs=(0x00,), tx_lane_map=0x0132,
              rx_lane_map=0x3201, tx_polarity_flip=True, rx_polarity_flip=False,
              leds=(0, 3),),
        _10GT(label="3",  serdes=(15,), phy_addrs=(0x03,), tx_lane_map=0x0132,
              rx_lane_map=0x3201, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(0, 6),),
        _10GT(label="4",  serdes=(14,), phy_addrs=(0x02,), tx_lane_map=0x0132,
              rx_lane_map=0x3201, tx_polarity_flip=True, rx_polarity_flip=True,
              leds=(0, 9),),
        _10GT(label="5",  serdes=(21,), phy_addrs=(0x05,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(0, 12),),
        _10GT(label="6",  serdes=(20,), phy_addrs=(0x04,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,
              leds=(0, 15),),
        _10GT(label="7",  serdes=(23,), phy_addrs=(0x07,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=True,
              leds=(0, 18),),
        _10GT(label="8",  serdes=(22,), phy_addrs=(0x06,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=True,
              leds=(0, 21),),
        _10GT(label="9",  serdes=(25,), phy_addrs=(0x09,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(0, 24),),
        _10GT(label="10",  serdes=(24,), phy_addrs=(0x08,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,
              leds=(0, 27),),
        _10GT(label="11",  serdes=(27,), phy_addrs=(0x0b,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(0, 30),),
        _10GT(label="12",  serdes=(26,), phy_addrs=(0x0a,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=True,
              leds=(0, 33),),
        _10GT(label="13",  serdes=(29,), phy_addrs=(0x2d,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(0, 36),),
        _10GT(label="14",  serdes=(28,), phy_addrs=(0x2c,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,
              leds=(0, 39),),
        _10GT(label="15",  serdes=(31,), phy_addrs=(0x2f,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(0, 42),),
        _10GT(label="16",  serdes=(30,), phy_addrs=(0x2e,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=True,
              leds=(0, 45),),
        _10GT(label="17",  serdes=(45,), phy_addrs=(0x31,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(0, 48),),
        _10GT(label="18",  serdes=(44,), phy_addrs=(0x30,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,
              leds=(0, 51),),
        _10GT(label="19",  serdes=(47,), phy_addrs=(0x33,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(0, 54),),
        _10GT(label="20",  serdes=(46,), phy_addrs=(0x32,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=True,
              leds=(0, 57),),
        _10GT(label="21",  serdes=(49,), phy_addrs=(0x35,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(0, 60),),
        _10GT(label="22",  serdes=(48,), phy_addrs=(0x34,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,
              leds=(0, 63),),
        _10GT(label="23",  serdes=(51,), phy_addrs=(0x37,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(0, 66),),
        _10GT(label="24",  serdes=(50,), phy_addrs=(0x36,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=True,
              leds=(0, 69),),
        _10GT(label="25",  serdes=(53,), phy_addrs=(0x41,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(0, 72),),
        _10GT(label="26",  serdes=(52,), phy_addrs=(0x40,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,
              leds=(0, 75),),
        _10GT(label="27",  serdes=(55,), phy_addrs=(0x43,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(0, 78),),
        _10GT(label="28",  serdes=(54,), phy_addrs=(0x42,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=True,
              leds=(0, 81),),
        _10GT(label="29",  serdes=(57,), phy_addrs=(0x45,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(0, 84),),
        _10GT(label="30",  serdes=(56,), phy_addrs=(0x44,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,
              leds=(0, 87),),
        _10GT(label="31",  serdes=(59,), phy_addrs=(0x47,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(0, 90),),
        _10GT(label="32",  serdes=(58,), phy_addrs=(0x46,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=True,
              leds=(0, 93),),
        _10GT(label="33",  serdes=(61,), phy_addrs=(0x49,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(0, 96),),
        _10GT(label="34",  serdes=(60,), phy_addrs=(0x48,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,
              leds=(0, 99),),
        _10GT(label="35",  serdes=(63,), phy_addrs=(0x4b,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(0, 102),),
        _10GT(label="36",  serdes=(62,), phy_addrs=(0x4a,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=True,
              leds=(0, 105),),
        _10GT(label="37",  serdes=(65,), phy_addrs=(0x6d,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(1, 0),),
        _10GT(label="38",  serdes=(64,), phy_addrs=(0x6c,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,
              leds=(1, 3),),
        _10GT(label="39",  serdes=(67,), phy_addrs=(0x6f,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(1, 6),),
        _10GT(label="40",  serdes=(66,), phy_addrs=(0x6e,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=True,
              leds=(1, 9),),
        _10GT(label="41",  serdes=(69,), phy_addrs=(0x71,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(1, 12),),
        _10GT(label="42",  serdes=(68,), phy_addrs=(0x70,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,
              leds=(1, 15),),
        _10GT(label="43",  serdes=(71,), phy_addrs=(0x73,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(1, 18),),
        _10GT(label="44",  serdes=(70,), phy_addrs=(0x72,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=True,
              leds=(1, 21),),
        _10GT(label="45",  serdes=(73,), phy_addrs=(0x75,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(1, 24),),
        _10GT(label="46",  serdes=(72,), phy_addrs=(0x74,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=False,
              leds=(1, 27),),
        _10GT(label="47",  serdes=(75,), phy_addrs=(0x77,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=False, rx_polarity_flip=False,
              leds=(1, 30),),
        _10GT(label="48",  serdes=(74,), phy_addrs=(0x76,), tx_lane_map=0x0123,
              rx_lane_map=0x3210, tx_polarity_flip=True, rx_polarity_flip=True,
              leds=(1, 33),),
        _QSFP(label="49",  serdes=(100, 101, 102, 103), tx_lane_map=0x2031,
              rx_lane_map=0x0132, tx_polarity_flip=0x2, rx_polarity_flip=0x5,
              leds=(1, 36),),
        _QSFP(label="50",  serdes=(96, 97, 98, 99), tx_lane_map=0x3021,
              rx_lane_map=0x1320, tx_polarity_flip=0xb, rx_polarity_flip=0xd,
              leds=(1, 44),),
        _QSFP(label="51",  serdes=(108, 109, 110, 111), tx_lane_map=0x0213,
              rx_lane_map=0x0213, tx_polarity_flip=0x8, rx_polarity_flip=0x5,
              leds=(1, 52),),
        _QSFP(label="52",  serdes=(104, 105, 106, 107), tx_lane_map=0x2031,
              rx_lane_map=0x1320, tx_polarity_flip=0x3, rx_polarity_flip=0x9,
              leds=(1, 60),),
        _QSFP(label="53",  serdes=(80, 81, 82, 83), tx_lane_map=0x0132,
              rx_lane_map=0x0132, tx_polarity_flip=False, rx_polarity_flip=0xb,
              leds=(1, 68),),
        _QSFP(label="54",  serdes=(76, 77, 78, 79), tx_lane_map=0x2031,
              rx_lane_map=0x1320, tx_polarity_flip=0x3, rx_polarity_flip=0xd,
              leds=(1, 76),),
        )

        
class DellS4048T(cumulus.platform.Platform):
    name = 'dell,s4048t_c2338'
    chip = cumulus.platform.TridentTwoPlus_56864_Chip()
    switch = DellS4048TSwitch(chip)
    cpld = DellS4048TCPLD()
    pwms = DellS4048TPWMs()
    sensors = DellS4048TSensors()
    statusleds = DellS4048TLEDs()    

Platform = DellS4048T
