#
# Copyright 2016.  Cumulus Networks, Inc.
#

import os
import cumulus.platforms.mlnx_SN2xxx
import cumulus.platforms.mlnx_x86_MSN2100


class MlxSN2100BQSFP(cumulus.platform.Spectrum40GPort):
    _set_led = '''
;;; MlxSN2100BQSFP
;;;  port LEDs controlled by HW
'''
    def __init__(self, label, phy_addrs, hw_ports,
                 blocked_adj_split=None):

        hw_port_names = []
        for hw_port in hw_ports:
            hw_port_names.append('xe%d' % hw_port)

        cumulus.platform.Spectrum40GPort.__init__(self, label=label,
                        adj_label=None, hw_ports=tuple(hw_port_names),
                        phy_addrs=phy_addrs, blocked_adj_split=blocked_adj_split)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_ports[0], 'qsfp')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls

class MlxSN2100BSwitch(cumulus.platform.SpectrumSwitch):
    _sfp = None
    _qsfp = MlxSN2100BQSFP

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    # label is front panel port label.
    # adj_label is front panel port label of the adjacent port (because the
    #    adjacent label is not necessarily label+1). Note that adj_label
    #    is meaningful only when blocked_adj_split is specified.
    #
    # phy_addrs correspond to XML lane-to-module.
    # hw_ports correspond to XML local-port; the length of the tuple
    #    (when > 1) implies the max number of ports into which this port can be split.
    # blocked_adj_split Port is blocked when its adjacent port is split into
    #    this number of ports (or more)
    #
    ports = (
        _qsfp(label="1",  phy_addrs=(0,0,0,0),     hw_ports=(25,26,27,28)),
        _qsfp(label="2",  phy_addrs=(1,1,1,1),     hw_ports=(29,30,31,32)),
        _qsfp(label="3",  phy_addrs=(2,2,2,2),     hw_ports=(17,18,19,20)),
        _qsfp(label="4",  phy_addrs=(3,3,3,3),     hw_ports=(21,22,23,24)),
        _qsfp(label="5",  phy_addrs=(4,4,4,4),     hw_ports=( 9,10,11,12)),
        _qsfp(label="6",  phy_addrs=(5,5,5,5),     hw_ports=(13,14,15,16)),
        _qsfp(label="7",  phy_addrs=(6,6,6,6),     hw_ports=(1,2,3,4)),
        _qsfp(label="8",  phy_addrs=(7,7,7,7),     hw_ports=(5,6,7,8)),
        _qsfp(label="9",  phy_addrs=(8,8,8,8),     hw_ports=(37,38,39,40)),
        _qsfp(label="10", phy_addrs=(9,9,9,9),     hw_ports=(33,34,35,36)),
        _qsfp(label="11", phy_addrs=(10,10,10,10), hw_ports=(45,46,47,48)),
        _qsfp(label="12", phy_addrs=(11,11,11,11), hw_ports=(41,42,43,44)),
        _qsfp(label="13", phy_addrs=(12,12,12,12), hw_ports=(53,54,55,56)),
        _qsfp(label="14", phy_addrs=(13,13,13,13), hw_ports=(49,50,51,52)),
        _qsfp(label="15", phy_addrs=(14,14,14,14), hw_ports=(61,62,63,64)),
        _qsfp(label="16", phy_addrs=(15,15,15,15), hw_ports=(57,58,59,60)),
    )

class MlxSN2100B(cumulus.platform.Platform):
    name = 'mlnx,x86_MSN2100B'
    chip = cumulus.platform.SpectrumChip()
    switch = MlxSN2100BSwitch(cumulus.platform.SpectrumChip())
    cpld = cumulus.platforms.mlnx_SN2xxx.Mlx2xxxCPLD()
    sensors = cumulus.platforms.mlnx_x86_MSN2100.MlxSN2100Sensors()
    pwms = cumulus.platforms.mlnx_x86_MSN2100.MlxSN2100PWMs()
    statusleds = cumulus.platforms.mlnx_x86_MSN2100.MlxSN2100LEDs()

Platform = MlxSN2100B
