#
# Copyright 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class MlxSX1710SFPp(MlxSX1400SFPp):
    pass

class MlxSX1710QSFPp(MlxSX1400QSFPp):
    pass

class MlxSX1710LED(cumulus.ledup.LEDController):
    pass

class MlxSX1710Switch(cumulus.platform.SwitchXTwoSwitch):
    ledup = MlxSX1710LED
    _sfp = MlxSX1710SFPp
    _qsfp = MlxSX1710QSFPp

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    # label is front panel port label.
    # adj_label is front panel port label of the adjacent port (because the
    #    adjacent label is not necessarily label+1). Note that adj_label
    #    is meaningful only when blocked_adj_split is specified.
    #
    # phy_addrs correspond to XML lane-to-module.
    # hw_ports correspond to XML local-port; the length of the tuple
    #    (when > 1) implies the max number of ports into which this port can be split.
    # blocked_adj_split Port is blocked when its adjacent port is split into
    #    this number of ports (or more)
    #
    ports = (
        _qsfp(label="1",  adj_label="2",  phy_addrs=(0,0,0,0),     hw_ports=(50,), blocked_adj_split=4),
        _qsfp(label="2",  adj_label=None, phy_addrs=(1,1,1,1),     hw_ports=(48,49,50,51)),
        _qsfp(label="3",  adj_label=None, phy_addrs=(2,2,2,2),     hw_ports=(52,)),
        _qsfp(label="4",  adj_label=None, phy_addrs=(3,3,3,3),     hw_ports=(57,58)),
        _qsfp(label="5",  adj_label="6",  phy_addrs=(4,4,4,4),     hw_ports=(55,), blocked_adj_split=4),
        _qsfp(label="6",  adj_label=None, phy_addrs=(5,5,5,5),     hw_ports=(53,54,55,56)),
        _qsfp(label="7",  adj_label="8",  phy_addrs=(6,6,6,6),     hw_ports=(61,), blocked_adj_split=4),
        _qsfp(label="8",  adj_label=None, phy_addrs=(7,7,7,7),     hw_ports=(59,60,61,62)),
        _qsfp(label="9",  adj_label=None, phy_addrs=(8,8,8,8),     hw_ports=(47,)),
        _qsfp(label="10", adj_label=None, phy_addrs=(9,9,9,9),     hw_ports=(63,64)),
        _qsfp(label="11", adj_label="12", phy_addrs=(10,10,10,10), hw_ports=(45,), blocked_adj_split=4),
        _qsfp(label="12", adj_label=None, phy_addrs=(11,11,11,11), hw_ports=(43,44,45,46)),
        _qsfp(label="13", adj_label="14", phy_addrs=(12,12,12,12), hw_ports=(40,), blocked_adj_split=4),
        _qsfp(label="14", adj_label=None, phy_addrs=(13,13,13,13), hw_ports=(38,39,40,41)),
        _qsfp(label="15", adj_label=None, phy_addrs=(14,14,14,14), hw_ports=(37,)),
        _qsfp(label="16", adj_label=None, phy_addrs=(15,15,15,15), hw_ports=(42,)),
        _qsfp(label="17", adj_label="18", phy_addrs=(16,16,16,16), hw_ports=(35,), blocked_adj_split=4),
        _qsfp(label="18", adj_label=None, phy_addrs=(17,17,17,17), hw_ports=(33,34,35,36)),
        _qsfp(label="19", adj_label="20", phy_addrs=(18,18,18,18), hw_ports=(3,), blocked_adj_split=4),
        _qsfp(label="20", adj_label=None, phy_addrs=(19,19,19,19), hw_ports=(1,2,3,4)),

        _qsfp(label="21", adj_label=None, phy_addrs=(20,20,20,20), hw_ports=(5,)),
        _qsfp(label="22", adj_label=None, phy_addrs=(21,21,21,21), hw_ports=(10,)),
        _qsfp(label="23", adj_label="24", phy_addrs=(22,22,22,22), hw_ports=(8,), blocked_adj_split=4),
        _qsfp(label="24", adj_label=None, phy_addrs=(23,23,23,23), hw_ports=(6,7,8,9)),
        _qsfp(label="25", adj_label="26", phy_addrs=(24,24,24,24), hw_ports=(13,), blocked_adj_split=4),
        _qsfp(label="26", adj_label=None, phy_addrs=(25,25,25,25), hw_ports=(11,12,13,14)),
        _qsfp(label="27", adj_label=None, phy_addrs=(26,26,26,26), hw_ports=(15,)),
        _qsfp(label="28", adj_label=None, phy_addrs=(27,27,27,27), hw_ports=(31,32)),
        _qsfp(label="29", adj_label="30", phy_addrs=(28,28,28,28), hw_ports=(29,), blocked_adj_split=4),
        _qsfp(label="30", adj_label=None, phy_addrs=(29,29,29,29), hw_ports=(27,28,29,30)),
        _qsfp(label="31", adj_label="32", phy_addrs=(30,30,30,30), hw_ports=(23,), blocked_adj_split=4),
        _qsfp(label="32", adj_label=None, phy_addrs=(31,31,31,31), hw_ports=(21,22,23,24)),
        _qsfp(label="33", adj_label=None, phy_addrs=(32,32,32,32), hw_ports=(20,)),
        _qsfp(label="34", adj_label=None, phy_addrs=(33,33,33,33), hw_ports=(25,26)),
        _qsfp(label="35", adj_label="36", phy_addrs=(34,34,34,34), hw_ports=(18,), blocked_adj_split=4),
        _qsfp(label="36", adj_label=None, phy_addrs=(35,35,35,35), hw_ports=(16,17,18,19)),
    )

class MlxSX1710CPLD(cumulus.cpld.SysFS):
    path = None

# Base class for power supply unit in MSX1710
class MlxSX1710PSU(cumulus.sunit.PSU):
    """
        Led is controlled by HW and there is no way to
        read the LED color
    """
    _state_ledcolor_map = {}
    led_location = None

    def _calculate_state(self, platform, expected_unit_count):
        """
        BAD - PSU is not present/working
        OK  - Running Ok
        """
        state = self.default_state
        for power_source in self.sources:
            status =  self.get_cpld_status(platform, power_source)
            if status != '1':
                state = 'BAD'
                break

        return state

class MlxSX1710PSU_1(MlxSX1710PSU):
    sources = ('psu_pwr1_all_ok',)

    def __init__(self):
        cumulus.sunit.PSU.__init__(self, 1)

# System Fan
class MlxSX1710Fan(cumulus.sunit.Fan):
    _state_ledcolor_map = {}
    led_location = None

    def _calculate_state(self, platform, expected_unit_count):
        """
        BAD - One of FAN is FAN_NOT_SPINNING or NOT_INSTALLED
        OK  - Two Fans are running OK
       """
        state = self.default_state
        for fan_source in self.sources:
            status =  self.get_cpld_status(platform, fan_source)
            if status != '1':
                state = 'BAD'
                break

        return state

class MlxSX1710Fan_1(MlxSX1710Fan):
   sources = ('fan1_all_ok',)
   operational_status_file = "fan1"

   def __init__(self):
        cumulus.sunit.Fan.__init__(self, "fan_1")

# Entire System status
class MlxSX1710System(cumulus.sunit.System):
    sources = ('psu_power', 'fan1')
    led_location = "led_status"
    _state_ledcolor_map = { 'ERROR' : 'green_blinking',
                            'OK' : 'green' }

# System LEDs present in switch
class MlxSX1710SysLED():
    sunits = (
            MlxSX1710PSU_1(),
            MlxSX1710Fan_1(),
            MlxSX1710System(),
        )

class MlxSX1710(cumulus.platform.Platform):
    name = 'mlnx,x86_MSX1710'
    chip = cumulus.platform.SwitchXTwoChip()
    switch = MlxSX1710Switch(chip)
    cpld = MlxSX1710CPLD()
    sysled = MlxSX1710SysLED()

Platform = MlxSX1710
