#!/bin/bash
#
# purge-old-kernels.sh
#

. /usr/share/cumulus/img/functions

cl_check_env root-priv

uninstall_rdepends()
{
	local pkgname=$1

	for p in `apt-cache rdepends $pkgname 2>/dev/null | sed 1,2d`
	do
		echo "uninstalling reverse dependency $p"
		dpkg -r $p
	done
}

get_dpkg_arch()
{
	local arch=$1

	case $arch in
	  x86_64)
	    echo "amd64"
	    ;;
	  ppc)
	    echo "powerpc"
	    ;;
	  arm*)
	    echo "arm"
	    ;;
	  *)
	    echo "$arch"
	esac
}	

running_kernel_version=$(uname -r)

echo "running_kernel_version: $running_kernel_version"

# If the current running kernel does not corresponds to the kernel
# slots default installed kernel, it really means the system has not
# gone through a reboot since the kernel upgrade
arch=$(uname -m 2>/dev/null)
if [ -n "$arch" ]; then
	arch=$(get_dpkg_arch $arch)
fi

if [ -n "$arch" -a "$arch" = "amd64" ]; then
	currslot=${slot_prefix}${active}
	if [ ! -e "/boot/cl-vmlinuz-${running_kernel_version}-$currslot" ]; then
		echo "running kernel is not the default kernel for the current slot, skip purging"
		exit 0
	fi
fi

echo "Getting installed kernel packages ..."
installed_kernel_pkgs=($(dpkg -l 2>/dev/null | grep linux-image | tr -s " " | cut -d " " -f 2,3 --output-delimiter=":"))

echo "Installed packages: "
for pkg in "${installed_kernel_pkgs[@]}"; do
	echo "    $pkg"
done

echo
purged_pkg_cnt=0
for pkg in ${installed_kernel_pkgs[@]}; do
	pkgname=$(echo $pkg | cut -d ":" -f 1)
	version=$(echo $pkg | cut -d ":" -f 2)

	# skip kernel meta package
	if [ $pkgname = "linux-image-$arch" ]; then
		echo "$pkg: skipping because it is a meta package"
		continue
	elif [ "$version" = "$running_kernel_version" ]; then
		# skip running kernel package
		echo "$pkg: skipping because it is the running kernel package"
		continue
	fi

	dpkg --compare-versions "$version" lt "$running_kernel_version" 2>/dev/null
	if [ $? -eq 0 ]; then
		echo "$pkg: uninstalling ..."
		uninstall_rdepends $pkgname
		dpkg -r "$pkgname"
		let purged_pkg_cnt=$purged_pkg_cnt+1

	else
		echo "$pkg: skipping because it is newer than the running kernel"
	fi
done

if [ $purged_pkg_cnt -eq 0 ]; then
	echo "No packages were purged"
fi

